@@ -12,7 +12,8 @@
 //
 // Description : Simple singleton implementation of log
 //============================================================================
-#include <assert.h>
+#include <stdexcept>
+#include <cassert>
 #include <vector>
 #include <iostream>
 
@@ -30,12 +31,12 @@ namespace fs = boost::filesystem;
 
 namespace ecf {
 
-Log* Log::instance_ = NULL;
+Log* Log::instance_ = nullptr;
 bool LogToCout::flag_ = false;
 
 void Log::create(const std::string& filename)
 {
-	if ( instance_ == NULL) {
+	if ( instance_ == nullptr) {
 		instance_ = new Log(filename);
 	}
 }
@@ -45,36 +46,33 @@ void Log::destroy()
 	if (instance_) instance_->flush();
 
 	delete instance_;
-	instance_ = NULL;
+	instance_ = nullptr;
 }
 
 Log::Log(const std::string& fileName)
-: fileName_(fileName), logImpl_( new LogImpl(fileName) )
-{
-}
+: logImpl_( std::make_unique<LogImpl>(fileName) ), fileName_(fileName)
+{}
 
 void Log::create_logimpl()
 {
    if (!logImpl_) {
-      logImpl_ = new LogImpl(fileName_);
+      logImpl_ = std::make_unique<LogImpl>(fileName_);
    }
 }
 
-Log::~Log()
-{
-	delete logImpl_;
-	logImpl_ = NULL;
-}
-
 bool Log::log(Log::LogType lt,const std::string& message)
 {
    create_logimpl();
 
+//   if (!logImpl_->log_open_error().empty()) {
+//      cerr << "Log::log: " << message << "\n";
+//   }
+
 	if (! logImpl_->log(lt,message)) {
 	   // handle write failure and Get the failure reason. This will delete logImpl_ & recreate
-      std::string fail_message = handle_write_failure();
+	   log_error_ = handle_write_failure();
 
-      (void)logImpl_->log(Log::ERR,fail_message); // must be after handle_write_failure
+      (void)logImpl_->log(Log::ERR,log_error_);
 	   (void)logImpl_->log(lt,message);
 	   return false;
 	}
@@ -85,11 +83,15 @@ bool Log::log_no_newline(Log::LogType lt,const std::string& message)
 {
    create_logimpl();
 
+//   if (!logImpl_->log_open_error().empty()) {
+//      cerr << "Log::log_no_newline : " << message << "\n";
+//   }
+
    if (! logImpl_->log_no_newline(lt,message)) {
       // handle write failure and Get the failure reason. This will delete logImpl_ & recreate
-      std::string fail_message = handle_write_failure();
+      log_error_ = handle_write_failure();
 
-      (void)logImpl_->log(Log::ERR,fail_message);  // must be after handle_write_failure
+      (void)logImpl_->log(Log::ERR,log_error_);
       (void)logImpl_->log_no_newline(lt,message);
       return false;
    }
@@ -100,11 +102,15 @@ bool Log::append(const std::string& message)
 {
    create_logimpl();
 
+//   if (!logImpl_->log_open_error().empty()) {
+//      cerr << "Log::append : " << message << "\n";
+//   }
+
    if (! logImpl_->append(message)) {
       // handle write failure and Get the failure reason. This will delete logImpl_ & recreate
-      std::string fail_message = handle_write_failure();
+      log_error_ = handle_write_failure();
 
-      (void)logImpl_->log(Log::ERR,fail_message);  // must be after handle_write_failure
+      (void)logImpl_->log(Log::ERR,log_error_);
       (void)logImpl_->append(message);
       return false;
    }
@@ -119,18 +125,14 @@ void Log::cache_time_stamp()
 
 const std::string& Log::get_cached_time_stamp() const
 {
-   if (!logImpl_)  {
-      return Str::EMPTY();
-   }
-   return logImpl_->get_cached_time_stamp();
+   return (logImpl_) ? logImpl_->get_cached_time_stamp() : Str::EMPTY();
 }
 
 void Log::flush()
 {
  	// will close ofstream and force data to be written to disk.
 	// Forcing writing to physical medium can't be guaranteed though!
-	delete logImpl_;
-	logImpl_ = NULL;
+   logImpl_.reset();
 }
 
 void Log::flush_only()
@@ -217,16 +219,20 @@ std::string Log::contents(int get_last_n_lines)
 
 std::string Log::handle_write_failure()
 {
-   std::string msg = "Failed to write to log file: ";
-   msg += File::stream_error_condition(logImpl_->stream());
-   msg += " : Attempting to close/reopen log file";
-
-   if (LogToCout::ok()) Indentor::indent(cout) << msg << '\n';
+   std::string msg = logImpl_->log_open_error();
+   if (msg.empty()) {
+      msg += "\nFailed to write to log file: ";
+      msg += File::stream_error_condition(logImpl_->stream());
+   }
 
    // handle write failure, by closing then re-opening log file
-   delete logImpl_; logImpl_ =  NULL;
+   logImpl_.reset();
    create_logimpl();
 
+   if (logImpl_->log_open_error().empty())  msg += "\nAttempting to close/reopen log file.";
+   else                                     msg += "\nAttempting to close/reopen log file did not work!";
+
+   if (LogToCout::ok()) Indentor::indent(cout) <<  msg << '\n';
    return msg;
 }
 
@@ -234,6 +240,11 @@ std::string Log::handle_write_failure()
 
 bool log(Log::LogType lt,const std::string& message)
 {
+// For debug of simulator enable this
+//	if (LogToCout::ok()) {
+//		Indentor::indent(cout) << message << '\n';
+//	}
+
 	if (Log::instance()) {
  		return Log::instance()->log(lt,message);
 	}
@@ -287,12 +298,12 @@ void log_assert(char const* expr,char const* file, long line, const std::string&
 void Log::get_log_types(std::vector<std::string>& vec)
 {
    vec.reserve(6);
-   vec.push_back("MSG");
-   vec.push_back("LOG");
-   vec.push_back("ERR");
-   vec.push_back("WAR");
-   vec.push_back("DBG");
-   vec.push_back("OTH");
+   vec.emplace_back("MSG");
+   vec.emplace_back("LOG");
+   vec.emplace_back("ERR");
+   vec.emplace_back("WAR");
+   vec.emplace_back("DBG");
+   vec.emplace_back("OTH");
 }
 
 //======================================================================================================
@@ -306,23 +317,49 @@ LogFlusher::~LogFlusher()
 }
 
 
+//======================================================================================================
+
+TestLog::TestLog(const std::string& log_path) : log_path_(log_path)
+{
+   Log::create(log_path);
+}
+
+TestLog::~TestLog() {
+
+	// Remove the log file. Comment out for debugging
+	fs::remove(log_path_);
+
+	// Explicitly destroy log. To keep valgrind happy
+	Log::destroy();
+}
+
+//======================================================================================================
+
+LogTimer::~LogTimer()
+{
+   Log* the_log = Log::instance();
+   if ( the_log ) {
+      std::stringstream ss; ss << " " << msg_ << " " << timer_.elapsed_seconds();
+      the_log->log(Log::DBG,ss.str());
+   }
+}
+
 //======================================================================================================
 LogImpl::LogImpl(const std::string& filename)
-: count_(0), file_(filename.c_str(), ios::out | ios::app)
+: file_(filename.c_str(), ios::out | ios::app)
 {
  	if (!file_.is_open()) {
- 	   std::string msg = "LogImpl::LogImpl: Could not open log file '";
- 	   msg += filename;
- 	   msg += "' ";
- 	   msg += File::stream_error_condition(file_);
-		std::cerr << msg << "\n";
-		// Do *NOT* throw std::runtime_error(msg), HERE as this can cause server to die.
-		// This would diagnose ECFLOW-1558 but is not acceptable
+ 	   log_open_error_ = "LogImpl::LogImpl: Could not open log file '";
+ 	   log_open_error_ += filename;
+ 	   log_open_error_ += "'. ";
+ 	   log_open_error_ += File::stream_error_condition(file_);
+	   //std::cerr << log_open_error_ << "\n";
+	   // Do *NOT* throw std::runtime_error(log_open_error_), HERE as this can cause server to die.
+	   // This would diagnose ECFLOW-1558 but is not acceptable
 	}
 }
 
-LogImpl::~LogImpl() {}
-
+LogImpl::~LogImpl() = default;
 
 static void append_log_type(std::string& str, Log::LogType lt)
 {
