@@ -12,23 +12,24 @@
 //
 // Description : Parser for white list file
 //============================================================================
-#include <vector>
+
+#include <unistd.h>  // for crypt
 #include <iostream>
+
 #include <boost/lexical_cast.hpp>
 #include <boost/algorithm/string/trim.hpp>
 
 #include "PasswdFile.hpp"
 #include "File.hpp"
 #include "Str.hpp"
-#include "Log.hpp"
 #include "User.hpp"
 
 using namespace ecf;
 using namespace std;
 using namespace boost;
 
-PasswdFile::PasswdFile() {}
-PasswdFile::~PasswdFile() {}
+PasswdFile::PasswdFile() = default;
+PasswdFile::~PasswdFile() = default;
 
 //#define DEBUG_ME 1
 
@@ -36,67 +37,82 @@ PasswdFile::~PasswdFile() {}
 // The parser expects version number  4.5.0
 bool PasswdFile::load(const std::string& file, bool debug,  std::string& errorMsg)
 {
-   if (debug)  std::cout << __func__ << "  " <<  passwd_file_ << " opening...\n";
-
+#ifdef DEBUG_ME
+   debug = true;
+#endif
    vec_.clear();
    passwd_file_ = file;
-
+   if (debug)  std::cout << __func__ << "  " <<  passwd_file_ << " opening...\n";
 
    std::vector<std::string> lines;
-   if (File::splitFileIntoLines(passwd_file_,lines,true /* ignore empty lines */)) {
-
-      bool foundVersionNumber = false; // can read from version 4.4.0 onwards
-      size_t lines_size = lines.size();
-      for(size_t i = 0; i < lines_size; ++i) {
-
-         if (lines[i].empty())   continue;
-
-         // ignore/remove all comments
-         if (lines[i][0] == '#') continue;
-         std::string theLine = lines[i];
-         string::size_type comment_pos = theLine.find("#");
-         if (comment_pos != std::string::npos ) {
-            theLine.erase(comment_pos);
-         }
-
-         boost::algorithm::trim(theLine);  // remove leading and trailing spaces
-         std::vector< std::string > lineTokens;
-         Str::split( theLine, lineTokens );
-         if ( lineTokens.empty() ) continue;
-
-         // version should be at the start
-         if (!foundVersionNumber) {
-
-            if (!validateVersionNumber(lineTokens[0], errorMsg )) {
-               std::stringstream ss;
-               ss << " " << i + 1 << ": " << lines[i] << "\n";
-               ss << "for ECF_PASSWD file " << passwd_file_ << "\n";
-               errorMsg += ss.str();
-               return false;
-            }
-            foundVersionNumber = true;
-            continue;
-         }
-         else  {
-           if (!add_user(lineTokens,errorMsg)) {
-              errorMsg += theLine;
-              return false;
-           }
-         }
-      }
-
+   if (!File::splitFileIntoLines(passwd_file_,lines,true /* ignore empty lines */)) {
+      errorMsg += "Could not open file specified by ECF_PASSWD/ECF_CUSTOM_PASSWD ";
+      errorMsg += passwd_file_;
+      errorMsg += " (";
+      errorMsg += strerror(errno);
+      errorMsg += ")";
       if (debug)  std::cout << dump() << "\n";
-
-      return true;
+      return false;
    }
 
-   errorMsg += "Could not open file specified by ECF_PASSWD ";
-   errorMsg += passwd_file_;
-   errorMsg += " (";
-   errorMsg += strerror(errno);
-   errorMsg += ")";
-   if (debug)  std::cout << dump() << "\n";
-   return false;
+   bool foundVersionNumber = false; // can read from version 4.4.0 onwards
+   size_t lines_size = lines.size();
+   for(size_t i = 0; i < lines_size; ++i) {
+
+      if (lines[i].empty())   continue;
+
+      // ignore/remove all comments
+      if (lines[i][0] == '#') continue;
+      std::string theLine = lines[i];
+      string::size_type comment_pos = theLine.find("#");
+      if (comment_pos != std::string::npos ) {
+         theLine.erase(comment_pos);
+      }
+
+      boost::algorithm::trim(theLine);  // remove leading and trailing spaces
+      std::vector< std::string > lineTokens;
+      Str::split( theLine, lineTokens );
+      if ( lineTokens.empty() ) continue;
+
+      // version should be at the start
+      if (!foundVersionNumber) {
+
+         if (!validateVersionNumber(lineTokens[0], errorMsg )) {
+            std::stringstream ss;
+            ss << " " << i + 1 << ": " << lines[i] << "\n";
+            ss << "for ECF_PASSWD/ECF_CUSTOM_PASSWD file " << passwd_file_ << "\n";
+            errorMsg += ss.str();
+            return false;
+         }
+         foundVersionNumber = true;
+         continue;
+      }
+      else  {
+         if (!add_user(lineTokens,errorMsg)) {
+            errorMsg += theLine;
+            return false;
+         }
+      }
+   }
+
+   if (debug)  std::cout << dump();
+
+   // Now check the user is unique for a given host/port:
+   //   fred host 3141 xxxx
+   //   fred host 3141 yyyy  this is illegal.
+   size_t vec_size = vec_.size();
+   for (size_t i = 0; i < vec_size; i++) {
+      for (size_t k = 0; k < vec_size; k++) {
+         if (i == k) continue;
+         if (vec_[i].user() == vec_[k].user() && vec_[i].host() == vec_[k].host() && vec_[i].port() == vec_[k].port()) {
+            std::stringstream ss;
+            ss << "user " << vec_[i].user() << " can only appear once for given host/port\n";
+            errorMsg += ss.str();
+            return false;
+         }
+      }
+   }
+   return true;
 }
 
 bool PasswdFile::check_at_least_one_user_with_host_and_port(const std::string& host, const std::string& port)
@@ -124,60 +140,66 @@ std::string PasswdFile::get_passwd(const std::string& user, const std::string& h
 
 bool PasswdFile::authenticate(const std::string& user, const std::string& passwd) const
 {
-//#ifdef DEBUG_ME
-//   cout << "   PasswdFile::authenticate user(" << user << ") passwd(" << passwd << ")\n";
-//#endif
+#ifdef DEBUG_ME
+   cout << "   PasswdFile::authenticate user:" << user << " passwd:" << passwd << " file:" <<  passwd_file_ << " vec_.size(): " << vec_.size() << "\n";
+   cout << dump() << "\n";
+#endif
 
    if (user.empty()) {
-//#ifdef DEBUG_ME
-//      cout << "      PasswdFile::authenticate FALSE user empty.\n";
-//#endif
+#ifdef DEBUG_ME
+      cout << "      PasswdFile::authenticate: user empty:  FAIL\n";
+#endif
       return false;
    }
 
+   // no password specified, and password file is empty
+   if (passwd.empty() && vec_.empty()) {
+#ifdef DEBUG_ME
+      cout << "      PasswdFile::authenticate no password and password file empty: PASS\n";
+#endif
+      return true;
+   }
+
    // Only fail if user exists in the passwd file
-   bool user_found  = false;
    size_t vec_size = vec_.size();
    for (size_t i = 0; i < vec_size; i++) {
       if (vec_[i].user() == user) {
-         user_found = true;
          if (vec_[i].passwd() == passwd) {
-//#ifdef DEBUG_ME
-//            cout << "      PasswdFile::authenticate TRUE \n";
-//#endif
+#ifdef DEBUG_ME
+            cout << "      PasswdFile::authenticate user and password match: PASS\n";
+#endif
             return true;
          }
+         else {
+#ifdef DEBUG_ME
+            cout << "      PasswdFile::authenticate: user found but passwd did not match: FAIL\n";
+#endif
+            return false;
+         }
       }
    }
 
-   // User found in password file and passwd did not match
-   if (user_found) {
-//#ifdef DEBUG_ME
-//      cout << "      PasswdFile::authenticate FALSE, user found but passwd did not match\n";
-//#endif
+   // Will ONLY reach here if user not found
+
+   // User not found, but if passwd is not empty, then fail.
+   if (!passwd.empty()) {
+#ifdef DEBUG_ME
+     cout << "      PasswdFile::authenticate: user NOT found, passwd is NOT EMPTY: FAIL\n";
+#endif
       return false;
    }
-   else {
-      // User not found, but if passwd is not empty, then fail.
-      if (!passwd.empty()) {
-//#ifdef DEBUG_ME
-//         cout << "      PasswdFile::authenticate FALSE, user NOT found, passwd is NOT EMPTY\n";
-//#endif
-         return false;
-      }
-   }
 
    // Server has a password file, but user not found and passwd is empty.
    if (vec_.empty()) {
-//#ifdef DEBUG_ME
-//    cout << "      PasswdFile::authenticate true, user NOT found, and passwd EMPTY, and password file EMPTY\n";
-//#endif
+#ifdef DEBUG_ME
+    cout << "      PasswdFile::authenticate: true, user NOT found, and passwd EMPTY, and password file EMPTY\n";
+#endif
       return true;
    }
 
-//#ifdef DEBUG_ME
-//  cout << "      PasswdFile::authenticate false, user NOT found, and passwd EMPTY\n";
-//#endif
+#ifdef DEBUG_ME
+  cout << "      PasswdFile::authenticate false, user NOT found, and passwd EMPTY\n";
+#endif
    return false;
 }
 
@@ -198,9 +220,9 @@ bool PasswdFile::validateVersionNumber(const std::string& line, std::string& err
       }
 
       try {
-         int major = boost::lexical_cast< int >( versionNumberTokens[0] );
-         int minor = boost::lexical_cast< int >( versionNumberTokens[1] );
-         int part = boost::lexical_cast< int >( versionNumberTokens[2] );
+         auto major = boost::lexical_cast< int >( versionNumberTokens[0] );
+         auto minor = boost::lexical_cast< int >( versionNumberTokens[1] );
+         auto part = boost::lexical_cast< int >( versionNumberTokens[2] );
          if ( major < 4  ) {
             errorMsg += "Only passwd files with a version >= 4.5.0 is supported\n";
             return false;
@@ -245,7 +267,7 @@ bool PasswdFile::add_user(std::vector<std::string>& tokens, std::string& error_m
       return false;
    }
 
-   vec_.push_back(Pass_wd(tokens[0],tokens[1],tokens[2],tokens[3]));
+   vec_.emplace_back(tokens[0],tokens[1],tokens[2],crypt(tokens[3].c_str(),tokens[0].c_str()));
 
    return true;
 }
@@ -253,8 +275,10 @@ bool PasswdFile::add_user(std::vector<std::string>& tokens, std::string& error_m
 std::string PasswdFile::dump() const
 {
    std::stringstream ss;
-   for (size_t i = 0; i < vec_.size(); i++) {
-      ss << vec_[i].user() << " " << vec_[i].host() << ":" << vec_[i].port() << "\n";
+   int count = 1;
+   for (const auto & i : vec_) {
+      ss << count << ": " << i.user() << " " << i.host() << ":" << i.port() << "\n";
+      count++;
    }
    return ss.str();
 }
@@ -266,9 +290,9 @@ bool PasswdFile::createWithAccess(
       const std::string& passwd,
       std::string& errorMsg)
 {
-   std::vector<std::string> lines; lines.reserve( 2 );
+   std::vector<std::string> lines; lines.reserve( 3 );
 
-   lines.push_back("4.5.0");
+   lines.emplace_back("4.5.0");
 
    string line;
    line += User::login_name();
