@@ -13,22 +13,31 @@
 // Description :
 //============================================================================
 
+#include <stdexcept>
 #include <boost/date_time/posix_time/time_formatters.hpp>  // requires boost date and time lib
 #include "TimeSeries.hpp"
-#include "Indentor.hpp"
+
 #include "Calendar.hpp"
 #include "Log.hpp"
 #include "Str.hpp"
-#include "Ecf.hpp"
 #include "Extract.hpp"
+#include "Serialization.hpp"
+#include "cereal_boost_time.hpp"
+#ifdef DEBUG
+#include "Ecf.hpp"
+#endif
+
+//#define DEBUG_TIME_SERIES 1
+//#define DEBUG_TIME_SERIES_IS_FREE 1
+#ifdef DEBUG_TIME_SERIES_IS_FREE
+#include "Indentor.hpp"
+#endif
 
 using namespace std;
 using namespace ecf;
 using namespace boost::gregorian;
 using namespace boost::posix_time;
 
-//#define DEBUG_TIME_SERIES 1
-//#define DEBUG_TIME_SERIES_IS_FREE 1
 
 static void testTimeSlot( const ecf::TimeSlot& ts)
 {
@@ -44,42 +53,32 @@ static void testTimeSlot( const ecf::TimeSlot& ts)
 
 namespace ecf {
 
-TimeSeries::TimeSeries()
-: relativeToSuiteStart_(false),
-  isValid_(true),
-  relativeDuration_(0,0,0,0),
-  lastTimeSlot_(0,0,0,0) {}
+TimeSeries::TimeSeries() = default;
 
 TimeSeries::TimeSeries(int hour, int minute, bool relative)
-: relativeToSuiteStart_(relative),
-  isValid_(true),
-  start_(hour,minute),
+: start_(hour,minute),
   nextTimeSlot_(hour,minute),
-  relativeDuration_(0,0,0,0),
-  lastTimeSlot_(start_.duration())
+  lastTimeSlot_(start_.duration()),
+  relativeToSuiteStart_(relative)
 {
 	testTimeSlot(start_);
 }
 
 TimeSeries::TimeSeries(const TimeSlot& t,bool relative)
-: relativeToSuiteStart_(relative),
-  isValid_(true),
-  start_(t),
+: start_(t),
   nextTimeSlot_(t),
-  relativeDuration_(0,0,0,0),
-  lastTimeSlot_(t.duration())
+  lastTimeSlot_(t.duration()),
+  relativeToSuiteStart_(relative)
 {
 	testTimeSlot(start_);
 }
 
 TimeSeries::TimeSeries(const TimeSlot& start, const TimeSlot& finish, const TimeSlot& incr,bool relative)
-: relativeToSuiteStart_(relative),
-  isValid_(true),
-  start_( start ),
+: start_( start ),
   finish_( finish ),
   incr_( incr ),
   nextTimeSlot_( start ),
-  relativeDuration_(0,0,0,0)
+  relativeToSuiteStart_(relative)
 {
 	testTimeSlot(start);
 	testTimeSlot(finish);
@@ -113,9 +112,9 @@ TimeSeries::TimeSeries(const TimeSlot& start, const TimeSlot& finish, const Time
 
  	compute_last_time_slot();
 
-#ifdef DEBUG_TIME_SERIES
-	cout << "TimeSeries::TimeSeries "  << dump() << "\n";
-#endif
+//#ifdef DEBUG_TIME_SERIES
+//	cout << "TimeSeries::TimeSeries "  << dump() << "\n";
+//#endif
 }
 
 bool TimeSeries::operator<(const TimeSeries& rhs) const
@@ -153,10 +152,15 @@ bool TimeSeries::resetRelativeDuration()
 {
 	if ( relativeToSuiteStart_ ) {
 	   relativeDuration_ = time_duration(0,0,0,0);
+
+#ifdef DEBUG_TIME_SERIES
+	   log(Log::DBG,"TimeSeries::resetRelativeDuration " + dump());
+#endif
+
 	   return true;
 	}
 #ifdef DEBUG_TIME_SERIES
-	cout << "TimeSeries::resetRelativeDuration "  << dump() << "\n";
+	log(Log::DBG,"TimeSeries::resetRelativeDuration " + dump());
 #endif
 	return false;
 }
@@ -184,9 +188,10 @@ void TimeSeries::reset(const ecf::Calendar& c)
  	time_duration current_time = duration(c);
    if (hasIncrement()) {
 
-      // only used when we have a series
-      suiteTimeAtReque_ = TimeSlot(c.suiteTime().time_of_day());
-      //cout << "TimeSeries::reset  suiteTimeAtReque_: " << suiteTimeAtReque_ << " =====================================================\n";
+      // only used when we have a series, and *NOT* relative. A relative time series does not care about midnight.
+	  if (!relativeToSuiteStart_) {
+		  suiteTimeAtReque_ = TimeSlot(c.suiteTime().time_of_day());
+	  }
 
       while( current_time > nextTimeSlot_.duration()) {
          time_duration value = nextTimeSlot_.duration();
@@ -204,8 +209,8 @@ void TimeSeries::reset(const ecf::Calendar& c)
    }
 
 #ifdef DEBUG_TIME_SERIES
-	LogToCout toCoutAsWell;
-	LOG(Log::DBG,"      TimeSeries::reset   "  << dump());
+//	LogToCout toCoutAsWell;
+	log(Log::DBG,"TimeSeries::reset   " + dump());
 #endif
 }
 
@@ -241,15 +246,16 @@ void TimeSeries::requeue(const ecf::Calendar& c,bool reset_next_time_slot)
 		if (current_time >= start_.duration() ) {
 			isValid_ = false; // time has expired
 #ifdef DEBUG_TIME_SERIES
-	 	 	LOG(Log::DBG,"      TimeSeries::increment (duration(c) >= start_.duration() ) "  << toString() << " duration=" << to_simple_string(duration(c)));
+	 	 	LOG(Log::DBG,"TimeSeries::requeue HOLDING TIME EXPIRED (duration(c) >= start_.duration() ) "  << dump() << " duration=" << to_simple_string(duration(c)));
 #endif
 		}
 		return;
 	}
 
-	// only used when we have a series
-   suiteTimeAtReque_ = TimeSlot(c.suiteTime().time_of_day());
-//   cout << "TimeSeries::requeue suiteTimeAtReque_: " << suiteTimeAtReque_ << " =====================================================\n";
+	// Only used when we have a series, and NOT relative. A relative time series does not care about midnight
+	if (!relativeToSuiteStart_) {
+		suiteTimeAtReque_ = TimeSlot(c.suiteTime().time_of_day());
+	}
 
 	// the nextTimeSlot_ needs to be set to a multiple of incr
 	// However the nextTimeSlot_ can not just be incremented by incr
@@ -267,16 +273,13 @@ void TimeSeries::requeue(const ecf::Calendar& c,bool reset_next_time_slot)
 		time_duration value = nextTimeSlot_.duration();
 		value += incr_.duration();
 		nextTimeSlot_ = TimeSlot(value.hours(),value.minutes());
-#ifdef DEBUG_TIME_SERIES
- 	 	LOG(Log::DBG,"      TimeSeries::increment "  << toString());
-#endif
 	}
 
  	if (nextTimeSlot_ > finish_) {
 		isValid_ = false;              // time has expired
 	   suiteTimeAtReque_ = TimeSlot(); // expire for new requeue
 #ifdef DEBUG_TIME_SERIES
- 	 	LOG(Log::DBG,"      TimeSeries::increment "  << toString());
+ 	 	log(Log::DBG,"TimeSeries::requeue  HOLDING TIME EXPIRED (nextTimeSlot_ > finish_) " + dump());
 #endif
  	}
 }
@@ -287,7 +290,7 @@ TimeSlot TimeSeries::compute_next_time_slot(const ecf::Calendar& c) const
    time_duration current_time = duration(c);
    if (!hasIncrement()) {
       if (current_time >= start_.duration() ) {
-         return TimeSlot(); // time has expired
+         return {}; // time has expired
       }
       return start_;
    }
@@ -300,7 +303,7 @@ TimeSlot TimeSeries::compute_next_time_slot(const ecf::Calendar& c) const
    }
 
    if (nextTimeSlot > finish_) {
-      return TimeSlot();  // time has expired
+      return {};  // time has expired
    }
    return nextTimeSlot;
 }
@@ -326,7 +329,7 @@ bool TimeSeries::isFree(const ecf::Calendar& calendar) const
 	if (!isValid_) {
 	   // time has expired, hence time is not free
 #ifdef DEBUG_TIME_SERIES_IS_FREE
-   	 	LOG(Log::DBG,"TimeSeries::isFree (!isValid_) HOLDING "  << dump());
+   	 	log(Log::DBG,"TimeSeries::isFree (!isValid_) HOLDING " + dump());
 #endif
 		return false;
 	}
@@ -396,7 +399,7 @@ bool TimeSeries::match_duration_with_time_series(const boost::posix_time::time_d
 	}
 
 #ifdef DEBUG_TIME_SERIES
-	LOG(Log::DBG,"TimeSeries::matches HOLDING (nextTimeSlot_td > endDuration)  " << dump() << " HOLDING at " << to_simple_string(relative_or_real_td));
+	LOG(Log::DBG,"TimeSeries::match_duration_with_time_series HOLDING (nextTimeSlot_td > endDuration)  " << dump() << " HOLDING at " << to_simple_string(relative_or_real_td));
 #endif
  	return false;
 }
@@ -418,7 +421,8 @@ void TimeSeries::miss_next_time_slot()
    }
 }
 
-bool TimeSeries::checkForRequeue( const ecf::Calendar& calendar, const TimeSlot& the_min, const TimeSlot& the_max) const
+
+bool TimeSeries::checkForRequeue( const ecf::Calendar& calendar, const TimeSlot& the_min, const TimeSlot& the_max,bool cmd_context) const
 {
    // ************************************************************************
    // THIS IS CALLED IN THE CONTEXT WHERE NODE HAS COMPLETED. Hence ****asyncronous****
@@ -426,10 +430,17 @@ bool TimeSeries::checkForRequeue( const ecf::Calendar& calendar, const TimeSlot&
    // HENCE THIS FUNCTION MUST RETURN FALSE, WHEN END OF TIME SLOT HAS BEEN REACHED/expired
    // The resolution is in minutes
    // *************************************************************************
-   //cout << "TimeSeries::checkForRequeue " << calendar.toString() << "\n";
+   //cout << "TimeSeries::checkForRequeue " << calendar.suite_time_str() << " min: " << the_min << " max: " << the_max << " cmd_context(" << cmd_context << ") +++++++++++++++\n";
+#ifdef DEBUG_TIME_SERIES
+   log(Log::DBG,"TimeSeries::checkForRequeue: " + dump());
+#endif
 
    if (!isValid_) {
       // time has expired, hence can no longer re-queues, i.e no future time dependency
+#ifdef DEBUG_TIME_SERIES
+	   //cout << " TimeSeries::checkForRequeue " << calendar.suite_time_str() << "  HOLDING\n";
+	  log(Log::DBG,"TimeSeries::checkForRequeue HOLDING !isValid_  " + dump());
+#endif
       return false;
    }
 
@@ -443,11 +454,16 @@ bool TimeSeries::checkForRequeue( const ecf::Calendar& calendar, const TimeSlot&
       // If the current value is greater that finish, then returning true would increment
       // value past the end, and force node state to be stuck in state queue.
       if ( nextTimeSlot_ > finish_ ) {
+#ifdef DEBUG_TIME_SERIES
+   	    //cout << " TimeSeries::checkForRequeue ( nextTimeSlot_ > finish_ )    HOLDING\n";
+   	    log(Log::DBG,"TimeSeries::checkForRequeue HOLDING  ( nextTimeSlot_ > finish_ )  " + dump());
+#endif
          return false;
       }
 
-      // ECFLOW-130 jobs that start before midnight and finish after midnight should not requeue
-      if (!suiteTimeAtReque_.isNULL()){
+      // ECFLOW-130 jobs that start before midnight and finish after midnight should not re-queue
+      // This should ONLY apply to non relative time series, since a relative time series does care about midnight
+      if (!relativeToSuiteStart_ && !suiteTimeAtReque_.isNULL()){
          TimeSlot suiteTimeNow(calendar.suiteTime().time_of_day());
          //cout << "TimeSeries::checkForRequeue suiteTimeNow = " << suiteTimeNow << " =====================================================\n";
          // we use >= specifically for unit test, to pass.
@@ -457,15 +473,30 @@ bool TimeSeries::checkForRequeue( const ecf::Calendar& calendar, const TimeSlot&
          }
          else {
             // The day changed between (requeue/reset):->queued->submitted->active->complete->(checkForRequeue)
-            //cout << "TimeSeries::checkForRequeue day changed =====================================================\n";
+#ifdef DEBUG_TIME_SERIES
+            //cout << " TimeSeries::checkForRequeue day changed ===================================================== HOLDING\n";
+       	     log(Log::DBG,"TimeSeries::checkForRequeue HOLDING suiteTimeNow < suiteTimeAtReque_  " + dump());
+#endif
             return false;
          }
       }
 
       time_duration calendar_duration = duration(calendar);
-      if ( calendar_duration < lastTimeSlot_) {
-         return true;
+      if (cmd_context) {
+    	  // In the *COMMAND* context, allow re-queue if we are *BEFORE* the last time slot
+    	  if (calendar_duration < lastTimeSlot_) {
+    		  return true;
+    	  }
       }
+      else {
+    	  if (calendar_duration >= start_.duration() && calendar_duration < lastTimeSlot_) {
+    		  return true;
+    	  }
+      }
+
+#ifdef DEBUG_TIME_SERIES
+      log(Log::DBG,"TimeSeries::checkForRequeue  HOLDING calendar_duration < start_ || calendar_duration >= lastTimeSlot_)  " + dump());
+#endif
       return false;
    }
 
@@ -477,15 +508,33 @@ bool TimeSeries::checkForRequeue( const ecf::Calendar& calendar, const TimeSlot&
 
    // We have a single time slot, *OR* multiple with same time slot
    if (the_min == the_max) {
+	   //cout << " TimeSeries::checkForRequeue (the_min == the_max) HOLDING \n";
+#ifdef DEBUG_TIME_SERIES
+	  log(Log::DBG,"TimeSeries::checkForRequeue  HOLDING  the_min == the_max  " + dump());
+#endif
       return false;
    }
 
    // The the_min/the_max takes into account *all* start/finish Time and Today attributes
    time_duration calendar_duration = duration(calendar);
-   if (calendar_duration < the_max.duration()) {
-      return true;
+   if (cmd_context) {
+ 	   // In the *COMMAND* context, allow re-queue if we are *BEFORE* the max time slot
+	   if (calendar_duration < the_max.duration()) {
+		   //cout << " TimeSeries::checkForRequeue *COMMAND* context calendar_duration < the_max.duration()) --> allow requeue\n";
+		   return true;
+	   }
+   }
+   else {
+	   if (calendar_duration >= the_min.duration() && calendar_duration < the_max.duration()) {
+		   //cout << " TimeSeries::checkForRequeue (calendar_duration > the_min.duration() && calendar_duration < the_max.duration()) --> allow requeue\n";
+		   return true;
+	   }
    }
 
+   //cout << " TimeSeries::checkForRequeue calendar duration " << to_simple_string(calendar_duration) << " not in range " <<  to_simple_string(the_min.duration())  << "-" <<  to_simple_string(the_max.duration()) << "  HOLDING \n";
+#ifdef DEBUG_TIME_SERIES
+   log(Log::DBG,"TimeSeries::checkForRequeue HOLDING multiple slots not in range " + dump());
+#endif
    return false;
 }
 
@@ -522,12 +571,12 @@ boost::posix_time::time_duration TimeSeries::duration(const ecf::Calendar& c ) c
 	if ( relativeToSuiteStart_ ) {
 
  		// relative to suite start only
-		return boost::posix_time::time_duration( relativeDuration_.hours(), relativeDuration_.minutes(), 0, 0 );
+		return { relativeDuration_.hours(), relativeDuration_.minutes(), 0, 0 };
 	}
 
 	LOG_ASSERT(!c.suiteTime().is_special(),"init has not been called on calendar. TimeSeries::duration");
 	time_duration time_of_day = c.suiteTime().time_of_day();
-	return boost::posix_time::time_duration( time_of_day.hours(), time_of_day.minutes(), 0, 0 );
+	return { time_of_day.hours(), time_of_day.minutes(), 0, 0 };
 }
 
 
@@ -588,34 +637,40 @@ bool TimeSeries::operator==(const TimeSeries& rhs) const
 	return structureEquals(rhs);
 }
 
-std::ostream& TimeSeries::print(std::ostream& os) const
+void TimeSeries::print(std::string& os) const
 {
-	os << toString() << "\n";
-	return os;
+   write(os);
+	os += "\n";
 }
 
 std::string TimeSeries::toString() const
 {
    std::string ret ;
+   write(ret);
+   return ret;
+}
+
+void TimeSeries::write(std::string& ret) const
+{
    if (relativeToSuiteStart_) ret += "+";
-   ret += start_.toString();
+   start_.write(ret);
    if (!finish_.isNULL()) {
       ret += " ";
-      ret += finish_.toString();
+      finish_.write(ret);
       ret += " ";
-      ret += incr_.toString();
+      incr_.write(ret);
    }
-   return ret;
 }
 
 std::string TimeSeries::dump() const
 {
 	std::stringstream ss;
 	ss << toString();
-	ss << " isValid_(" << isValid_ << ")";
-	ss << " value(" << nextTimeSlot_.toString() << ")";
-	ss << " relativeDuration_(" <<  to_simple_string( relativeDuration_) << ")";
- 	ss << " lastTimeSlot_(" <<  to_simple_string(lastTimeSlot_) << ")";
+	ss << " isValid_("          << isValid_ << ")";
+	ss << " nextTimeSlot_("     << nextTimeSlot_.toString() << ")";
+	ss << " relativeDuration_(" << to_simple_string(relativeDuration_) << ")";
+ 	ss << " lastTimeSlot_("     << to_simple_string(lastTimeSlot_) << ")";
+ 	ss << " suiteTimeAtReque_(" << suiteTimeAtReque_.toString() << ")";
  	return ss.str();
 }
 
@@ -650,12 +705,6 @@ bool TimeSeries::checkInvariants(std::string& errormsg) const
 		cout <<  errormsg << "  " << toString() << "\n";
 		return false;
 	}
-	if (relativeToSuiteStart_  && relativeDuration_.hours() > 99) {
-		errormsg += "TimeSeries::checkInvariants. Max relative duration is 99 hours & 59 minutes";
-		cout <<  errormsg << "  " << toString() << "\n";
-		return false;
-	}
-
 
 	if (!relativeToSuiteStart_ && relativeDuration_.total_seconds() > 0) {
 		errormsg += "TimeSeries::checkInvariants Can only have RelativeDuration if relativeToSuiteStart_ flag is set";
@@ -667,10 +716,15 @@ bool TimeSeries::checkInvariants(std::string& errormsg) const
 }
 
 std::ostream& operator<<(std::ostream& os, const TimeSeries* d) {
-	if (d) return d->print(os);
+	if (d) {
+	   string s;
+	   d->print(s);
+	   os << s;
+	   return os;
+	}
 	return os << "TimeSlot == NULL";
 }
-std::ostream& operator<<(std::ostream& os, const TimeSeries& d)  { return d.print(os); }
+std::ostream& operator<<(std::ostream& os, const TimeSeries& d)  { string s; d.print(s); os << s; return os; }
 
 
 ecf::TimeSeries TimeSeries::create(const std::string& str)
@@ -689,6 +743,17 @@ std::string TimeSeries::state_to_string(bool isFree) const
    //             Hence use of '/' character
    // time 10:30 # free isValid:false nextTimeSlot/10:30 relativeDuration/00:00:00
    std::string ret;
+   write_state(ret,isFree);
+   return ret;
+}
+
+void TimeSeries::write_state(std::string& ret,bool isFree) const
+{
+   // *IMPORTANT* we *CANT* use ';' character, since is used in the parser, when we have
+   //             multiple statement on a single line i.e.
+   //                 task a; task b;
+   //             Hence use of '/' character
+   // time 10:30 # free isValid:false nextTimeSlot/10:30 relativeDuration/00:00:00
    bool next_time_slot_changed = ( nextTimeSlot_ != start_);
    bool relative_duration_changed = (!relativeDuration_.is_special() && relativeDuration_.total_seconds() != 0);
    if (isFree || !isValid_ || next_time_slot_changed || relative_duration_changed) {
@@ -698,7 +763,19 @@ std::string TimeSeries::state_to_string(bool isFree) const
       if (next_time_slot_changed) { ret += " nextTimeSlot/"; ret += nextTimeSlot_.toString(); }
       if (relative_duration_changed) { ret += " relativeDuration/"; ret += to_simple_string(relativeDuration_); }
    }
-   return ret;
+}
+
+void TimeSeries::write_state_for_gui(std::string& ret,bool isFree) const
+{
+   bool next_time_slot_changed = ( nextTimeSlot_ != start_);
+   bool relative_duration_changed = (!relativeDuration_.is_special() && relativeDuration_.total_seconds() != 0);
+   if (isFree || !isValid_ || next_time_slot_changed || relative_duration_changed) {
+      ret += " #";
+      if (isFree) ret += " free";
+      if (!isValid_) ret += " expired";
+      if (next_time_slot_changed) { ret += " nextTimeSlot=";   ret += nextTimeSlot_.toString(); }
+      if (relative_duration_changed) { ret += " relDuration="; ret += to_simple_string(relativeDuration_); }
+   }
 }
 
 void TimeSeries::parse_state(size_t index,const std::vector<std::string>& lineTokens, ecf::TimeSeries& ts)
@@ -712,7 +789,8 @@ void TimeSeries::parse_state(size_t index,const std::vector<std::string>& lineTo
    // time 10:30              # free isValid:false nextTimeSlot/10:30 relativeDuration/00:00:00
    // cron 10:00 20:00 01:00  # free isValid:false nextTimeSlot/10:30 relativeDuration/00:00:00
    bool comment_fnd = false;
-   for(size_t i = index; i < lineTokens.size(); i++) {
+   size_t line_tokens_size = lineTokens.size();
+   for(size_t i = index; i < line_tokens_size; i++) {
       if (comment_fnd) {
          if (lineTokens[i] == "isValid:false") { ts.isValid_ = false; continue;}
          if (lineTokens[i].find("nextTimeSlot") != std::string::npos) {
@@ -742,7 +820,8 @@ void TimeSeries::parse_state(size_t index,const std::vector<std::string>& lineTo
 
 ecf::TimeSeries TimeSeries::create( size_t& index,const std::vector<std::string>& lineTokens,bool read_state )
 {
-	assert( index < lineTokens.size() );
+   size_t line_tokens_size = lineTokens.size();
+	assert( index < line_tokens_size );
 	int startHour = -1;
 	int startMin = -1;
 
@@ -759,11 +838,11 @@ ecf::TimeSeries TimeSeries::create( size_t& index,const std::vector<std::string>
 	TimeSlot start( startHour, startMin );
 
 	index++; // on 20:00
-	if ( index < lineTokens.size() && lineTokens[index][0] != '#' ) {
+	if ( index < line_tokens_size && lineTokens[index][0] != '#' ) {
 
 		// if third token is not a comment the time must be of the form
 		// cron 10:00 20:00 01:00
-		if ( index+1 >= lineTokens.size() ) throw std::runtime_error( "TimeSeries::create: Invalid time series :");
+		if ( index+1 >= line_tokens_size ) throw std::runtime_error( "TimeSeries::create: Invalid time series :");
 
 		int finishHour = -1;
 		int finishMin = -1;
@@ -836,5 +915,23 @@ void TimeSeries::testTime(int hour, int minute)
   	}
 }
 
-}
 
+
+template<class Archive>
+void TimeSeries::serialize(Archive & ar, std::uint32_t const  /*version*/)
+{
+   ar(CEREAL_NVP(start_ ));
+   CEREAL_OPTIONAL_NVP(ar,finish_,               [this](){return !finish_.isNULL();});
+   CEREAL_OPTIONAL_NVP(ar,incr_ ,                [this](){return !incr_.isNULL();});
+   CEREAL_OPTIONAL_NVP(ar,nextTimeSlot_,         [this](){return !nextTimeSlot_.isNULL() && nextTimeSlot_ != start_;});
+   CEREAL_OPTIONAL_NVP(ar,relativeDuration_,     [this](){return relativeDuration_ != boost::posix_time::time_duration(0,0,0,0);});
+   CEREAL_OPTIONAL_NVP(ar, relativeToSuiteStart_,[this](){return relativeToSuiteStart_ ;});
+   CEREAL_OPTIONAL_NVP(ar, isValid_,             [this](){return !isValid_ ;});
+
+   if (Archive::is_loading::value) {
+      if (nextTimeSlot_.isNULL()) nextTimeSlot_ = start_;
+      if (!finish_.isNULL())  compute_last_time_slot();
+   }
+}
+CEREAL_TEMPLATE_SPECIALIZE_V(TimeSeries);
+}
