@@ -14,13 +14,8 @@
 //============================================================================
 #include <string>
 #include <iostream>
-#include <fstream>
 
-#include <boost/archive/tmpdir.hpp>
 #include <boost/test/unit_test.hpp>
-#include "boost/filesystem/operations.hpp"
-#include "boost/filesystem/path.hpp"
-#include "boost/progress.hpp"
 
 #include "Defs.hpp"
 #include "Family.hpp"
@@ -30,7 +25,6 @@
 #include "Memento.hpp"
 #include "MyDefsFixture.hpp"
 
-namespace fs = boost::filesystem;
 using namespace std;
 using namespace ecf;
 
@@ -71,7 +65,7 @@ BOOST_AUTO_TEST_CASE( test_state_parser )
       // Change state other the default
       defs.beginAll();
       suite->set_state(NState::ABORTED);
-      for (size_t i = 0; i < flag_list.size(); ++i)  suite->flag().set( flag_list[i] );
+      for (auto & i : flag_list)  suite->flag().set( i );
       suite->suspend();
       BOOST_CHECK_MESSAGE( helper.test_state_persist_and_reload_with_checkpt(defs),"Add one suite failed: " << helper.errorMsg());
    }
@@ -82,7 +76,7 @@ BOOST_AUTO_TEST_CASE( test_state_parser )
 
       // Change state other the default
       f1->set_state(NState::COMPLETE);
-      for (size_t i = 0; i < flag_list.size(); ++i)  f1->flag().set( flag_list[i] );
+      for (auto & i : flag_list)  f1->flag().set( i );
       f1->suspend();
       BOOST_CHECK_MESSAGE( helper.test_state_persist_and_reload_with_checkpt(defs),"Add one family failed: " <<  helper.errorMsg());
 
@@ -96,7 +90,7 @@ BOOST_AUTO_TEST_CASE( test_state_parser )
       family_ptr f1 = defs.add_suite("s1")->add_family("f1");
       task_ptr t1 = f1->add_task("t1");
 
-      for (size_t i = 0; i < flag_list.size(); ++i)  t1->flag().set( flag_list[i] );
+      for (auto & i : flag_list)  t1->flag().set( i );
       t1->suspend();
       t1->set_state(NState::COMPLETE);
 
@@ -116,7 +110,7 @@ BOOST_AUTO_TEST_CASE( test_state_parser )
       Defs defs;
       task_ptr task = defs.add_suite("s1")->add_family("f1")->add_task("t1");
       alias_ptr t1 = task->add_alias_only();
-      for (size_t i = 0; i < flag_list.size(); ++i)  t1->flag().set( flag_list[i] );
+      for (auto & i : flag_list)  t1->flag().set( i );
       t1->suspend();
       t1->set_state(NState::COMPLETE);
       // Use memento to modify alias state
@@ -173,12 +167,30 @@ BOOST_AUTO_TEST_CASE( test_state_node_attributes )
       Event event("event");     event.set_value(true);
       Event event2(10,"event"); event2.set_value(true);
       Event event3(10);         event3.set_value(true);
+      Event event4(10,"event4",true/*init value*/,true/*value*/);
+      Event event5(10,"event5",true/*init value*/,false/*value*/);
       task->addEvent(event);
       task->addEvent(event2);
       task->addEvent(event3);
+      task->addEvent(event4);
+      task->addEvent(event5);
       //      PrintStyle style(PrintStyle::MIGRATE); std::cout << defs;
       BOOST_CHECK_MESSAGE( helper.test_state_persist_and_reload_with_checkpt(defs), "Event state: failed: " << helper.errorMsg());
    }
+   {
+      Defs defs;
+      suite_ptr suite = defs.add_suite("s1");
+      family_ptr fam = suite->add_family("f1");
+      task_ptr t = fam->add_task("t");
+      std::vector<std::string> queue_items; queue_items.emplace_back("001"); queue_items.emplace_back("002"); queue_items.emplace_back("003");
+      QueueAttr queue("queue",queue_items); queue.active(); queue.active();
+      QueueAttr queue1("queue1",queue_items); queue1.active();
+      suite->add_queue(queue); suite->add_queue(queue1);
+      fam->add_queue(queue); fam->add_queue(queue1);
+      t->add_queue(queue); t->add_queue(queue1);
+      //PrintStyle::setStyle(PrintStyle::MIGRATE); std::cout << defs;
+      BOOST_CHECK_MESSAGE( helper.test_state_persist_and_reload_with_checkpt(defs), "Queue state: failed: " << helper.errorMsg());
+   }
    {
       {
          Defs defs;
@@ -214,24 +226,34 @@ BOOST_AUTO_TEST_CASE( test_state_node_attributes )
       }
    }
    {
-       Defs defs;
-       suite_ptr suite = defs.add_suite("s1");
-       task_ptr t1 = suite->add_task("t1");
-       task_ptr t2 = suite->add_task("t2");
-       task_ptr t3 = suite->add_task("t3");
-       task_ptr t4 = suite->add_task("t4");
-       Limit limit("limit",10);
-       limit.increment(1,t1->absNodePath());
-       limit.increment(1,t2->absNodePath());
-       limit.increment(1,t3->absNodePath());
-       limit.increment(1,t4->absNodePath());
-       suite->addLimit(limit);
-       //      PrintStyle style(PrintStyle::MIGRATE); std::cout << defs;
-       BOOST_CHECK_MESSAGE( helper.test_state_persist_and_reload_with_checkpt(defs), "Limit state: failed: " << helper.errorMsg());
-    }
-
-   // **** Note InLimit does not have any changeable state
+      Defs defs;
+      suite_ptr suite = defs.add_suite("s1");
+      task_ptr t1 = suite->add_task("t1");
+      task_ptr t2 = suite->add_task("t2");
+      task_ptr t3 = suite->add_task("t3");
+      task_ptr t4 = suite->add_task("t4");
+      Limit limit("limit",10);
+      limit.increment(1,t1->absNodePath());
+      limit.increment(1,t2->absNodePath());
+      limit.increment(1,t3->absNodePath());
+      limit.increment(1,t4->absNodePath());
+      suite->addLimit(limit);
+      //       PrintStyle style(PrintStyle::MIGRATE); std::cout << defs;
+      BOOST_CHECK_MESSAGE( helper.test_state_persist_and_reload_with_checkpt(defs), "Limit state: failed: " << helper.errorMsg());
+   }
+   {
+      Defs defs;
+      suite_ptr suite = defs.add_suite("s1");
+      task_ptr t1 = suite->add_task("t1");
+      InLimit inlimit1("limit","/s1",10,true);   // limit has state when 'limit this node only' is set
+      inlimit1.set_incremented(true);            // the state
+      InLimit inlimit2("limit2","/s1",10);
+      t1->addInLimit(inlimit1);
+      t1->addInLimit(inlimit2);
 
+      //       PrintStyle style(PrintStyle::MIGRATE); std::cout << defs;
+      BOOST_CHECK_MESSAGE( helper.test_state_persist_and_reload_with_checkpt(defs), "InLimit state: failed: " << helper.errorMsg());
+   }
    {
         Defs defs;
         suite_ptr suite = defs.add_suite("s1");
@@ -243,9 +265,9 @@ BOOST_AUTO_TEST_CASE( test_state_node_attributes )
         task_ptr t6 = suite->add_task("t6");
 
         std::vector<std::string> stringList; stringList.reserve(3);
-        stringList.push_back("20130101");
-        stringList.push_back("20130201");
-        stringList.push_back("20130301");
+        stringList.emplace_back("20130101");
+        stringList.emplace_back("20130201");
+        stringList.emplace_back("20130301");
 
         RepeatEnumerated rep("AEnum",stringList);
         rep.increment();
@@ -268,6 +290,10 @@ BOOST_AUTO_TEST_CASE( test_state_node_attributes )
         rep5.increment();
         t5->addRepeat( rep5 );
 
+        RepeatDateList rep6("YMD",{20090916,20090916});
+        rep6.increment();
+        t6->addRepeat( rep6 );
+
         //      PrintStyle style(PrintStyle::MIGRATE); std::cout << defs;
         BOOST_CHECK_MESSAGE( helper.test_state_persist_and_reload_with_checkpt(defs), "Repeat state: failed: " << helper.errorMsg());
      }
@@ -406,9 +432,9 @@ BOOST_AUTO_TEST_CASE( test_server_state )
    {
       Defs defs;
       std::vector<Variable> vec;
-      vec.push_back(Variable("name","value1"));
-      vec.push_back(Variable("name2","val with 'spaces' "));
-      vec.push_back(Variable("name3",""));
+      vec.emplace_back("name","value1");
+      vec.emplace_back("name2","val with 'spaces' ");
+      vec.emplace_back("name3","");
       defs.set_server().set_user_variables(vec);
       //      PrintStyle style(PrintStyle::MIGRATE); std::cout << defs;
       BOOST_CHECK_MESSAGE( helper.test_state_persist_and_reload_with_checkpt(defs), "Set server variables failed " << helper.errorMsg());
