@@ -22,34 +22,22 @@
 // Hence externs are *ONLY* used on the client side.
 /////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
 
-#include <ostream>
+#include <iosfwd>
 #include <set>
-
-#include <boost/noncopyable.hpp>
-#include "boost/date_time/posix_time/posix_time_types.hpp"
-#include <boost/serialization/serialization.hpp>
-#include <boost/serialization/utility.hpp>
-#include <boost/serialization/vector.hpp>         // no need to include <vector>
-#include <boost/serialization/deque.hpp>          // no need to include <deque>
-#include <boost/serialization/map.hpp>            // no need to include <map>
-#include <boost/serialization/string.hpp>         // no need to include <string>
-#include <boost/serialization/shared_ptr.hpp>     // no need to include shared_ptr
-#include <boost/foreach.hpp> // used so often just placed here for convenience
+#include <vector>
+#include <unordered_map>
 
 #include "ServerState.hpp"
 #include "ClientSuiteMgr.hpp"
 #include "NState.hpp"
 #include "NOrder.hpp"
 #include "NodeFwd.hpp"
-#include "JobCreationCtrl.hpp"
-#include "Suite.hpp"
-#include "CheckPt.hpp"
-#include "Archive.hpp"
 #include "Attr.hpp"
 #include "PrintStyle.hpp"
+#include "Flag.hpp"
+#include "Aspect.hpp"
 
-class Limit;
-class AbstractObserver;
+namespace cereal { class access; }
 namespace ecf { class NodeTreeVisitor; class CalendarUpdateParams; } // forward declare
 
 class Defs {
@@ -63,9 +51,16 @@ public:
 
    ~Defs();
 
-   void copy_defs_state_only(defs_ptr defs); // needed when creating defs for client handles
+   void copy_defs_state_only(const defs_ptr& defs); // needed when creating defs for client handles
    bool operator==(const Defs& rhs) const;
-   std::ostream& print(std::ostream&) const ;
+   void print(std::string&) const;
+   std::string print(PrintStyle::Type_t t = PrintStyle::MIGRATE) const;
+
+   /// Handle migration of checkpoint 4->5, and 5.0-5.4.0 -> 5.5
+   /// This essentially update date data member of the Day attribute
+   /// Can be removed when ecflow 5 is used everywhere.
+   void handle_migration();
+
 
    /// State related functions:
    /// Defs acts like the root node.
@@ -114,12 +109,19 @@ public:
    /// dependencies are free'd
    void updateCalendar( const ecf::CalendarUpdateParams &);
 
-   /// Used by simulator
-   void update_calendar(suite_ptr suite, const ecf::CalendarUpdateParams& cal_update_params ); // used by simulator
+   /// Used by simulator & catch_up_to_real_time
+   void update_calendar(Suite* suite, const ecf::CalendarUpdateParams& cal_update_params ); // used by simulator
 
    /// returns the number of times the calendar has been updated. For DEBUG
    unsigned int updateCalendarCount() const { return updateCalendarCount_;}
 
+   /// When the server is terminated and then restarted, Allow time attributes to catch up.
+   /// This may expire time attributes allowing the task to run
+   /// We only update suite that have time attributes and where the calendars that are 1 hour out of sync with real time.
+   /// It can be quite expensive to update time attributes 1 minute at a time, hence we chose 1 hour.
+   /// Return true if any suite where updated
+   bool catch_up_to_real_time();
+
    // Implements visitor pattern
    void accept(ecf::NodeTreeVisitor&); // Node Tree structure does the traversal
    void acceptVisitTraversor(ecf::NodeTreeVisitor& v); // visitor does traversal
@@ -136,7 +138,7 @@ public:
 
    /// Add a suite to the definition, will throw std::runtime_error if duplicate
    suite_ptr add_suite(const std::string& name);
-   void addSuite(suite_ptr,size_t position = std::numeric_limits<std::size_t>::max());
+   void addSuite(const suite_ptr&,size_t position = std::numeric_limits<std::size_t>::max());
    size_t child_position(const Node*) const;
 
    /// Externs refer to Nodes or, variable, events, meter, repeat, or generated variable
@@ -160,7 +162,7 @@ public:
    /// Flag the chosen suite, so that it can resolve dependencies. This also
    /// changes the state of all children to QUEUED/defstatus and initialise the node attributes
    /// Once a suite is begun, it stays in that state
-   void beginSuite(suite_ptr);
+   void beginSuite(const suite_ptr&);
 
    /// Enables all suites to resolve dependencies
    /// Once a suite is begun, it stays in that state
@@ -170,7 +172,7 @@ public:
    void reset_begin();
 
    /// throws runtime_error if suite cant begin
-   void check_suite_can_begin(suite_ptr) const;
+   void check_suite_can_begin(const suite_ptr&) const;
 
    /// Will requeue all suites. Current used in test only
    void requeue();
@@ -180,7 +182,8 @@ public:
 
    /// recursively sort the attributes
    // expect one attr to be [ event | meter | label | limits | variable ]
-   void sort_attributes(ecf::Attr::Type attr, bool recursive = true);
+   // if recursive is true, ignore nodes on no_sort list
+   void sort_attributes(ecf::Attr::Type attr, bool recursive = true, const std::vector<std::string>& no_sort = std::vector<std::string>());
 
    /// This function is called when ALL definition has been parsed successfully
    /// Client Side:: The client side has externs, hence any references to node paths
@@ -255,6 +258,7 @@ public:
 
    // Order the suite
    void order(Node* immediateChild, NOrder::Order);
+   void move_peer(Node* src, Node* dest);
 
    /// determines why the node is not running.
    void top_down_why(std::vector<std::string>& theReasonWhy,bool html_tags = false) const;
@@ -262,9 +266,8 @@ public:
 
    /// Function to save/restore the defs as a checkpoint file. Can throw exception
    /// If the Defs file has content, this is deleted first, i.e. suites, externs, Can throw an exception
-   void boost_save_as_checkpt(const std::string& fileName,ecf::Archive::Type = ecf::Archive::default_archive()) const;
-   void boost_save_as_filename(const std::string& fileName,ecf::Archive::Type = ecf::Archive::default_archive()) const; // used in test only
-   void boost_restore_from_checkpt(const std::string& fileName,ecf::Archive::Type = ecf::Archive::default_archive());
+   void cereal_save_as_checkpt(const std::string& fileName) const;
+   void cereal_restore_from_checkpt(const std::string& fileName);
 
    // defs format
    void save_as_checkpt(const std::string& fileName) const;
@@ -283,10 +286,12 @@ public:
    const ecf::Flag& get_flag() const { return flag_;}
 
    void add_edit_history(const std::string& path, const std::string& request);
-   const std::deque<std::string>& get_edit_history(const std::string& path) const;
+   void clear_edit_history();
+   const std::vector<std::string>& get_edit_history(const std::string& path) const;
+   const std::unordered_map<std::string, std::vector<std::string> >&  get_edit_history() const { return edit_history_;}
    void save_edit_history(bool f) const { save_edit_history_ = f ;}
-   static const std::deque<std::string>& empty_edit_history();
-   static size_t max_edit_history_size_per_node() { return 20; }
+   static const std::vector<std::string>& empty_edit_history();
+   constexpr static size_t max_edit_history_size_per_node() { return 10; }
 
    /// Memento functions:
    void collateChanges(unsigned int client_handle,DefsDelta&) const;
@@ -322,17 +327,19 @@ public:
    void read_history(const std::string& line,const std::vector<std::string>& lineTokens);
    bool compare_edit_history(const Defs&) const;
    bool compare_change_no(const Defs&) const;
+
+   std::string stats() const; // dump out definition stats
 private:
    void do_generate_scripts( const std::map<std::string,std::string>& override) const;
-   std::string write_state() const;
+   void write_state(std::string&) const;
    void collate_defs_changes_only(DefsDelta&) const;
    void setupDefaultEnv();
-   void add_suite_only(suite_ptr, size_t position);
+   void add_suite_only(const suite_ptr&, size_t position);
 
    /// Removes the suite, from defs returned as suite_ptr, asserts if suite does not exist
    suite_ptr removeSuite(suite_ptr);
    node_ptr removeChild(Node*);
-   bool addChild( node_ptr, size_t position = std::numeric_limits<std::size_t>::max());
+   bool addChild( const node_ptr&, size_t position = std::numeric_limits<std::size_t>::max());
    friend class Node;
 
    /// For use by python interface,
@@ -349,22 +356,22 @@ private:
 
 private:
    /// Note: restoring from a check point file will reset, defs state and modify numbers
-   unsigned int    state_change_no_;            // persisted since passed to client, however side effect, is it will be in checkpoint file
-   unsigned int    modify_change_no_;           // persisted since passed to client, however side effect, is it will be in checkpoint file
-   unsigned int    updateCalendarCount_;
-   unsigned int    order_state_change_no_;      // *NOT* persisted
-   NState          state_;                      // state & change_no, i,e attribute changed
-   ServerState     server_;
+   mutable size_t print_cache_{0};            // NOT persisted
+   unsigned int   state_change_no_{0};        // persisted since passed to client, however side effect, is it will be in checkpoint file
+   unsigned int   modify_change_no_{0};       // persisted since passed to client, however side effect, is it will be in checkpoint file
+   unsigned int   updateCalendarCount_{0};
+   unsigned int   order_state_change_no_{0};  // *NOT* persisted
+   NState         state_;                     // state & change_no, i,e attribute changed
+   ServerState    server_;
    std::vector<suite_ptr> suiteVec_;
-   std::map<std::string, std::deque<std::string> > edit_history_;      // path,request
-   mutable bool                                    save_edit_history_; // NOT persisted
+   std::unordered_map<std::string, std::vector<std::string> > edit_history_;  // path,request
    ecf::Flag                    flag_;
 
-   ClientSuiteMgr client_suite_mgr_;                    // NOT persisted
+   ClientSuiteMgr client_suite_mgr_{this};     // NOT persisted
 
    /// Externs are *NEVER* loaded in the server, since they can be computed and
    /// save on network band with, and check point file size.
-   std::set<std::string> externs_;                      // NOT persisted
+   std::set<std::string> externs_;             // NOT persisted
 
    friend class SaveEditHistoryWhenCheckPointing;
 
@@ -372,8 +379,10 @@ private:
    /// Observer notifications Start. Allow client to query if they are in syncing with server
    void notify_start() { in_notification_ = true;  }
    void notify_end()   { in_notification_ = false; }
-   bool in_notification_;
    std::vector<AbstractObserver*> observers_;
+   mutable bool save_edit_history_{false};   // NOT persisted
+   bool in_notification_{false};
+
    friend class ChangeStartNotification;
    void notify_delete();
 public:
@@ -385,40 +394,9 @@ public:
    bool in_notification() const { return in_notification_;}
 
 private:
-   friend class boost::serialization::access;
+   friend class cereal::access;
    template<class Archive>
-   void serialize(Archive & ar, const unsigned int /*version*/)
-   {
-      ar & state_change_no_;
-      ar & modify_change_no_;
-      ar & updateCalendarCount_;
-      ar & state_;
-      ar & server_;
-      ar & suiteVec_;
-      ar & flag_;
-
-      // only save the edit history when check pointing.
-      if (Archive::is_saving::value) {
-         if (save_edit_history_) {
-            ar & edit_history_;
-            save_edit_history_ = false; // reset
-         }
-         else {
-            std::map<std::string, std::deque<std::string> > empty_edit_history;
-            ar & empty_edit_history;
-         }
-      }
-      else {
-         ar & edit_history_;
-      }
-
-      if (Archive::is_loading::value) {
-         size_t vec_size = suiteVec_.size();
-         for(size_t i = 0; i < vec_size; i++) {
-            suiteVec_[i]->set_defs(this);
-         }
-      }
-   }
+   void serialize(Archive & ar, std::uint32_t const version );
 };
 
 std::ostream& operator<<(std::ostream& os, const Defs*);
@@ -426,32 +404,31 @@ std::ostream& operator<<(std::ostream& os, const Defs&);
 
 // =====================================================================
 // This class is used to read the History
-class DefsHistoryParser : private boost::noncopyable {
+class DefsHistoryParser {
 public:
+   DefsHistoryParser(const DefsHistoryParser&) = delete;
+   const DefsHistoryParser& operator=(const DefsHistoryParser&) = delete;
    DefsHistoryParser();
 
    void parse(const std::string& line);
    const std::vector<std::string>& parsed_messages() const { return parsed_messages_;}
 
 private:
-
    std::string::size_type find_log(const std::string& line, std::string::size_type pos) const;
-
-private:
-   std::vector<std::string> log_types_;
    std::vector<std::string> parsed_messages_;
 };
 
 
-
 // Start notification. End notification automatically signalled, Even if exception raised.
-class ChangeStartNotification : private boost::noncopyable {
+class ChangeStartNotification {
 public:
    explicit ChangeStartNotification(defs_ptr defs) : defs_ptr_(defs) { defs_ptr_->notify_start();}
    ~ChangeStartNotification() {  defs_ptr_->notify_end();}
+private:
+  ChangeStartNotification(const ChangeStartNotification&) = delete;
+  const ChangeStartNotification& operator=(const ChangeStartNotification&) = delete;
 private:
    defs_ptr defs_ptr_;
 };
 
-
 #endif /* DEFS_HPP_ */
