@@ -17,14 +17,16 @@
 //============================================================================
 
 #include <string>
-#include <memory> // for auto_ptr
-#include <boost/noncopyable.hpp>
+#include <memory> // for unique_ptr
 #include "ExprAst.hpp"
 
 
 /// This class will parse a expression and create the abstract syntax tree
 /// It will own the AST unless specifically released calling ast();
-class ExprParser : private boost::noncopyable {
+class ExprParser {
+private:
+  ExprParser(const ExprParser&) = delete;
+  const ExprParser& operator=(const ExprParser&) = delete;
 public:
    explicit ExprParser(const std::string& expression);
 
@@ -33,20 +35,23 @@ public:
 	bool doParse(std::string& errorMsg);
 
 	/// return the Abstract syntax tree, and release memory
-	std::auto_ptr<AstTop> ast() { return ast_;}
+	std::unique_ptr<AstTop> ast() { return std::move(ast_);}
 
 	/// return the Abstract syntax tree, without release memory
 	AstTop* getAst() const { return ast_.get();}
 
 private:
-	std::auto_ptr<AstTop> ast_;
+	std::unique_ptr<AstTop> ast_;
 	std::string           expr_;
 };
 
 // This class was added to mitigate the slowness of the boost classic spirit parser
 // we will recognise very simple expression, and bypass spirit. Very limited
 // But the simple expression do form a very large subset
-class SimpleExprParser : private boost::noncopyable {
+class SimpleExprParser {
+private:
+  SimpleExprParser(const SimpleExprParser&) = delete;
+  const SimpleExprParser& operator=(const SimpleExprParser&) = delete;
 public:
    explicit SimpleExprParser(const std::string& expression) : expr_(expression) {}
 
@@ -54,11 +59,11 @@ public:
    bool doParse();
 
    /// return the Abstract syntax tree, and release memory
-   std::auto_ptr<AstTop> ast() { return ast_;}
+   std::unique_ptr<AstTop> ast() { return std::move(ast_);}
 
 private:
    const std::string& expr_;
-   std::auto_ptr<AstTop> ast_;
+   std::unique_ptr<AstTop> ast_;
 };
 
 #endif
