@@ -10,12 +10,11 @@
 // granted to it by virtue of its status as an intergovernmental organisation
 // nor does it submit to any jurisdiction.
 /////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
+#include <stdexcept>
 #include <iostream>
-#include <fstream>
-#include <stdlib.h>
+#include <cstdlib> // for getenv
 
 #include "boost/filesystem/operations.hpp"
-#include "boost/filesystem/path.hpp"
 #include <boost/test/unit_test.hpp>
 
 #include "Defs.hpp"
@@ -24,13 +23,12 @@
 #include "Task.hpp"
 #include "EcfFile.hpp"
 #include "JobsParam.hpp"
-#include "Jobs.hpp"
 #include "System.hpp"
 #include "File.hpp"
 #include "Str.hpp"
 #include "Ecf.hpp"
-#include "PrintStyle.hpp"
 #include "Pid.hpp"
+//#include "PrintStyle.hpp"
 
 using namespace std;
 using namespace ecf;
@@ -815,12 +813,11 @@ BOOST_AUTO_TEST_CASE( test_ecf_file )
    // Create a defs file, where the task name mirrors the sms files in the given directory
    task_ptr task_t1 = Task::create( "t1" );
    suite_ptr suite = Suite::create( Pid::unique_name("test_ecf_file") );
-   std::pair<std::string,std::string> p;
    Defs theDefs; {
       suite->addVariable( Variable( Str::ECF_INCLUDE(), "$ECF_HOME/includes" ) );
       suite->addVariable( Variable( "SLEEPTIME", "1" ) );
       suite->addVariable( Variable( "ECF_CLIENT_EXE_PATH",  "a/made/up/path" ) );
-      BOOST_FOREACH(p,expected_used_variables) { task_t1->addVariable( Variable( p.first,  p.second) );}
+      for(const std::pair<std::string,std::string>& p : expected_used_variables) { task_t1->addVariable( Variable( p.first,  p.second) );}
       suite->addTask( task_t1 );
       theDefs.addSuite( suite );
    }
@@ -844,8 +841,7 @@ BOOST_AUTO_TEST_CASE( test_ecf_file )
    comment_body +=       " end.\n";
    string comment_tail ="%end\n\n";
    string ecf_body; {
-      std::pair<std::string,std::string> p;
-      BOOST_FOREACH(p,expected_used_variables) {  ecf_body += Ecf::MICRO() + p.first + Ecf::MICRO() + "\n";}
+      for(std::pair<std::string,std::string> p:expected_used_variables) {  ecf_body += Ecf::MICRO() + p.first + Ecf::MICRO() + "\n";}
       ecf_body +="%VAR3:substitute_var%\n";
    }
    string tail = "\n%include <tail.h>\n# ===================================";
@@ -899,7 +895,7 @@ BOOST_AUTO_TEST_CASE( test_ecf_file )
    ecfFile.edit_used_variables(file_with_used_variables);
    //    std::cout << "file_with_used_variables:----------------------------------------------------------------\n" << file_with_used_variables << "\n";
    BOOST_CHECK_MESSAGE(file_with_used_variables.find("%comment") == 0, "Expected to find variable %comment on the very first line: but found at: " << file_with_used_variables.find("%comment"));
-   BOOST_FOREACH(p,expected_used_variables) {
+   for(std::pair<std::string,std::string> p:expected_used_variables) {
       BOOST_CHECK_MESSAGE(file_with_used_variables.find(p.first) != string::npos, "Expected to find variable" << p.first);
    }
 
@@ -908,11 +904,11 @@ BOOST_AUTO_TEST_CASE( test_ecf_file )
    Str::split(file_with_used_variables,script_lines,"\n"); //  will ignore empty lines, but will do for this case
    NameValueMap extracted_used_variables;
    EcfFile::extract_used_variables( extracted_used_variables, script_lines );
-   BOOST_FOREACH(p,expected_used_variables) {
+   for(std::pair<std::string,std::string> p:expected_used_variables) {
       BOOST_CHECK_MESSAGE( extracted_used_variables.find(p.first) != extracted_used_variables.end()," expected to find variable " << p.first << " in the extracted variables\n");
    }
-   //    cout << "Expected:----\n"; BOOST_FOREACH(p,expected_used_variables) { cout << p.first << " " << p.second << "\n";}
-   //    cout << "Actual:------\n"; BOOST_FOREACH(p,extracted_used_variables) { cout << p.first << " " << p.second << "\n";}
+   //    cout << "Expected:----\n"; for(p:expected_used_variables) { cout << p.first << " " << p.second << "\n";}
+   //    cout << "Actual:------\n"; for(p:extracted_used_variables) { cout << p.first << " " << p.second << "\n";}
 
 
    /// Test pre-processing
@@ -941,7 +937,6 @@ BOOST_AUTO_TEST_CASE( test_ecf_file )
     // Test the contents of the job file.
 //    cout << "\n" << job_file_contents << "\n";
     BOOST_CHECK_MESSAGE(job_file_contents.find("%ECF_PORT%") == string::npos,"Expected variables to be substituted:");
-    BOOST_CHECK_MESSAGE(job_file_contents.find("%ECF_NODE%") == string::npos,"Expected variables to be substituted:");
     BOOST_CHECK_MESSAGE(job_file_contents.find("%ECF_HOST%") == string::npos,"Expected variables to be substituted:");
     BOOST_CHECK_MESSAGE(job_file_contents.find("%ECF_NAME%") == string::npos,"Expected variables to be substituted:");
     BOOST_CHECK_MESSAGE(job_file_contents.find("%ECF_PASS%") == string::npos,"Expected variables to be substituted");
@@ -1468,4 +1463,73 @@ BOOST_AUTO_TEST_CASE( test_include_with_variable_not_defined )
    basic_test_template("test_include_with_variable_not_defined",ecf_file,expected_job_file_contents,""/*ecf_micro*/,false/*expect failure*/);
 }
 
+BOOST_AUTO_TEST_CASE( test_nopp_preserves_contents )
+{
+   string ecf_file;
+   ecf_file += "%include <simple_head.h>\n";
+   ecf_file += "#body\n";
+   ecf_file += "%nopp\n";
+   ecf_file += "%comment\n";
+   ecf_file += "--comment\n";
+   ecf_file += "%end\n";
+   ecf_file += "%manual\n";
+   ecf_file += "--manual\n";
+   ecf_file += "%end\n";
+   ecf_file += "%end\n";
+   ecf_file += "%include <simple_tail.h>\n";
+
+   std::string expected_job_file_contents = "#head.h\n#body\n%comment\n--comment\n%end\n%manual\n--manual\n%end\n#tail.h";
+
+   basic_test_template("test_nopp_preserves_contents",ecf_file,expected_job_file_contents);
+}
+
+BOOST_AUTO_TEST_CASE( test_nopp_preserves_contents2 )
+{
+   string ecf_file;
+   ecf_file += "%include <simple_head.h>\n";
+   ecf_file += "#body\n";
+   ecf_file += "%nopp\n";
+   ecf_file += "%ecfmicro @\n";
+   ecf_file += "%end\n";
+   ecf_file += "%include <simple_tail.h>\n";
+
+   std::string expected_job_file_contents = "#head.h\n#body\n%ecfmicro @\n#tail.h";
+
+   basic_test_template("test_nopp_preserves_contents2",ecf_file,expected_job_file_contents);
+}
+
+BOOST_AUTO_TEST_CASE( test_comment_and_manual_removal )
+{
+   string ecf_file;
+   ecf_file += "%include <simple_head.h>\n";
+   ecf_file += "#body\n";
+   ecf_file += "%comment\n";
+   ecf_file += "--comment\n";
+   ecf_file += "%end\n";
+   ecf_file += "%manual\n";
+   ecf_file += "--manual\n";
+   ecf_file += "%end\n";
+   ecf_file += "%include <simple_tail.h>\n";
+
+   std::string expected_job_file_contents = "#head.h\n#body\n#tail.h";
+
+   basic_test_template("test_comment_and_manual_removal",ecf_file,expected_job_file_contents);
+}
+
+BOOST_AUTO_TEST_CASE( test_unterminated_manual )
+{
+   string ecf_file;
+   ecf_file += "%include <simple_head.h>\n";
+   ecf_file += "#body\n";
+   ecf_file += "%comment\n";
+   ecf_file += "--comment\n";
+   ecf_file += "%end\n";
+   ecf_file += "%manual\n";
+   ecf_file += "%include <simple_tail.h>\n";
+
+   std::string expected_job_file_contents = "#head.h\n#body%manual\n#tail.h";
+
+   basic_test_template("test_unterminated_manual",ecf_file,expected_job_file_contents,"",false);
+}
+
 BOOST_AUTO_TEST_SUITE_END()
