@@ -15,13 +15,8 @@
 #include <string>
 #include <map>
 #include <iostream>
-#include <fstream>
 
 #include <boost/test/unit_test.hpp>
-#include <boost/foreach.hpp>
-#include <boost/date_time/posix_time/posix_time_types.hpp>
-#include <boost/date_time/posix_time/conversion.hpp>
-#include <boost/date_time/posix_time/time_formatters.hpp>  // requires boost date and time lib, for to_simple_string
 
 #include "ExprParser.hpp"
 #include "ExprAst.hpp"
@@ -46,69 +41,69 @@ BOOST_AUTO_TEST_CASE( test_expression_parser_basic )
    // Note: we can use NOT,eq,ne,le,ge, or brackets
    //       we can't use a:event_name  ==> a:event_name == set
    std::vector<std::string> vec;
-   vec.push_back("a == complete");
-   vec.push_back("a != complete");
-   vec.push_back("a:value == 10");
-   vec.push_back("a:value != 10");
-   vec.push_back("a:value >= 10");
-   vec.push_back("a:value <= 10");
-   vec.push_back("a:value > 10");
-   vec.push_back("a:value < 10");
-   vec.push_back("1 == 1");
-   vec.push_back("1 == 0");
-   vec.push_back("a:event_name == set");
-   vec.push_back("a:event_name != set");
-   vec.push_back("a:event_name == clear");
-   vec.push_back("a:event_name != clear");
-   vec.push_back("../a/b:eventname == set");
-   vec.push_back("../a/b:eventname == clear");
-   vec.push_back("../a/b:eventname != clear");
-   vec.push_back("../a:event_name >= 10");
-   vec.push_back("a == unknown and b != complete");
-   vec.push_back("a == unknown or b != complete");
-   vec.push_back("a == complete and b == complete or c == complete");
-   vec.push_back("! a == unknown");
-   vec.push_back("/mc/main:YMD <= /mc/main/ref:MC_STOP");
-   vec.push_back("! ../../../prod2diss/operation_is_late:yes == set or ! a == complete");
-   vec.push_back("./a:YMD - ./b:YMD < 5");
-   vec.push_back("./a:YMD + ./b:YMD < 5");
-   vec.push_back("./a:YMD / ./b:YMD < 5");
-   vec.push_back("./a:YMD * ./b:YMD < 5");
-   vec.push_back("./a:YMD % ./b:YMD < 5");
-   vec.push_back("inigroup:YMD == ! 1");
-   vec.push_back("inigroup:YMD == ! 0");
-   vec.push_back("comp == complete and notready == complete");  // ECFLOW-493
-   vec.push_back("comp == complete and not ready == complete");
-   vec.push_back("comp == complete and ! ready == complete");   // we now store the not from the parse, for test comparison
-   vec.push_back("comp == complete and ~ ready == complete");
+   vec.emplace_back("a == complete");
+   vec.emplace_back("a != complete");
+   vec.emplace_back("a:value == 10");
+   vec.emplace_back("a:value != 10");
+   vec.emplace_back("a:value >= 10");
+   vec.emplace_back("a:value <= 10");
+   vec.emplace_back("a:value > 10");
+   vec.emplace_back("a:value < 10");
+   vec.emplace_back("1 == 1");
+   vec.emplace_back("1 == 0");
+   vec.emplace_back("a:event_name == set");
+   vec.emplace_back("a:event_name != set");
+   vec.emplace_back("a:event_name == clear");
+   vec.emplace_back("a:event_name != clear");
+   vec.emplace_back("../a/b:eventname == set");
+   vec.emplace_back("../a/b:eventname == clear");
+   vec.emplace_back("../a/b:eventname != clear");
+   vec.emplace_back("../a:event_name >= 10");
+   vec.emplace_back("a == unknown and b != complete");
+   vec.emplace_back("a == unknown or b != complete");
+   vec.emplace_back("a == complete and b == complete or c == complete");
+   vec.emplace_back("! a == unknown");
+   vec.emplace_back("/mc/main:YMD <= /mc/main/ref:MC_STOP");
+   vec.emplace_back("! ../../../prod2diss/operation_is_late:yes == set or ! a == complete");
+   vec.emplace_back("./a:YMD - ./b:YMD < 5");
+   vec.emplace_back("./a:YMD + ./b:YMD < 5");
+   vec.emplace_back("./a:YMD / ./b:YMD < 5");
+   vec.emplace_back("./a:YMD * ./b:YMD < 5");
+   vec.emplace_back("./a:YMD % ./b:YMD < 5");
+   vec.emplace_back("inigroup:YMD == ! 1");
+   vec.emplace_back("inigroup:YMD == ! 0");
+   vec.emplace_back("comp == complete and notready == complete");  // ECFLOW-493
+   vec.emplace_back("comp == complete and not ready == complete");
+   vec.emplace_back("comp == complete and ! ready == complete");   // we now store the not from the parse, for test comparison
+   vec.emplace_back("comp == complete and ~ ready == complete");
 
-   vec.push_back(":VAR == 1");
-   vec.push_back(":VAR == /mc/main/ref:MC_STOP");
-   vec.push_back(":YMD - :YMD < 5");
-   vec.push_back(":YMD + :YMD < 5");
-   vec.push_back(":YMD / :YMD < 5");
-   vec.push_back(":YMD * :YMD < 5");
-   vec.push_back(":YMD % :YMD < 5");
+   vec.emplace_back(":VAR == 1");
+   vec.emplace_back(":VAR == /mc/main/ref:MC_STOP");
+   vec.emplace_back(":YMD - :YMD < 5");
+   vec.emplace_back(":YMD + :YMD < 5");
+   vec.emplace_back(":YMD / :YMD < 5");
+   vec.emplace_back(":YMD * :YMD < 5");
+   vec.emplace_back(":YMD % :YMD < 5");
 
-   for(size_t i = 0; i < vec.size(); i++) {
+   for(const auto & i : vec) {
 
-      PartExpression part(vec[i]);
+      PartExpression part(i);
       string parseErrorMsg;
-      std::auto_ptr<AstTop> ast = part.parseExpressions( parseErrorMsg );
-      BOOST_REQUIRE_MESSAGE(ast.get(),"Failed to parse\n" << vec[i] << "  " << parseErrorMsg);
+      std::unique_ptr<AstTop> ast = part.parseExpressions( parseErrorMsg );
+      BOOST_REQUIRE_MESSAGE(ast.get(),"Failed to parse\n" << i << "  " << parseErrorMsg);
 
       std::stringstream s2;
       ast->print_flat(s2);
       std::string ast_expr = s2.str();
-      BOOST_CHECK_MESSAGE(vec[i]==ast_expr," Failed\n'" << vec[i] << "' != '" << ast_expr << "'" );
+      BOOST_CHECK_MESSAGE(i==ast_expr," Failed\n'" << i << "' != '" << ast_expr << "'" );
 
       std::string why; ast->why(why);
       //cout << "why: " << vec[i] << " -> " << why << "\n";
       if (ast->evaluate()) {
-         BOOST_CHECK_MESSAGE(why.empty(),"Expected why to be empty when expression evaluates: " << vec[i]);
+         BOOST_CHECK_MESSAGE(why.empty(),"Expected why to be empty when expression evaluates: " << i);
       }
       else {
-         BOOST_CHECK_MESSAGE(!why.empty(),"When ast does not evaluate we expect to find why: " << vec[i]);
+         BOOST_CHECK_MESSAGE(!why.empty(),"When ast does not evaluate we expect to find why: " << i);
       }
    }
 }
@@ -122,37 +117,37 @@ BOOST_AUTO_TEST_CASE( test_expression_parser_basic_with_brackets )
    // Note: we can use NOT,eq,ne,le,ge,
    //       we can't use a:event_name  ==> a:event_name == set
    std::vector<std::string> vec;
-   vec.push_back("((a == complete) and (b == complete))");
-   vec.push_back("(((a == complete) or (b == complete)) and (c == complete))");
-   vec.push_back("((a == complete) and ((b == complete) or (nodepath:eventname == set)))");
-   vec.push_back("((a == complete) and ((b == complete) or ((a == complete) and (b == complete))))");
-   vec.push_back("! ((a == unknown))");
-   vec.push_back("((t:step + 20) >= (t:step1 - 20))");
-   vec.push_back("(((/o/main/12/an/slwet == complete) and ((/o/main/12/an/4dvar/ifstraj:finalwave == set) or (/o/main/12/an/4dvar == complete))) or (/o/main/12/an == complete))");
-   vec.push_back("((obs:YMD <= (main:YMD + 1)) and ((../make/setup == complete) and (obs:YMD <= /o/lag:YMD)))");
-   vec.push_back("(((stage == complete) or (./stage:YMD > ./retrieve:YMD)) and ((./retrieve:YMD - ./load:YMD) < 5))");
-   vec.push_back("((./a:YMD - ./b:YMD) < 5)");
-   vec.push_back("((:YMD + :YMD) < 5)");
+   vec.emplace_back("((a == complete) and (b == complete))");
+   vec.emplace_back("(((a == complete) or (b == complete)) and (c == complete))");
+   vec.emplace_back("((a == complete) and ((b == complete) or (nodepath:eventname == set)))");
+   vec.emplace_back("((a == complete) and ((b == complete) or ((a == complete) and (b == complete))))");
+   vec.emplace_back("! ((a == unknown))");
+   vec.emplace_back("((t:step + 20) >= (t:step1 - 20))");
+   vec.emplace_back("(((/o/main/12/an/slwet == complete) and ((/o/main/12/an/4dvar/ifstraj:finalwave == set) or (/o/main/12/an/4dvar == complete))) or (/o/main/12/an == complete))");
+   vec.emplace_back("((obs:YMD <= (main:YMD + 1)) and ((../make/setup == complete) and (obs:YMD <= /o/lag:YMD)))");
+   vec.emplace_back("(((stage == complete) or (./stage:YMD > ./retrieve:YMD)) and ((./retrieve:YMD - ./load:YMD) < 5))");
+   vec.emplace_back("((./a:YMD - ./b:YMD) < 5)");
+   vec.emplace_back("((:YMD + :YMD) < 5)");
 
-   for(size_t i = 0; i < vec.size(); i++) {
+   for(const auto & i : vec) {
 
-      PartExpression part(vec[i]);
+      PartExpression part(i);
       string parseErrorMsg;
-      std::auto_ptr<AstTop> ast = part.parseExpressions( parseErrorMsg );
-      BOOST_REQUIRE_MESSAGE(ast.get(),"Failed to parse " << vec[i] << "  " << parseErrorMsg);
+      std::unique_ptr<AstTop> ast = part.parseExpressions( parseErrorMsg );
+      BOOST_REQUIRE_MESSAGE(ast.get(),"Failed to parse " << i << "  " << parseErrorMsg);
 
       std::stringstream s2;
       ast->print_flat(s2,true/*add_brackets*/);
       std::string ast_expr = s2.str();
-      BOOST_CHECK_MESSAGE(vec[i]==ast_expr," Failed '" << vec[i] << "' != '" << ast_expr << "'" );
+      BOOST_CHECK_MESSAGE(i==ast_expr," Failed '" << i << "' != '" << ast_expr << "'" );
 
       std::string why; ast->why(why);
       //cout << "why: " << vec[i] << " -> " << why << "\n";
       if (ast->evaluate()) {
-         BOOST_CHECK_MESSAGE(why.empty(),"Expected why to be empty when expression evaluates: " << vec[i]);
+         BOOST_CHECK_MESSAGE(why.empty(),"Expected why to be empty when expression evaluates: " << i);
       }
       else {
-         BOOST_CHECK_MESSAGE(!why.empty(),"When ast does not evaluate we expect to find why: " << vec[i]);
+         BOOST_CHECK_MESSAGE(!why.empty(),"When ast does not evaluate we expect to find why: " << i);
       }
    }
 }
@@ -361,6 +356,30 @@ BOOST_AUTO_TEST_CASE( test_parser_good_expressions )
    exprMap["(/s/f/t<flag>late or 1)"] = std::make_pair(AstOr::stype(),true);
    exprMap["/<flag>late"] = std::make_pair(AstFlag::stype(),false);
 
+   exprMap["/s/f/t<flag>zombie"] = std::make_pair(AstFlag::stype(),false);
+   exprMap["./s<flag>zombie"] = std::make_pair(AstFlag::stype(),false);
+   exprMap["../s/f/t<flag>zombie"] = std::make_pair(AstFlag::stype(),false);
+   exprMap["/s/f/t<flag>zombie == 0"] = std::make_pair(AstEqual::stype(),true);
+   exprMap["0 == /s/f/t<flag>zombie"] = std::make_pair(AstEqual::stype(),true);
+   exprMap["/s/f/t<flag>zombie and /s/f/t<flag>zombie"] = std::make_pair(AstAnd::stype(),false);
+   exprMap["/s/f/t<flag>zombie or /s/f/t<flag>zombie"] = std::make_pair(AstOr::stype(),false);
+   exprMap["! /s/f/t<flag>zombie and ! /s/f/t<flag>zombie"] = std::make_pair(AstAnd::stype(),true);
+   exprMap["! /s/f/t<flag>zombie"] = std::make_pair(AstNot::stype(),true);
+   exprMap["/s/f/t<flag>zombie + 2 >= 2"] = std::make_pair(AstGreaterEqual::stype(),true);
+   exprMap["(/s/f/t<flag>zombie or 1)"] = std::make_pair(AstOr::stype(),true);
+
+   exprMap["/s/f/t<flag>archived"] = std::make_pair(AstFlag::stype(),false);
+   exprMap["./s<flag>archived"] = std::make_pair(AstFlag::stype(),false);
+   exprMap["../s/f/t<flag>archived"] = std::make_pair(AstFlag::stype(),false);
+   exprMap["/s/f/t<flag>archived == 0"] = std::make_pair(AstEqual::stype(),true);
+   exprMap["0 == /s/f/t<flag>archived"] = std::make_pair(AstEqual::stype(),true);
+   exprMap["/s/f/t<flag>archived and /s/f/t<flag>archived"] = std::make_pair(AstAnd::stype(),false);
+   exprMap["/s/f/t<flag>archived or /s/f/t<flag>archived"] = std::make_pair(AstOr::stype(),false);
+   exprMap["! /s/f/t<flag>archived and ! /s/f/t<flag>archived"] = std::make_pair(AstAnd::stype(),true);
+   exprMap["! /s/f/t<flag>archived"] = std::make_pair(AstNot::stype(),true);
+   exprMap["/s/f/t<flag>archived + 2 >= 2"] = std::make_pair(AstGreaterEqual::stype(),true);
+   exprMap["(/s/f/t<flag>archived or 1)"] = std::make_pair(AstOr::stype(),true);
+
    exprMap[":VAR == 0"] = std::make_pair(AstEqual::stype(),true);
    exprMap[":VAR == 1"] = std::make_pair(AstEqual::stype(),false);
    exprMap[":VAR == /mc/main/ref:MC_STOP"] = std::make_pair(AstEqual::stype(),true);
@@ -371,8 +390,8 @@ BOOST_AUTO_TEST_CASE( test_parser_good_expressions )
 
    int parse_failure = 0;
    int ast_failure = 0;
-   std::pair<string, std::pair<string,bool> > p;
-   BOOST_FOREACH(p, exprMap ) {
+   //std::pair<string, std::pair<string,bool> > p;
+   for(const auto& p: exprMap ) {
 
       //cout << "parsing: " << p.first << "\n";
       ExprParser theExprParser(p.first);
@@ -434,8 +453,7 @@ BOOST_AUTO_TEST_CASE( test_trigger_functions )
 
    int parse_failure = 0;
    int ast_failure = 0;
-   std::pair<string, std::pair<string,bool> > p;
-   BOOST_FOREACH(p, exprMap ) {
+   for(std::pair<string, std::pair<string,bool> > p: exprMap ) {
 
       //cout << "parsing: " << p.first << "\n";
       ExprParser theExprParser(p.first);
@@ -497,9 +515,9 @@ BOOST_AUTO_TEST_CASE( test_date_to_julian_with_repeat_YMD )
     BOOST_CHECK_MESSAGE(theReasonWhy.empty() ,"When all is well expected empty reason vec");
 
     // be more flexible, of vector is returned we should not get: variable-not-found
-    for(size_t i = 0; i < theReasonWhy.size(); i++) {
-       cout << theReasonWhy[i] << "\n";
-       BOOST_CHECK_MESSAGE(theReasonWhy[i].find("variable-not-found") == string::npos,"Variable YMD not found: " << theReasonWhy[i]);
+    for(auto & i : theReasonWhy) {
+       cout << i << "\n";
+       BOOST_CHECK_MESSAGE(i.find("variable-not-found") == string::npos,"Variable YMD not found: " << i);
     }
 }
 
@@ -531,8 +549,7 @@ BOOST_AUTO_TEST_CASE( test_trigger_functions_with_boost_date )
 
    int parse_failure = 0;
    int ast_failure = 0;
-   std::pair<string, std::pair<string,bool> > p;
-   BOOST_FOREACH(p, exprMap ) {
+   for(std::pair<string, std::pair<string,bool> > p: exprMap ) {
 
       //cout << "parsing: " << p.first << "\n";
       ExprParser theExprParser(p.first);
@@ -580,8 +597,8 @@ BOOST_AUTO_TEST_CASE( test_trigger_expression_divide_by_zero )
    exprMap[":YMD % 0 == 0"] = std::make_pair(AstEqual::stype(),true);
    exprMap[":YMD / 0 == 0"] = std::make_pair(AstEqual::stype(),true);
 
-   std::pair<string, std::pair<string,bool> > p;
-   BOOST_FOREACH(p, exprMap ) {
+   //std::pair<string, std::pair<string,bool> > p;
+   for(const auto& p: exprMap ) {
 
       ExprParser theExprParser(p.first);
       std::string errorMsg;
@@ -608,58 +625,59 @@ BOOST_AUTO_TEST_CASE( test_parser_bad_expressions )
 {
    std::cout << "ANode:: ...test_parser_bad_expressions\n";
    vector<string> exprvec;
-   exprvec.push_back("a <= complete");
-   exprvec.push_back("a >= complete");
-   exprvec.push_back("a = complete");
-   exprvec.push_back("a e complete");
-   exprvec.push_back("a=complete");
-   exprvec.push_back("a ! complete");
-   exprvec.push_back("a==complet e");
-   exprvec.push_back("a eq complet e");
-   exprvec.push_back("a::eventname");
-   exprvec.push_back("a:eventname =  set");
-   exprvec.push_back("a:eventname == ");
-   exprvec.push_back("a:eventname !  set");
-   exprvec.push_back("a:eventname ! = set");
-   exprvec.push_back("a:eventname %");
-   exprvec.push_back("a:event <= set");
-   exprvec.push_back("a:event >= set");
-   exprvec.push_back("a:event >= clear");
-   exprvec.push_back("a:event >= fred");
-   exprvec.push_back("a:metername  100");
-   exprvec.push_back(". == complete");
-   exprvec.push_back("/ == complete");
-   exprvec.push_back(". == error");
-   exprvec.push_back("./ == error");
-   exprvec.push_back(".a == error");
-   exprvec.push_back(".a == unknown");
-   exprvec.push_back(".a/. == unknown");
-   exprvec.push_back(".. == unknown");
-   exprvec.push_back(".a/b == queued");
-   exprvec.push_back("./a/b/ == active");
-   exprvec.push_back("..a == complete");
-   exprvec.push_back(".../a == complete");
-   exprvec.push_back("../.../a == complete");
-   exprvec.push_back(".. /a == complete");
-   exprvec.push_back("../.. /a == complete");
-   exprvec.push_back("../../.a == complete");
-   exprvec.push_back("..a/b == aborted");
-   exprvec.push_back("..a/b/c == aborted");
-   exprvec.push_back("a == complete and");
-   exprvec.push_back("a %");
-   exprvec.push_back("(a == complete   b == complete)");
-   exprvec.push_back("a == complete and  b == complete)");
-   exprvec.push_back("(a == complete and  b == complete");
-   exprvec.push_back("(a = complete and b = complete or c = complete)");
-   exprvec.push_back("(a erro complete and b == complete) or nodepath:eventname");
-   exprvec.push_back("(a == complete and b == complete or (a == complete and b == complete)");
+   exprvec.emplace_back("/s/f/t<flag>restored");
+   exprvec.emplace_back("a <= complete");
+   exprvec.emplace_back("a >= complete");
+   exprvec.emplace_back("a = complete");
+   exprvec.emplace_back("a e complete");
+   exprvec.emplace_back("a=complete");
+   exprvec.emplace_back("a ! complete");
+   exprvec.emplace_back("a==complet e");
+   exprvec.emplace_back("a eq complet e");
+   exprvec.emplace_back("a::eventname");
+   exprvec.emplace_back("a:eventname =  set");
+   exprvec.emplace_back("a:eventname == ");
+   exprvec.emplace_back("a:eventname !  set");
+   exprvec.emplace_back("a:eventname ! = set");
+   exprvec.emplace_back("a:eventname %");
+   exprvec.emplace_back("a:event <= set");
+   exprvec.emplace_back("a:event >= set");
+   exprvec.emplace_back("a:event >= clear");
+   exprvec.emplace_back("a:event >= fred");
+   exprvec.emplace_back("a:metername  100");
+   exprvec.emplace_back(". == complete");
+   exprvec.emplace_back("/ == complete");
+   exprvec.emplace_back(". == error");
+   exprvec.emplace_back("./ == error");
+   exprvec.emplace_back(".a == error");
+   exprvec.emplace_back(".a == unknown");
+   exprvec.emplace_back(".a/. == unknown");
+   exprvec.emplace_back(".. == unknown");
+   exprvec.emplace_back(".a/b == queued");
+   exprvec.emplace_back("./a/b/ == active");
+   exprvec.emplace_back("..a == complete");
+   exprvec.emplace_back(".../a == complete");
+   exprvec.emplace_back("../.../a == complete");
+   exprvec.emplace_back(".. /a == complete");
+   exprvec.emplace_back("../.. /a == complete");
+   exprvec.emplace_back("../../.a == complete");
+   exprvec.emplace_back("..a/b == aborted");
+   exprvec.emplace_back("..a/b/c == aborted");
+   exprvec.emplace_back("a == complete and");
+   exprvec.emplace_back("a %");
+   exprvec.emplace_back("(a == complete   b == complete)");
+   exprvec.emplace_back("a == complete and  b == complete)");
+   exprvec.emplace_back("(a == complete and  b == complete");
+   exprvec.emplace_back("(a = complete and b = complete or c = complete)");
+   exprvec.emplace_back("(a erro complete and b == complete) or nodepath:eventname");
+   exprvec.emplace_back("(a == complete and b == complete or (a == complete and b == complete)");
    // triggers that dont make sense in the operational suites.
-   exprvec.push_back("../../../legA/fc/pf/01 eq complete eq complete");
+   exprvec.emplace_back("../../../legA/fc/pf/01 eq complete eq complete");
 
-   exprvec.push_back("/mofc/mon/hind/14/back == complete or %s:DOW ne 5"); // ECFLOW-888
+   exprvec.emplace_back("/mofc/mon/hind/14/back == complete or %s:DOW ne 5"); // ECFLOW-888
 
 
-   BOOST_FOREACH(const string& expr, exprvec ) {
+   for(const string& expr: exprvec ) {
 
       //std::cout << "parsing expression " << expr << "\n";
       ExprParser theExprParser(expr);
