@@ -16,7 +16,7 @@
 #include "ExprDuplicate.hpp"
 
 #include <boost/test/unit_test.hpp>
-#include <boost/foreach.hpp>
+
 #include <boost/date_time/posix_time/posix_time_types.hpp>
 #include <boost/date_time/posix_time/conversion.hpp>
 #include <boost/date_time/posix_time/time_formatters.hpp>  // requires boost date and time lib, for to_simple_string
@@ -50,8 +50,7 @@ BOOST_AUTO_TEST_CASE( test_single_expression )
    exprMap[":var == 0"] = std::make_pair(AstEqual::stype(),true);
    exprMap[":var != 1"] = std::make_pair(AstNotEqual::stype(),true);
 
- 	std::pair<string, std::pair<string,bool> > p;
-	BOOST_FOREACH(p, exprMap ) {
+	for(std::pair<string, std::pair<string,bool>> p : exprMap ) {
 
   		ExprParser theExprParser(p.first);
 		std::string errorMsg;
@@ -61,18 +60,26 @@ BOOST_AUTO_TEST_CASE( test_single_expression )
 		string expectedRootType       = p.second.first;
 		bool expectedEvaluationResult = p.second.second;
 
-      std::stringstream ss;
-
 		Ast* top = theExprParser.getAst();
 		BOOST_REQUIRE_MESSAGE( top ,"No abstract syntax tree");
 		BOOST_REQUIRE_MESSAGE( top->left() ,"No root created");
 		BOOST_REQUIRE_MESSAGE( top->left()->isRoot() || top->left()->is_attribute() ,"First child of top should be a root or attribute " + p.first);
 		BOOST_REQUIRE_MESSAGE( top->left()->is_evaluateable(),"expected ast to be evaluatable. found: " << top->left()->type() << " " << p.first);
 		BOOST_REQUIRE_MESSAGE( top->left()->type() == expectedRootType || top->left()->type() == "variable","expected root type " << expectedRootType << " or 'variable' but found " << top->left()->type() << " " << p.first);
-      top->print_flat(ss);
-		BOOST_REQUIRE_MESSAGE( expectedEvaluationResult == top->evaluate(),"evaluation not as expected for:\n" << p.first << "\n" << ss.str() << "\n" << *top);
+      {
+		   std::stringstream ss;
+		   top->print_flat(ss);
+		   std::string print_flat = ss.str();
+		   cout << print_flat << "\n";
+		   BOOST_REQUIRE_MESSAGE( expectedEvaluationResult == top->evaluate(),"evaluation not as expected for:\n" << p.first << "\n" << print_flat << "\n" << *top);
+      }
+      {
+         std::stringstream ss;
+         top->print(ss);
+         cout << ss.str() << "\n";
+      }
 
-		std::string why;
+      std::string why;
 		top->why(why);
 		cout << "why: " << why << "\n";
 	}
