@@ -12,11 +12,8 @@
 //
 // Description :
 //============================================================================
-#include <assert.h>
-#include <fstream>
-
+#include <stdexcept>
 #include "ClientToServerRequest.hpp"
-#include "AbstractServer.hpp"
 
 using namespace std;
 
@@ -27,14 +24,13 @@ STC_Cmd_ptr ClientToServerRequest::handleRequest(AbstractServer* as) const
 	}
 
 	/// means programming error somewhere
- 	std::stringstream ss;
-	ss << "ClientToServerRequest::handleRequest: Can not send a NULL request to the server !";
-  	throw std::runtime_error(ss.str());
+  	throw std::runtime_error("ClientToServerRequest::handleRequest: Can not send a NULL request to the server !");
 }
 
 std::ostream& ClientToServerRequest::print( std::ostream& os ) const {
 	if (cmd_.get()) {
-		return cmd_->print(os);
+		os << cmd_->print_short(); // avoid printing hundreds of paths in the command
+		return os;
 	}
 	return os << "NULL request";
 }
