@@ -12,18 +12,13 @@
 //
 // Description :
 /////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
+#include <stdexcept>
 #include <iostream>
-#include <sstream>
-#include <fstream>
-
-#include <sys/types.h>
-#include <stdio.h>      /* tolower */
+#include <cstdio>      /* tolower */
 
 #include "ClientToServerCmd.hpp"
-
 #include "AbstractServer.hpp"
 #include "AbstractClientEnv.hpp"
-#include "Log.hpp"
 #include "Str.hpp"
 #include "User.hpp"
 
@@ -33,16 +28,17 @@ using namespace ecf;
 
 bool UserCmd::equals(ClientToServerCmd* rhs) const
 {
-   UserCmd* the_rhs = dynamic_cast< UserCmd* > ( rhs );
+   auto* the_rhs = dynamic_cast< UserCmd* > ( rhs );
    if ( !the_rhs ) return false;
-   return user_ == the_rhs->user();
+   if (user_ != the_rhs->user()) return false;
+   return ClientToServerCmd::equals(rhs);
 }
 
 bool UserCmd::authenticate(AbstractServer* as, STC_Cmd_ptr& cmd) const
 {
    // The user should NOT be empty. Rather than asserting and killing the server, fail authentication
    // ECFLOW-577 and ECFLOW-512. When user_ empty ??
-   if (!user_.empty() && as->authenticateReadAccess(user_,passwd_)) {
+   if (!user_.empty() && as->authenticateReadAccess(user_,cu_,pswd_)) {
 
       // Does this user command require write access
       if ( isWrite() ) {
@@ -71,7 +67,7 @@ bool UserCmd::authenticate(AbstractServer* as, STC_Cmd_ptr& cmd) const
 
 bool UserCmd::do_authenticate(AbstractServer* as, STC_Cmd_ptr&, const std::string& path) const
 {
-   if (!user_.empty() && as->authenticateReadAccess(user_,passwd_,path)) {
+   if (!user_.empty() && as->authenticateReadAccess(user_,cu_,pswd_,path)) {
 
       // Does this user command require write access
       if ( isWrite() ) {
@@ -102,7 +98,7 @@ bool UserCmd::do_authenticate(AbstractServer* as, STC_Cmd_ptr&, const std::strin
 
 bool UserCmd::do_authenticate(AbstractServer* as, STC_Cmd_ptr&, const std::vector<std::string>& paths) const
 {
-   if (!user_.empty() && as->authenticateReadAccess(user_,passwd_,paths)) {
+   if (!user_.empty() && as->authenticateReadAccess(user_,cu_,pswd_,paths)) {
 
       // Does this user command require write access
       if ( isWrite() ) {
@@ -113,7 +109,7 @@ bool UserCmd::do_authenticate(AbstractServer* as, STC_Cmd_ptr&, const std::vecto
          std::string msg = "[ authentication failed ] User ";
          msg += user_;
          msg += " has no *write* access to paths(";
-         for(size_t i=0; i < paths.size(); ++i) { msg += paths[i];msg += ",";}
+         for(const auto & path : paths) { msg += path;msg += ",";}
          msg += ") Please see your administrator.";
          throw std::runtime_error( msg );
       }
@@ -126,7 +122,7 @@ bool UserCmd::do_authenticate(AbstractServer* as, STC_Cmd_ptr&, const std::vecto
    std::string msg = "[ authentication failed ] User '";
    msg += user_;
    msg += "' is not allowed any access. paths(";
-   for(size_t i=0; i < paths.size(); ++i) { msg += paths[i];msg += ",";}
+   for(const auto & path : paths) { msg += path;msg += ",";}
    msg += ")";
    throw std::runtime_error( msg );
 
@@ -136,21 +132,44 @@ bool UserCmd::do_authenticate(AbstractServer* as, STC_Cmd_ptr&, const std::vecto
 void UserCmd::setup_user_authentification(const std::string& user, const std::string& passwd)
 {
    user_ = user;
-   passwd_ = passwd;
+   pswd_ = passwd; // assumes this has been crypted()
+   assert(!hostname().empty());
    assert(!user_.empty());
 }
 
-void UserCmd::setup_user_authentification(AbstractClientEnv& clientEnv)
+bool UserCmd::setup_user_authentification(AbstractClientEnv& clientEnv)
 {
-   setup_user_authentification(User::login_name(),clientEnv.get_user_password());
+   // A Custom user is one where:
+   //   o/ ECF_USER environment used
+   //   o/ --user on the command line
+   //   o/ ClientInvoker::set_user_name()
+   // Having a custom user name is an exception, to avoid having ALL user to provide a password
+   // we have a CUSTOM password file ECF_CUSTOM_PASSWD,   <host>.<port>.ecf.custom_passwd
+   const std::string& user_name = clientEnv.get_user_name(); // Using ECF_USER || --user <user> || set_user_name()/GUI
+   if (!user_name.empty()) {
+      cu_ = true;  // inform server side custom USER used, server will expect password in ECF_CUSTOM_PASSWD files
+      const std::string& passwd = clientEnv.get_custom_user_password(user_name);
+      if (passwd.empty()) return false; // invalid user as password is empty, must have a password when user specified
+
+      setup_user_authentification(user_name,passwd);
+      return true;
+   }
+
+   // User name is same as login name, if ECF_PASSWD is defined on server side *ALL* user must provide a password.
+   std::string the_user = User::login_name();
+   const std::string& passwd = clientEnv.get_user_password( the_user );
+   setup_user_authentification(the_user,passwd);
+   return true;
 }
 
 void UserCmd::setup_user_authentification()
 {
-   if (user_.empty())  user_ = User::login_name();
-   assert(!user_.empty());
+   if (user_.empty()) {
+      setup_user_authentification(User::login_name(),Str::EMPTY());
+   }
 }
 
+
 void UserCmd::prompt_for_confirmation(const std::string& prompt)
 {
    cout << prompt;
@@ -161,9 +180,13 @@ void UserCmd::prompt_for_confirmation(const std::string& prompt)
    }
 }
 
-std::ostream& UserCmd::user_cmd(std::ostream& os, const std::string& the_cmd) const
+void UserCmd::user_cmd(std::string& os, const std::string& the_cmd) const
 {
-   return os << the_cmd << " :" << user_;
+	os += the_cmd;
+	os += " :";
+	os += user_;
+	os += '@';
+	os += hostname();
 }
 
 //#define DEBUG_ME 1
