@@ -16,24 +16,25 @@
 /////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
 #include "ServerToClientCmd.hpp"
 
-class SClientHandleCmd : public ServerToClientCmd {
+class SClientHandleCmd final : public ServerToClientCmd {
 public:
    explicit SClientHandleCmd(int handle) : handle_(handle) {}
-	SClientHandleCmd() : ServerToClientCmd() , handle_(0) {}
+	SClientHandleCmd() : ServerToClientCmd(){}
 
 	void init(int handle) { handle_ = handle; }
-	virtual std::ostream& print(std::ostream& os) const { return os << "cmd:SClientHandleCmd [ " << handle_ << " ]";}
-	virtual bool equals(ServerToClientCmd*) const;
-  	virtual bool handle_server_response( ServerReply&, Cmd_ptr cts_cmd, bool debug ) const;
+	std::string print() const override;
+	bool equals(ServerToClientCmd*) const override;
+  	bool handle_server_response( ServerReply&, Cmd_ptr cts_cmd, bool debug ) const override;
 
 private:
- 	int handle_;
+ 	int handle_{0};
 
-	friend class boost::serialization::access;
-	template<class Archive>
-	void serialize( Archive & ar, const unsigned int /*version*/ ) {
- 		ar & boost::serialization::base_object< ServerToClientCmd >( *this );
- 		ar & handle_;
+   friend class cereal::access;
+   template<class Archive>
+   void serialize(Archive & ar, std::uint32_t const version )
+   {
+ 		ar(cereal::base_class< ServerToClientCmd >( this ),
+ 		   CEREAL_NVP(handle_));
   	}
 };
 
