@@ -17,11 +17,10 @@
 #include "SSyncCmd.hpp"
 #include "Defs.hpp"
 #include "Ecf.hpp"
-#include "Log.hpp"
 #include "AbstractServer.hpp"
-#include "boost_archive.hpp" // collates boost archive includes
-#include "Serialization.hpp"
+//#include "Log.hpp"
 
+using namespace ecf;
 using namespace std;
 using namespace boost;
 
@@ -35,19 +34,17 @@ SSyncCmd::SSyncCmd(
          unsigned int client_modify_change_no,
          AbstractServer* as
 )
-: full_defs_(false),no_defs_(false),
-  incremental_changes_(client_state_change_no)
+:  incremental_changes_(client_state_change_no)
 {
-   init(client_handle, client_state_change_no, client_modify_change_no, false, as);
+   init(client_handle, client_state_change_no, client_modify_change_no, false, false, as);
 }
 
-void SSyncCmd::reset_data_members(unsigned int client_state_change_no)
+void SSyncCmd::reset_data_members(unsigned int client_state_change_no, bool sync_suite_clock)
 {
    full_defs_ = false;
-   no_defs_ = false;
-   incremental_changes_.init(client_state_change_no); // persisted, used for returning INCREMENTAL changes
-   server_defs_ = defs_ptr();                         // persisted, used for returning FULL definition
-   full_server_defs_as_string_.clear();               // semi-persisted, i.e on load & not on saving
+   incremental_changes_.init(client_state_change_no,sync_suite_clock); // persisted, used for returning INCREMENTAL changes
+   server_defs_.clear();                         // persisted, used for returning FULL definition
+   full_server_defs_as_string_.clear();          // semi-persisted, i.e on load & not on saving used to return cached defs
 }
 
 void SSyncCmd::init(
@@ -55,6 +52,7 @@ void SSyncCmd::init(
    unsigned int    client_state_change_no,
    unsigned int    client_modify_change_no,
    bool            do_full_sync,
+   bool            sync_suite_clock,
    AbstractServer* as
 )
 {
@@ -62,11 +60,11 @@ void SSyncCmd::init(
    // This is called in the server
    // ********************************************************
 #ifdef DEBUG_SERVER_SYNC
-      cout << "SSyncCmd::init: client(" << client_state_change_no << "," << client_modify_change_no << ") server(" << Ecf::state_change_no() << "," << Ecf::modify_change_no() << ")";
+      cout << "SSyncCmd::init: client(" << client_state_change_no << "," << client_modify_change_no << ") server(" << Ecf::state_change_no() << "," << Ecf::modify_change_no() << ") sync_suite_clock(" << sync_suite_clock << ")";
 #endif
 
    // Reset all data members since this command can be re-used
-   reset_data_members(client_state_change_no);
+   reset_data_members(client_state_change_no,sync_suite_clock);
 
    // explicit request
    if (do_full_sync) {
@@ -103,6 +101,13 @@ void SSyncCmd::init(
          return;
       }
 
+//      // small scale changes.
+//      // When we going to get thousand of memento, quicker to do a full sync
+//      if (Ecf::state_change_no()-client_state_change_no > 200000) {
+//         full_sync(client_handle,as);
+//         return;
+//      }
+
       // small scale changes. Collate changes over *defs* and all suites.
       // Suite stores the maximum state change, over *all* its children, this is used by client handle mechanism
       // and here to avoid traversing down the hierarchy.
@@ -115,6 +120,7 @@ void SSyncCmd::init(
       if (incremental_changes_.size()) cout << ":*small* scale changes: no of changes(" << incremental_changes_.size() << ")\n";
       else cout << ": *No changes*\n";
 #endif
+//      LOG(Log::DBG,"SSyncCmd::init incremental_changes_.size() " << incremental_changes_.size() << " Ecf::state_change_no()-client_state_change_no " << Ecf::state_change_no()-client_state_change_no );
       return;
    }
 
@@ -126,7 +132,7 @@ void SSyncCmd::init(
 #ifdef DEBUG_SERVER_SYNC
    cout << ": *handle* used " << client_handle << " : ";
    std::vector<string> suites; client_suite_mgr.suites(client_handle,suites);
-   BOOST_FOREACH(const std::string& name, suites) { std::cout << name << " "; }
+   for(const std::string& name: suites) { std::cout << name << " "; }
 #endif
 
    /// *** The client_modify_change_no and client_state_change_no should always be trailing the server
@@ -223,26 +229,33 @@ void SSyncCmd::full_sync(unsigned int client_handle, AbstractServer* as)
 #endif
    }
    else {
-      server_defs_ = the_server_defs;
+      the_server_defs->save_as_string(server_defs_, PrintStyle::NET);
    }
 
 #ifdef DEBUG_SERVER_SYNC
-   if (server_defs_) cout << ": no of suites(" << server_defs_->suiteVec().size() << ")" << endl;
-   else              cout << ": NULL defs!" << endl;
+   if (the_server_defs) cout << ": no of suites(" <<  the_server_defs->suiteVec().size() << ")" << endl;
+   else                 cout << ": NULL defs!" << endl;
 #endif
 }
 
+void SSyncCmd::cleanup()
+{
+   /// run in the server, after command sent to client
+   incremental_changes_.cleanup();
+   std::string().swap(server_defs_);
+   std::string().swap(full_server_defs_as_string_); // will typically be empty in server
+}
+
 bool SSyncCmd::equals(ServerToClientCmd* rhs) const
 {
-   SSyncCmd* the_rhs = dynamic_cast<SSyncCmd*>(rhs);
+   auto* the_rhs = dynamic_cast<SSyncCmd*>(rhs);
    if (!the_rhs) return false;
    return ServerToClientCmd::equals(rhs);
 }
 
-std::ostream& SSyncCmd::print(std::ostream& os) const
+std::string SSyncCmd::print() const
 {
-   os << "cmd:SSyncCmd";
-   return os;
+   return "cmd:SSyncCmd";
 }
 
 bool SSyncCmd::handle_server_response( ServerReply& server_reply, Cmd_ptr /*cts_cmd*/, bool debug ) const
@@ -258,29 +271,19 @@ bool SSyncCmd::do_sync( ServerReply& server_reply, bool debug) const
    // ****************************************************
    // On the client side
    // ****************************************************
-   if (no_defs_) {
-#ifdef DEBUG_CLIENT_SYNC
-      std::cout << "SSyncCmd::do_sync: No defs in the server. Reset client caches\n";
-#endif
-      if (debug) std::cout << "  SSyncCmd::do_sync:: No defs in the server. Reset client caches\n";
-      server_reply.client_handle_ = 0;
-      server_reply.client_defs_ = defs_ptr();
-      server_reply.client_node_ = node_ptr();
-      server_reply.set_sync( true );
-      server_reply.set_full_sync( true );
-      return true;
-   }
-
-   if (server_defs_.get()) {
+   if (!server_defs_.empty()) {
        // *FULL* sync
        // to keep pace with the state changes. Passed back later on, get further changes
        // If non zero handle will contain suites specified in the client handle, including their max change numbers
-       server_reply.client_defs_ = server_defs_;
+       defs_ptr server_defs = Defs::create();
+       server_defs->restore_from_string(server_defs_);  // this can throw
+
+       server_reply.client_defs_ = server_defs;
        server_reply.set_sync( true );
        server_reply.set_full_sync( true );
-       if (debug) cout << "  SSyncCmd::do_sync::*FULL sync*, client side state/modify numbers(" << server_defs_->state_change_no() << "," << server_defs_->modify_change_no() << ")\n";
+       if (debug) cout << "  SSyncCmd::do_sync::*FULL sync*, client side state/modify numbers(" << server_defs->state_change_no() << "," << server_defs->modify_change_no() << ")\n";
 #ifdef DEBUG_CLIENT_SYNC
-       cout << "SSyncCmd::do_sync: defs *FULL sync*, client side state/modify numbers(" << server_defs_->state_change_no() << "," << server_defs_->modify_change_no() << ")\n";
+       cout << "SSyncCmd::do_sync: defs *FULL sync*, client side state/modify numbers(" << server_defs->state_change_no() << "," << server_defs->modify_change_no() << ")\n";
 #endif
        return true;
     }
@@ -346,4 +349,5 @@ bool SSyncCmd::do_sync( ServerReply& server_reply, bool debug) const
    return false;
 }
 
-std::ostream& operator<<(std::ostream& os, const SSyncCmd& c)  { return c.print(os); }
+std::ostream& operator<<(std::ostream& os, const SSyncCmd& c) { os << c.print(); return os; }
+
