@@ -13,25 +13,24 @@
 // Description :
 //
 /////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
+#include <stdexcept>
 #include <iostream>
-#include <fstream>
 #include <iterator>
-#include <stdlib.h> // for getenv()
+#include <cstdlib> // for getenv()
 
-#include "boost/foreach.hpp"
 #include "boost/filesystem/operations.hpp"
-#include "boost/filesystem/path.hpp"
+#include "boost/lexical_cast.hpp"
 
 #include "ClientEnvironment.hpp"
-#include "ClientToServerCmd.hpp"
 #include "File.hpp"
 #include "Str.hpp"
 #include "Ecf.hpp"
-#include "boost_archive.hpp"
 #include "TimeStamp.hpp"
 #include "Version.hpp"
-#ifdef ECF_SECURE_USER
 #include "PasswdFile.hpp"
+
+#ifdef ECF_OPENSSL
+#include "Openssl.hpp"
 #endif
 
 namespace fs = boost::filesystem;
@@ -43,11 +42,13 @@ using namespace boost;
 /// The timeout is used control for how long we continue to iterate over the hosts
 /// attempting to connect to the servers.
 #ifdef DEBUG
-#define MAX_TIMEOUT        120     // Max time in seconds for client to deliver message
-#define MIN_TIMEOUT          5     // Some reasonable value
+#define MAX_TIMEOUT            120     // Max time in seconds for client to deliver message
+#define DEFAULT_ZOMBIE_TIMEOUT 120     // Max time in seconds for client to deliver message
+#define MIN_TIMEOUT              5     // Some reasonable value
 #else
-#define MAX_TIMEOUT    24*3600     // We don't try forever, only 24 hours
-#define MIN_TIMEOUT      10*60     // Some reasonable value
+#define MAX_TIMEOUT            24*3600  // We don't try forever, only 24 hours
+#define DEFAULT_ZOMBIE_TIMEOUT 12*3600  // We don't try forever, only 12 hours
+#define MIN_TIMEOUT            10*60    // Some reasonable value
 #endif
 
 //#define DEBUG_ENVIRONMENT 1
@@ -60,25 +61,25 @@ ostream& operator<<(ostream& os, const vector<T>& v)
 }
 
 
-ClientEnvironment::ClientEnvironment()
+ClientEnvironment::ClientEnvironment(bool gui)
 : AbstractClientEnv(),
-  task_try_num_(1),timeout_(MAX_TIMEOUT),connect_timeout_(0),
-  denied_(false),no_ecf_(false), debug_(false),under_test_(false),
-  host_file_read_(false),
-  host_vec_index_(0),
-  allow_new_client_old_server_(0)
+  timeout_(MAX_TIMEOUT),zombie_timeout_(DEFAULT_ZOMBIE_TIMEOUT),gui_(gui)
 {
 	init();
 }
 
+ClientEnvironment::ClientEnvironment(bool gui,const std::string& host, const std::string& port)
+: AbstractClientEnv(),
+  timeout_(MAX_TIMEOUT),zombie_timeout_(DEFAULT_ZOMBIE_TIMEOUT),gui_(gui)
+{
+	init();
+	set_host_port(host,port); //assumes we NOT going to read host file
+}
+
 // test constructor
 ClientEnvironment::ClientEnvironment(const std::string& hostFile, const std::string& host, const std::string& port)
 : AbstractClientEnv(),
-  task_try_num_(1),timeout_(MAX_TIMEOUT),connect_timeout_(0),
-  denied_(false),no_ecf_(false), debug_(false),under_test_(false),
-  host_file_read_(false),
-  host_vec_index_(0),
-  allow_new_client_old_server_(0)
+  timeout_(MAX_TIMEOUT),zombie_timeout_(DEFAULT_ZOMBIE_TIMEOUT)
 {
 	init();
 
@@ -88,7 +89,7 @@ ClientEnvironment::ClientEnvironment(const std::string& hostFile, const std::str
 	/// Override config, and environment.
 	if (!host.empty()) {
 	   host_vec_.clear();
-	   host_vec_.push_back(std::make_pair(host,port));
+	   host_vec_.emplace_back(host,port);
 	}
 }
 
@@ -101,13 +102,20 @@ void ClientEnvironment::init()
 
 	// If no host specified default to local host and default port number
 	if (host_vec_.empty())
-		host_vec_.push_back(std::make_pair(Str::LOCALHOST(),Str::DEFAULT_PORT_NUMBER()));
+		host_vec_.emplace_back(Str::LOCALHOST(),Str::DEFAULT_PORT_NUMBER());
 
 	// Program option are read in last, and will override any previous setting
 	// However program option are delayed until later. See notes in header
 	if (debug_) std::cout << toString() << "\n";
 }
 
+void ClientEnvironment::set_debug(bool flag)
+{
+   // show client environment when debug is enabled
+   debug_ = flag;
+   if (debug_) std::cout << toString() << "\n";
+}
+
 bool ClientEnvironment::get_next_host(std::string& errorMsg)
 {
 	if (debug_) cout << "ClientEnvironment::get_next_host() host_file_read_ = " << host_file_read_ << " host_file_ = " << host_file_ << "\n";
@@ -135,12 +143,14 @@ const std::string& ClientEnvironment::host() const
 // cout << "ClientEnvironment::host()  host_vec_index_ = " << host_vec_index_
 //      << " host_vec_[host_vec_index_] = " << host_vec_[host_vec_index_] << "\n";
 #endif
+   assert(!host_vec_.empty());
    assert( host_vec_index_ >=0 && host_vec_index_ <  static_cast<int>(host_vec_.size()));
    return host_vec_[host_vec_index_].first;
 }
 
 const std::string& ClientEnvironment::port() const
 {
+   assert(!host_vec_.empty());
    assert( host_vec_index_ >=0 && host_vec_index_ <  static_cast<int>(host_vec_.size()));
    return host_vec_[host_vec_index_].second;
 }
@@ -158,14 +168,17 @@ void ClientEnvironment::set_host_port(const std::string& the_host, const std::st
    host_vec_.clear();
 
    // make sure there only one host:port in host_vec_
-   host_vec_.push_back(std::make_pair(the_host,the_port));
+   host_vec_.emplace_back(the_host,the_port);
 
    // Make sure we don't look in hosts file.
    // When there is only one host:port in host_vec_, calling get_next_host() will always return host_vec_[0]
    host_file_read_ = true;
 
-   // Update 'int allow_new_client_old_server_' *IF* ECF_ALLOW_NEW_CLIENT_OLD_SERVER set, and matches input host/port
-   update_allow_new_client_old_server(the_host,the_port);
+#ifdef ECF_OPENSSL
+   // Must be done *AFTER* host and port set
+   // Avoid enabling SSL for the GUI, via environment, this must be done explicitly by the GUI
+   if (!gui_) enable_ssl_if_defined();
+#endif
 }
 
 bool ClientEnvironment::checkTaskPathAndPassword(std::string& errorMsg) const
@@ -186,31 +199,28 @@ std::string ClientEnvironment::toString() const
 	std::stringstream ss;
 	ss << "ClientEnvironment:\n";
 	ss << TimeStamp::now() << Version::description() << "\n";
-	if (host_vec_.empty()) ss << "   ECF_HOST =\n   ";
-	else  {
-		ss << "   ECF_HOST : host_vec_index_ = " << host_vec_index_ << " host_vec_.size() = " << host_vec_.size() << "\n";
-		std::pair<std::string,std::string> i;
-		BOOST_FOREACH(i, host_vec_) { ss << "   " << i.first << Str::COLON() << i.second << "\n";}
+	ss << "   ECF_HOST/ECF_PORT : host_vec_index_ = " << host_vec_index_ << " host_vec_.size() = " << host_vec_.size() << "\n";
+	if (!host_vec_.empty()) {
+		for(std::pair<std::string,std::string> i : host_vec_) { ss << "   " << i.first << Str::COLON() << i.second << "\n";}
   	}
 	ss << "   ECF_NAME = " << task_path_ << "\n";
 	ss << "   ECF_PASS = " << jobs_password_ << "\n";
 	ss << "   ECF_RID = " << remote_id_ << "\n";
  	ss << "   ECF_TRYNO = " << task_try_num_ << "\n";
 	ss << "   ECF_HOSTFILE = " << host_file_ << "\n";
-   ss << "   ECF_TIMEOUT = " << timeout_ << "\n";
-   ss << "   ECF_CONNECT_TIMEOUT = " << connect_timeout_ << "\n";
+    ss << "   ECF_TIMEOUT = " << timeout_ << "\n";
+    ss << "   ECF_ZOMBIE_TIMEOUT = " << zombie_timeout_ << "\n";
+    ss << "   ECF_CONNECT_TIMEOUT = " << connect_timeout_ << "\n";
 	ss << "   ECF_DENIED = " << denied_ << "\n";
-	assert( host_vec_index_ >=0 && host_vec_index_ <  static_cast<int>(host_vec_.size()));
-	if (host_vec_.empty())  ss << "   ECF_PORT =  \n";
-	else                    ss << "   ECF_PORT = " <<  host_vec_[host_vec_index_].second << "\n";
 	ss << "   NO_ECF = " << no_ecf_ << "\n";
-	for(size_t i = 0; i < env_.size(); i++) {
-		ss << "   " << env_[i].first << " = " << env_[i].second << "\n";
+	for(const auto & i : env_) {
+		ss << "   " << i.first << " = " << i.second << "\n";
 	}
-   ss << "   ECF_ALLOW_NEW_CLIENT_OLD_SERVER = " << env_ecf_new_client_old_server_ << "\n";
-   ss << "   allow_new_client_old_server_ = " << allow_new_client_old_server_ << "\n";
 
    ss << "   ECF_DEBUG_CLIENT = " << debug_ << "\n";
+#ifdef ECF_OPENSSL
+   if (ssl()) ss << "   ECF_SSL = " << ssl_ << "\n";
+#endif
 	return ss.str();
 }
 
@@ -218,8 +228,6 @@ std::string ClientEnvironment::hostSpecified()
 {
    char* the_host = getenv(Str::ECF_HOST().c_str());
    if (the_host)  return std::string(the_host);
-	char* theEnv = getenv(Str::ECF_NODE().c_str());
-	if (theEnv)  return std::string(theEnv);
 	return std::string();
 }
 
@@ -242,17 +250,22 @@ void ClientEnvironment::read_environment_variables()
 	if (getenv(Str::ECF_PASS().c_str())) jobs_password_ = getenv(Str::ECF_PASS().c_str());
  	if (getenv(Str::ECF_TRYNO().c_str()))  task_try_num_ = atoi(getenv(Str::ECF_TRYNO().c_str()));
 	if (getenv("ECF_HOSTFILE")) host_file_ = getenv("ECF_HOSTFILE");
-	if (getenv(Str::ECF_RID().c_str())) remote_id_ = getenv(Str::ECF_RID().c_str());
+   if (getenv(Str::ECF_RID().c_str())) remote_id_ = getenv(Str::ECF_RID().c_str());
+   if (getenv("ECF_USER")) user_name_ = getenv("ECF_USER");
 
 	if (getenv("ECF_TIMEOUT"))  timeout_ = atoi(getenv("ECF_TIMEOUT"));  // host file timeout
 	if( timeout_ > MAX_TIMEOUT ) timeout_ = MAX_TIMEOUT;
 	if( timeout_ < MIN_TIMEOUT )  timeout_ = MIN_TIMEOUT;
 
+   if (getenv("ECF_ZOMBIE_TIMEOUT"))  zombie_timeout_ = atoi(getenv("ECF_ZOMBIE_TIMEOUT"));  // time out for zombies
+   if( zombie_timeout_ > MAX_TIMEOUT ) zombie_timeout_ = MAX_TIMEOUT;
+   if( zombie_timeout_ < MIN_TIMEOUT )  zombie_timeout_ = MIN_TIMEOUT;
+
    if (getenv("ECF_CONNECT_TIMEOUT"))  connect_timeout_ = atoi(getenv("ECF_CONNECT_TIMEOUT")); // for test only
 
 	if (getenv("ECF_DENIED")) denied_ = true;
 	if (getenv("NO_ECF")) no_ecf_ = true;
-	if (getenv("ECF_DEBUG_CLIENT")) debug_ = true;
+   if (getenv("ECF_DEBUG_CLIENT")) debug_ = true;
 
    char *debug_level = getenv("ECF_DEBUG_LEVEL");
    if (debug_level) {
@@ -271,7 +284,7 @@ void ClientEnvironment::read_environment_variables()
   	if (getenv(Str::ECF_PORT().c_str())) {
 		port = getenv(Str::ECF_PORT().c_str());
 		host_vec_.clear(); // remove config settings, net effect is overriding
- 		host_vec_.push_back(std::make_pair(host,port));
+ 		host_vec_.emplace_back(host,port);
 	}
 
 	// Add the ECF_HOST host into list of hosts. Make sure its first in host_vec_
@@ -280,74 +293,18 @@ void ClientEnvironment::read_environment_variables()
 	if (!env_host.empty()) {
 	   host = env_host;
 		host_vec_.clear(); // remove previous setting if any
- 		host_vec_.push_back(std::make_pair(host,port));
+ 		host_vec_.emplace_back(host,port);
 	}
-   if (!getenv(Str::ECF_HOST().c_str()) && getenv(Str::ECF_NODE().c_str()))
-      cerr << "Warning: ECF_NODE has been replaced with ECF_HOST. In future ecflow releases, ECF_NODE will be deprecated\n";
 
-	if (getenv("ECF_ALLOW_NEW_CLIENT_OLD_SERVER")) {
-	   env_ecf_new_client_old_server_ = getenv("ECF_ALLOW_NEW_CLIENT_OLD_SERVER");
-	   update_allow_new_client_old_server(host,port);
-	}
+#ifdef ECF_OPENSSL
+	// Note: This must be placed here for child commands, where we we typically only use environment variables
+   // Must be done last *AFTER* host and port set
+   // Can't use enable_sll(), since that calls host()/port() which use host_vec_, which may be empty
+   // Avoid enabling SSL for the GUI, via environment, this must be done explicitly by the GUI
+   if (!gui_) ssl_.enable_if_defined(host,port);
+#endif
 }
 
-
-static std::string ecf_allow_new_client_old_server_error_msg(const std::string& env)
-{
-   int current_archive_version = ecf::boost_archive::version();
-   int previous_archive_version = current_archive_version -1;
-   std::stringstream ss;
-   ss << "\nFound environment variable ECF_ALLOW_NEW_CLIENT_OLD_SERVER=" << env << "\n";
-   ss << "ECF_ALLOW_NEW_CLIENT_OLD_SERVER must have one of the formats:\n";
-   ss << "   ECF_ALLOW_NEW_CLIENT_OLD_SERVER=<int>\n";
-   ss << "   ECF_ALLOW_NEW_CLIENT_OLD_SERVER=<host>:<port>:<int>,<host>:<port>:<int>,<host>:<port>:<int>\n";
-   ss << "Where <int> represents the archive version of the server. i.e the old server\n";
-   ss << "i.e. export ECF_ALLOW_NEW_CLIENT_OLD_SERVER=" << previous_archive_version << "\n";
-   ss << "or   export ECF_ALLOW_NEW_CLIENT_OLD_SERVER=host1:3142:" << previous_archive_version << ",localhost:3142:" << previous_archive_version << "\n";
-   ss << "Please ensure host/port of the server you want connect too, is on the list";
-   return ss.str();
-}
-
-void ClientEnvironment::update_allow_new_client_old_server(const std::string& host, const std::string& port)
-{
-   // If we update host/port via command line, we need to update this again.
-   // cout << "ClientEnvironment::update_allow_new_client_old_server:  host:" << host << " port:" << port << " env=" << env_ecf_new_client_old_server_ << "\n";
-
-   // In the viewer context, we can have multiple clients, some where client <-> server is compatible , others which are not:
-   // To allow for this, we expect following syntax:
-   //    option 1/ export ECF_ALLOW_NEW_CLIENT_OLD_SERVER=<int>
-   //    option 2/ export ECF_ALLOW_NEW_CLIENT_OLD_SERVER=<host>:<port>:<int>,<host>:<port>:<int>,<host>:<port>:<int>
-   // Where <int> represents the archive version of the server. i.e the old server.
-   // Only if the host/port matches in in our list do we update allow_new_client_old_server_
-   //
-   if (!env_ecf_new_client_old_server_.empty()) {
-      try {
-         // option 1:
-         allow_new_client_old_server_ = boost::lexical_cast<int>(env_ecf_new_client_old_server_);
-      }
-      catch (...) {
-         // cout << "option 2: Match the archive referring to *this* host/port specified.\n";
-         std::vector<std::string> list;
-         Str::split(env_ecf_new_client_old_server_,list,",");
-         if (list.empty())  throw std::runtime_error(ecf_allow_new_client_old_server_error_msg(env_ecf_new_client_old_server_));
-         // Find our host/port in the list:
-         for(size_t i = 0; i < list.size(); ++i) {
-            std::vector<std::string> host_port_archive;
-            Str::split( list[i], host_port_archive, ":");
-            if (host_port_archive.size() == 3) {
-               // cout << host_port_archive[0] << ":" << host_port_archive[1] << ":" << host_port_archive[2] << "\n";
-               if (host_port_archive[0] == host && host_port_archive[1] == port) {
-                  try { allow_new_client_old_server_ = boost::lexical_cast<int>(host_port_archive[2]); break;}
-                  catch (...) { throw std::runtime_error(ecf_allow_new_client_old_server_error_msg(env_ecf_new_client_old_server_));}
-               }
-            }
-            else { throw std::runtime_error(ecf_allow_new_client_old_server_error_msg(env_ecf_new_client_old_server_));}
-         }
-      }
-   }
-}
-
-
 bool ClientEnvironment::parseHostsFile(std::string& errorMsg)
 {
 	std::vector<std::string> lines;
@@ -366,8 +323,8 @@ bool ClientEnvironment::parseHostsFile(std::string& errorMsg)
 	//       host:port   # another comment
 	// If no port is specified we default to port read in from the environment
 	// if that was not specified we default to Str::DEFAULT_PORT_NUMBER()
-	std::vector<std::string>::iterator theEnd = lines.end();
-	for(std::vector<std::string>::iterator i = lines.begin(); i!= theEnd; ++i) {
+	auto theEnd = lines.end();
+	for(auto i = lines.begin(); i!= theEnd; ++i) {
 
 		std::vector< std::string > tokens;
 		Str::split((*i),tokens);
@@ -395,41 +352,52 @@ bool ClientEnvironment::parseHostsFile(std::string& errorMsg)
 		}
 
 		// std::cout << "Host = " << theBackupHost << "  " << thePort << "\n";
-		host_vec_.push_back(std::make_pair(theBackupHost,thePort));
+		host_vec_.emplace_back(theBackupHost,thePort);
 	}
 
 	return true;
 }
 
 
-const std::string& ClientEnvironment::get_user_password() const
+const std::string& ClientEnvironment::get_user_password(const std::string& user) const
 {
-#ifdef ECF_SECURE_USER
-   //cout << "ClientEnvironment::get_user_password() ECF_SECURE_USER passwd_(" << passwd_ << ")\n";
+   return get_password("ECF_PASSWD",user);
+}
+
+const std::string& ClientEnvironment::get_custom_user_password(const std::string& user) const
+{
+   return get_password("ECF_CUSTOM_PASSWD",user);
+}
+
+const std::string& ClientEnvironment::get_password(const char* env, const std::string& user) const
+{
+   //cout << "ClientEnvironment::get_user_password() env:" << env << " user:" << user << " passwd_(" << passwd_ << ")\n";
+   if (user.empty()) throw std::runtime_error("ClientEnvironment::get_user_password: No user specified");
    if (!passwd_.empty()) {
       //cout << "  ClientEnvironment::get_user_password() CACHED returning " << passwd_ << "\n";
       return passwd_;
    }
 
-   char* file = getenv("ECF_PASSWD");
+   char* file = getenv(env);
    if (file) {
       std::string user_passwd_file = file;
-      //cout << "  ClientEnvironment::get_user_password() ECF_PASSWD " << user_passwd_file  << "\n";
+      //cout << "  ClientEnvironment::get_user_password() ECF_CUSTOM_PASSWD " << user_passwd_file  << "\n";
       if (!user_passwd_file.empty() && fs::exists(user_passwd_file)) {
          //cout << "  ClientEnvironment::get_user_password() LOADING password file\n";
          PasswdFile passwd_file;
          std::string errorMsg;
          if (!passwd_file.load(user_passwd_file,debug(),errorMsg)) {
-            std::stringstream ss; ss << "Could not parse ECF_PASSWD file. " << errorMsg;
+            std::stringstream ss; ss << "Could not parse ECF_CUSTOM_PASSWD file. " << errorMsg;
             throw std::runtime_error(ss.str());
          }
          //std::cout << "ClientEnvironment::get_user_password() PasswdFile.dump()\n" << passwd_file.dump() << "\n";
-         passwd_ = passwd_file.get_passwd(User::login_name(), host(), port());
+         passwd_ = passwd_file.get_passwd(user, host(), port());
          //cout << "  ClientEnvironment::get_user_password() returning " << passwd_ << "\n";
          return passwd_;
       }
    }
-#endif
+
    //cout << "  ClientEnvironment::get_user_password() returning EMPTY \n";
    return Str::EMPTY();
 }
+
