@@ -12,26 +12,19 @@
 //
 // Description :
 //============================================================================
-#include <fstream>
 
 #include <boost/test/unit_test.hpp>
-#include "boost/filesystem/operations.hpp"
-#include "boost/filesystem/path.hpp"
 
 #include "ClientInvoker.hpp"
 #include "ClientEnvironment.hpp"
 #include "Suite.hpp"
-#include "Family.hpp"
-#include "Task.hpp"
+#include "Submittable.hpp"
 #include "File.hpp"
-#include "PrintStyle.hpp"
-#include "WhiteListFile.hpp"
 #include "InvokeServer.hpp"
 #include "SCPort.hpp"
-#include "Str.hpp"
 #include "System.hpp"             // kill singleton for valgrind
+//#include "PrintStyle.hpp"
 
-namespace fs = boost::filesystem;
 using namespace std;
 using namespace ecf;
 
@@ -121,7 +114,7 @@ BOOST_AUTO_TEST_CASE( test_client_lifecyle )
    {
       theClient.taskPath(suite1_family1_a);
       theClient.set_jobs_password(Submittable::DUMMY_JOBS_PASSWORD());
-      BOOST_REQUIRE_MESSAGE(theClient.eventTask("myEvent") == 0,TaskApi::event("myEvent") << " failed should return 0\n" << theClient.errorMsg());
+      BOOST_REQUIRE_MESSAGE(theClient.eventTask("myEvent") == 0," failed should return 0\n" << theClient.errorMsg());
       BOOST_REQUIRE_MESSAGE(theClient.forceDependencyEval() == 0,CtsApi::forceDependencyEval() << " failed should return 0\n" << theClient.errorMsg());
       BOOST_REQUIRE_MESSAGE(theClient.getDefs() == 0,CtsApi::get() << " failed should return 0\n" << theClient.errorMsg());
       defs_ptr serverDefs =  theClient.defs();
