@@ -14,11 +14,11 @@
 // Description :
 //============================================================================
 #include <fstream>
+#include <cstdlib>  // getenv
 
 #include <boost/test/unit_test.hpp>
 #include "boost/filesystem/operations.hpp"
 #include "boost/filesystem/path.hpp"
-#include <boost/timer.hpp>
 
 #include "ClientInvoker.hpp"
 #include "ClientEnvironment.hpp"
@@ -44,19 +44,31 @@ BOOST_AUTO_TEST_SUITE( ClientTestSuite )
 // Note: If you make edits to node tree, they will have no effect until the server is rebuilt
 // ************************************************************************************
 
+// These thresholds are based on the largest definition in the tests.
+static int load_threshold_ms  = 4500;
+static int begin_threshold_ms = 400;
+static double sync_full_threshold_s = 2.6;
+static double full_threshold_s = 2.8;
+static double suspend_threshold_s = 3.5;
+static double resume_threshold_s = 3.5;
+static double force_threshold_s = 8.5;
+static double check_pt_threshold_s = 1.0;
+static double client_cmds_threshold_s = 8.5;
+
+
 static void sync_and_news_local(ClientInvoker& theClient)
 {
    {
       cout << "   news_local() : ";
       DurationTimer duration_timer;
       theClient.news_local();
-      cout << (double)duration_timer.elapsed().total_milliseconds()/(double)1000;
+      cout << duration_timer.elapsed_seconds();
    }
    {
       cout << "   sync_local() : ";
       DurationTimer duration_timer;
       theClient.sync_local();
-      cout << (double)duration_timer.elapsed().total_milliseconds()/(double)1000 << endl;
+      cout << duration_timer.elapsed_seconds() << endl;
    }
 }
 
@@ -74,6 +86,7 @@ void time_load_and_downloads(
    BOOST_CHECK(fs::is_directory( full_path ));
 
    int count = 10;
+   std::vector<std::string> paths;
 
    //std::cout << "\nIn directory: " << full_path.directory_string() << "\n\n";
    fs::directory_iterator end_iter;
@@ -94,12 +107,17 @@ void time_load_and_downloads(
             DurationTimer duration_timer;
             BOOST_REQUIRE_MESSAGE(theClient.loadDefs(relPath.string()) == 0,"load defs failed \n" << theClient.errorMsg());
             cout << " Load:                " << duration_timer.elapsed().total_milliseconds() << "ms" << endl;
+            BOOST_CHECK_MESSAGE(duration_timer.elapsed().total_milliseconds() < load_threshold_ms,
+            		"regression load(" << duration_timer.elapsed().total_milliseconds() << "), exceeded threshold of " << load_threshold_ms);
          }
          {
             DurationTimer duration_timer;
             BOOST_REQUIRE_MESSAGE(theClient.begin_all_suites() == 0,"begin failed \n" << theClient.errorMsg());
             cout << " Begin:               " << duration_timer.elapsed().total_milliseconds() << "ms" << endl;
+            BOOST_CHECK_MESSAGE(duration_timer.elapsed().total_milliseconds() < begin_threshold_ms,
+            		"regression begin(" << duration_timer.elapsed().total_milliseconds() << "), exceeded threshold of " << begin_threshold_ms );
          }
+
          {
             cout << " Download(news_local):"; cout.flush();
             ClientInvoker client_news(host,port);
@@ -111,7 +129,6 @@ void time_load_and_downloads(
                      case ServerReply::DO_FULL_SYNC: cout << "FULL ";break;
                      case ServerReply::NEWS: cout << "NEWS ";break;
                      case ServerReply::NO_NEWS: cout << "NO_NEWS ";break;
-                     case ServerReply::NO_DEFS: cout << "NO_DEFS ";break;
                   }
                }
                else if (i == 1) client_news.sync_local();
@@ -141,7 +158,10 @@ void time_load_and_downloads(
                cout << mil_secs  << " ";
                total += mil_secs;
             }
-            cout << ": Avg:" << (double)(total)/((double)count*1000) << "(sec)  : sync_local() with *different* clients.uses cache!" << endl;
+            double average = (double)(total)/((double)count*1000);
+            cout << ": Avg:" << average << "(sec)  : sync_local() with *different* clients.uses cache!" << endl;
+            BOOST_CHECK_MESSAGE(average < sync_full_threshold_s,
+            		"regression Sync-FULL(" << average << "), exceeded threshold of " << sync_full_threshold_s );
          }
          {
             cout << " Download(FULL):      "; cout.flush();
@@ -154,62 +174,88 @@ void time_load_and_downloads(
                cout << seconds << " ";
                total += seconds;
             }
-            cout << ": Avg:" << (double)(total)/((double)count*1000) << "(sec)  : get_defs() from different client" << endl;
+            double average = (double)(total)/((double)count*1000);
+            cout << ": Avg:" <<  average << "(sec)  : get_defs() from different client" << endl;
+            BOOST_CHECK_MESSAGE(average < full_threshold_s,
+            		"regression FULL(sync)(" << average << ") exceeded threshold of " << full_threshold_s );
          }
          {
             std::vector<task_ptr> all_tasks;
+            if (!theClient.defs()) theClient.sync_local();
             theClient.defs()->get_all_tasks(all_tasks);
-            std::vector<std::string> paths;paths.reserve(all_tasks.size());
+            paths.clear();
+            paths.reserve(all_tasks.size());
             for(size_t i = 0; i < all_tasks.size(); i++)  paths.push_back(all_tasks[i]->absNodePath());
 
             {
                cout << " Suspend " << paths.size() << " tasks : "; cout.flush();
                DurationTimer duration_timer;
                theClient.suspend(paths);
-               cout << (double)duration_timer.elapsed().total_milliseconds()/(double)1000;
+               cout << duration_timer.elapsed_seconds();
                sync_and_news_local(theClient);
             }
             {
                cout << " Resume " << paths.size() << " tasks  : "; cout.flush();
                DurationTimer duration_timer;
                theClient.resume(paths);
-               cout << (double)duration_timer.elapsed().total_milliseconds()/(double)1000;
+               cout << duration_timer.elapsed_seconds();
                sync_and_news_local(theClient);
             }
+            {
+               cout << " Suspend " << paths.size() << " tasks : "; cout.flush();
+               theClient.set_auto_sync(true);
+               DurationTimer duration_timer;
+               theClient.suspend(paths);
+               cout << duration_timer.elapsed_seconds() << " : auto-sync" << endl;
+               BOOST_CHECK_MESSAGE(duration_timer.elapsed_seconds() < suspend_threshold_s,
+            		   "regression suspend(" << duration_timer.elapsed_seconds() << ") exceeded threshold of " << suspend_threshold_s);
+               theClient.set_auto_sync(false);
+            }
+            {
+               cout << " Resume " << paths.size() << " tasks  : "; cout.flush();
+               theClient.set_auto_sync(true);
+               DurationTimer duration_timer;
+               theClient.resume(paths);
+               cout << duration_timer.elapsed_seconds() << " : auto-sync" << endl;
+               BOOST_CHECK_MESSAGE(duration_timer.elapsed_seconds() < resume_threshold_s,
+            		   "regression resume(" << duration_timer.elapsed_seconds() << ") exceeded threshold of " << resume_threshold_s);
+               theClient.set_auto_sync(false);
+            }
             {
                cout << " check  " << paths.size() << " tasks  : "; cout.flush();
                DurationTimer duration_timer;
                theClient.check(paths);
-               cout << (double)duration_timer.elapsed().total_milliseconds()/(double)1000;
+               cout << duration_timer.elapsed_seconds();
                sync_and_news_local(theClient);
             }
             {
                cout << " kill   " << paths.size() << " tasks  : "; cout.flush();
                DurationTimer duration_timer;
                theClient.kill(paths);
-               cout << (double)duration_timer.elapsed().total_milliseconds()/(double)1000;
+               cout << duration_timer.elapsed_seconds();
                sync_and_news_local(theClient);
             }
             {
                // force complete on all tasks, on some suites(3199.def), can cause infinite recursion, i.e cause repeats to loop
-               // Hence we will call force aborted instead
+               // Hence we will call force aborted instead.
+               // Also AVOID active/submitted otherwise we spend a huge time adding zombies, for the subsequent force cmd below.
+               // Node: state change memento, will also persist duration time of state change.
                cout << " force  " << paths.size() << " tasks  : "; cout.flush();
                DurationTimer duration_timer;
                theClient.force(paths,"aborted");
-               cout << (double)duration_timer.elapsed().total_milliseconds()/(double)1000;
+               cout << duration_timer.elapsed_seconds() << " force(aborted)";
                sync_and_news_local(theClient);
             }
-            // These tests take too long
-//            size_t max_no_of_paths = 100000;
-//            size_t max_path = std::min(paths.size(),max_no_of_paths);
-//            {
-//               cout << " log    " << max_path << " tasks  : "; cout.flush();
-//               DurationTimer duration_timer;
-//               for(size_t i =0; i < max_path; i++){
-//                  theClient.logMsg(paths[i]);
-//               }
-//               cout << (double)duration_timer.elapsed().total_milliseconds()/(double)1000 << endl;
-//            }
+            {
+               cout << " force  " << paths.size() << " tasks  : "; cout.flush();
+               theClient.set_auto_sync(true);
+               DurationTimer duration_timer;
+               theClient.force(paths,"queued");  // can't use aborted again, as it already aborted, and hence will be ignored
+               cout << duration_timer.elapsed_seconds() << " : auto-sync :  force(queued)" << endl;
+               BOOST_CHECK_MESSAGE(duration_timer.elapsed_seconds() < force_threshold_s,
+            		   "regression force(" << duration_timer.elapsed_seconds() << ") exceeded threshold of " << force_threshold_s);
+               theClient.set_auto_sync(false);
+            }
          }
          {
             cout << " Check pt:            "; cout.flush();
@@ -221,11 +267,45 @@ void time_load_and_downloads(
                cout << seconds << " ";
                total += seconds;
             }
-            cout << ": Avg:" << (double)(total)/((double)count*1000) << "(sec)" << endl;
+            double average = (double)(total)/((double)count*1000);
+            cout << ": Avg:" << average << "(s)" << endl;
+            BOOST_CHECK_MESSAGE(average < check_pt_threshold_s,
+            		"regression in check_pt(" << average << ") exceeded threshold of " << check_pt_threshold_s );
          }
          {
+            cout << " client cmds:         " ;  cout.flush();
             DurationTimer duration_timer;
-            BOOST_REQUIRE_MESSAGE(theClient.delete_all(true)  == 0,"delete all defs failed \n" << theClient.errorMsg());
+            size_t i;
+            for(i = 0; i < paths.size() && i < 2000; i++) {
+               theClient.suspend(paths[i]);         theClient.sync_local();
+               theClient.resume(paths[i]);          theClient.sync_local();
+               theClient.force(paths[i],"aborted"); theClient.sync_local();
+               theClient.alter(paths[i],"add","variable","XXXX","XXXX"); theClient.sync_local();
+               theClient.requeue(paths[i]);         theClient.sync_local();
+            }
+            cout << i << " times " << duration_timer.elapsed_seconds()  << "(s) (sync_local) with same client (suspend,resume,force,alter,requeue)" << endl;
+            BOOST_CHECK_MESSAGE(duration_timer.elapsed_seconds() < client_cmds_threshold_s,"regression, exceeded threshold of " <<  client_cmds_threshold_s );
+         }
+         {
+            cout << " client cmds:         " ;  cout.flush();
+            theClient.set_auto_sync(true);
+            DurationTimer duration_timer;
+            size_t i;
+            for(i = 0; i < paths.size() && i < 2000; i++) {
+               theClient.suspend(paths[i]);
+               theClient.resume(paths[i]);
+               theClient.force(paths[i],"aborted");
+               theClient.alter(paths[i],"add","variable","ZZZZ","XXXX");
+               theClient.requeue(paths[i]);
+            }
+            cout << i << " times " << duration_timer.elapsed_seconds() << "(s) (auto_sync ) with same client (suspend,resume,force,alter,requeue)" << endl;
+            BOOST_CHECK_MESSAGE(duration_timer.elapsed_seconds() < client_cmds_threshold_s,"regression, exceeded threshold of " <<  client_cmds_threshold_s );
+            theClient.set_auto_sync(false);
+         }
+
+         {
+            DurationTimer duration_timer;
+            BOOST_REQUIRE_MESSAGE(theClient.delete_all(true) == 0,"delete all defs failed \n" << theClient.errorMsg());
             cout << " Delete:              " << duration_timer.elapsed().total_milliseconds() << "ms" << endl;
          }
       }
@@ -237,6 +317,19 @@ void time_load_and_downloads(
 
 BOOST_AUTO_TEST_CASE( test_perf_for_large_defs )
 {
+   if (getenv("ECF_SSL")) {
+      load_threshold_ms  = 5000;     // 4500;
+      begin_threshold_ms = 800;      // 400;
+      sync_full_threshold_s = 3.6;   // 2.6;
+      full_threshold_s = 3.8;        // 2.8;
+      suspend_threshold_s = 4.5;     // 3.5;
+      resume_threshold_s = 4.5;      // 3.5;
+      force_threshold_s = 10.5;      // 8.5;
+      check_pt_threshold_s = 1.5;    // 1.0;
+      client_cmds_threshold_s = 12;  // 8.5;
+   }
+
+
    if (fs::exists("/var/tmp/ma0/BIG_DEFS")) {
       /// This will remove checkpt and backup , to avoid server from loading it. (i.e from previous test)
       InvokeServer invokeServer("Client:: ...test_perf_for_large_defs:",SCPort::next());
