@@ -1,11 +1,23 @@
 #!/usr/bin/env python
 # -*- coding= UTF-8 -*-
+#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
+# Name        :
+# Author      : Avi
+# Revision    : $Revision: #10 $
+#
+# Copyright 2009-2020 ECMWF.
+# This software is licensed under the terms of the Apache Licence version 2.0
+# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+# In applying this licence, ECMWF does not waive the privileges and immunities
+# granted to it by virtue of its status as an intergovernmental organisation
+# nor does it submit to any jurisdiction.
+#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
 """ tkinter example with ecflow
 """
-from __future__ import with_statement, print_function
-import Tkinter as tki
-from Tkinter import *
-import Queue
+
+import tkinter as tki
+from tkinter import *
+import queue
 import sys
 import time
 import os
@@ -217,7 +229,7 @@ class Client(object):
                 self.process_nc(item, win)
 
     def process_node(self, item, wins):
-        for kind, win in wins.items():
+        for kind, win in list(wins.items()):
             status = "%s" % item.get_state()
             if status != kind:
                 continue
@@ -276,9 +288,9 @@ class ClientSMS(Client):
             kid = node.kids
             while kid:
                 self.process_node(kid, wins)
-                kid = kid.next
+                kid = kid.__next__
         else:
-            for kind, win in wins.items():
+            for kind, win in list(wins.items()):
                 state = "%s" % ClientSMS.status(node)
                 if state != kind:
                     continue
@@ -387,7 +399,7 @@ class Application(tki.Frame):
     def check_queue(self):
         try:
             self.__queue.get(block=False)
-        except Queue.Empty:
+        except queue.Empty:
             pass
         else:
             self.update()
@@ -395,7 +407,7 @@ class Application(tki.Frame):
 
     def update(self, event=None):
         self.label.update()
-        for kind, win in self.__wins.items():
+        for kind, win in list(self.__wins.items()):
             win.clear()
         for client in self.__clients:
             if type(client) == list:
@@ -419,7 +431,7 @@ if __name__ == '__main__':
 
     if len(sys.argv) > 1:
         clients = []
-        for num in xrange(1, len(sys.argv)):            
+        for num in range(1, len(sys.argv)):            
             sep = ','
             arg = sys.argv[num]
             if sep in arg:
@@ -433,7 +445,7 @@ if __name__ == '__main__':
         print("# using", one)
         clients = [Client(one), ]
 
-    queue = Queue.Queue()
+    queue = queue.Queue()
     app = Application(client=clients, queue=queue)
     app.master.title(PROGRAM_NAME)
     app.columnconfigure(0, weight=1)
