@@ -196,12 +196,14 @@ const char* NodeAttrDoc::inlimit_doc()
             "between tasks 't1' and 't2'. There is no need to change the tasks. The jobs are\n"
             "created in the order they are defined\n"
             "\nConstructor::\n\n"
-            "   InLimit(name, optional<path = ''>, optional<token = 1>)\n"
+            "   InLimit(name, optional<path = ''>, optional<token = 1>, optional<limit_this_node_only = false>)\n"
             "      string name           : The name of the referenced Limit\n"
             "      string path<optional> : The path to the Limit, if this is left out, then Limit of 'name' must be specified\n"
             "                              some where up the parent hierarchy\n"
             "      int value<optional>   : The usage of the Limit. Each job submission will consume 'value' tokens\n"
             "                              from the Limit. defaults to 1 if no value specified.\n"
+            "      bool limit_this_node_only<optional> : Only limits this node and *NOT* its children\n"
+            "                                            Can be used load balance families\n"
             "\nUsage::\n\n"
             "   inlimit = InLimit('fast','/x/f', 2)\n"
             "    ...\n"
@@ -281,6 +283,13 @@ const char* NodeAttrDoc::meter_doc()
             ;
 }
 
+const char* NodeAttrDoc::queue_doc()
+{
+   return
+            "Queue allows specification of queue on Task, Family and Suite nodes\n\n"
+            ;
+}
+
 const char* NodeAttrDoc::date_doc()
 {
    return
@@ -461,6 +470,54 @@ const char* NodeAttrDoc::autocancel_doc()
             ;
 }
 
+const char* NodeAttrDoc::autoarchive_doc()
+{
+   return
+            "Provides a way to automatically archive a suite/family which has completed\n\n"
+            "This is required when dealing with super large suite/families, they can be archived off, and then restored later.\n"
+            "This reduces the time to checkpoint and network band width.\n"
+            "The archived node is written to disk, as ECF_HOME/<host>.<port>.ECF_NAME.check,\n"
+            "where '/' is replaced with ':' in ECF_NAME.\n"
+            "The node can be recovered using 'autorestore', begin,re-queue and manually via ecflow_client --restore.\n"
+            "The archiving is never immediate. The nodes are checked one a minute, and expired autoarchive nodes are archived.\n"
+            "If the node is suspended or any of its parent are suspended then then the archive does not happen.\n"
+            "\nConstructor::\n\n"
+            "   Autoarchive(TimeSlot,relative)\n"
+            "      TimeSlot     : A time\n"
+            "      bool relative:   Relative to completion. False means archive the node at the real time specified.\n\n"
+            "   Autoarchive(hour,minute,relative)\n"
+            "      int hour:        hour in 24 hrs\n"
+            "      int minute:      minute <= 59\n"
+            "      bool relative:   Relative to completion. False means archive the node at the real time specified.\n\n"
+            "   AutoArchive(days)\n"
+            "      int days:        archive the node 'days' after completion\n"
+            "\nUsage::\n\n"
+            "   attr = Autoarchive( 1,30, true )              # archive node 1 hour and 30 minutes after completion\n"
+            "   attr = Autoarchive( TimeSlot(0,10), true )    # archive node 10 minutes after completion\n"
+            "   attr = Autoarchive( TimeSlot(10,10), false )  # archive node at 10:10 after completion\n"
+            "   attr = Autoarchive( 3  )                      # archive node 3 days after completion\n"
+            ;
+}
+
+const char* NodeAttrDoc::autorestore_doc()
+{
+   return
+            "Provides a way to automatically restore a previously archived node\n\n"
+            "This is required when dealing with super large suite/families, they can be archived off, and then restored later.\n"
+            "This reduces the time to checkpoint the definition and network band width.\n"
+            "The archived node is written to disk, as ECF_HOME/<host>.<port>.ECF_NAME.check,\n"
+            "where '/' is replaced with ':' in ECF_NAME.\n"
+            "The node can be recovered using 'autorestore', begin,re-queue and manually via ecflow_client --restore.\n"
+            "The archiving is never immediate. The nodes are checked one a minute, and expired autoarchive nodes are archived.\n"
+            "If the node is suspended or any of its parent are suspended then then the archive does not happen.\n"
+            "\nConstructor::\n\n"
+            "   Autorestore( list of paths )\n"
+            "\nUsage::\n\n"
+            "   attr = Autorestore( ['/s1/f1'] )           # restore archived node at /s1/f1\n"
+            "   attr = Autorestore( ['/s1/f1','/s1/f2'] )  # restore archived node at /s1/f1 and /s1/f2\n"
+            ;
+}
+
 const char* NodeAttrDoc::repeat_doc()
 {
    return
@@ -474,6 +531,9 @@ const char* NodeAttrDoc::repeat_date_doc()
             "Allows a `node`_ to be repeated using a yyyymmdd format\n\n"
             "A node can only have one `repeat`_.\n"
             "The repeat name can be referenced in `trigger`_ expressions.\n"
+            "The trigger expression referencing a RepeatDate will use date arithmetic\n"
+            "Here (/suite/family:YMD + 1) uses date arithmetic only, the result is still an integer\n"
+            "   trigger /suite/family:YMD + 1 > 20190101\n"
             "\nConstructor::\n\n"
             "   RepeatDate(variable,start,end,delta)\n"
             "      string variable:     The name of the repeat. The current date can referenced in\n"
@@ -485,9 +545,32 @@ const char* NodeAttrDoc::repeat_date_doc()
             "- Throws a RuntimeError if start/end are not valid dates\n"
             "\nUsage::\n\n"
             "   rep = RepeatDate('YMD', 20050130, 20050203 )\n"
-            "   rep = RepeatDate('YMD', 20050130, 20050203, 2 )\n"
+            "   rep = RepeatDate('YMD', 20050130, 20050203, 2)\n"
             "   t = Task('t1',\n"
-            "            RepeatDate('YMD', 20050130, 20050203 ) )\n"
+            "            RepeatDate('YMD', 20050130, 20050203 ))\n"
+            ;
+}
+
+const char* NodeAttrDoc::repeat_date_list_doc()
+{
+   return
+            "Allows a `node`_ to be repeated using arbitrary list of yyyymmdd integers\n\n"
+            "A node can only have one `repeat`_.\n"
+            "The repeat name can be referenced in `trigger`_ expressions.\n"
+            "The trigger expression referencing a RepeatDateList will use date arithmetic\n"
+            "Here (/suite/family:YMD + 1) uses date arithmetic only, the result is still an integer\n"
+            "   trigger /suite/family:YMD + 1 > 20190101\n"
+            "\nConstructor::\n\n"
+            "   RepeatDateList(variable,list)\n"
+            "      string variable:     The name of the repeat. The current date can referenced in\n"
+            "                           in trigger expressions using the variable name\n"
+            "      list list_of_int:    Arbitrary list of integers following a yyyymmdd format\n"
+            "\nException:\n\n"
+            "- Throws a RuntimeError if list of integers are not valid dates\n"
+            "\nUsage::\n\n"
+            "   rep = RepeatDateList('YMD', [20050130, 20050203] )\n"
+            "   t = Task('t1',\n"
+            "            RepeatDateList('YMD',[20050130, 20050203] ))\n"
             ;
 }
 
