@@ -179,9 +179,9 @@ class Server(object):
     def __init__(self):
         print("Server:__init__: Starting server")    
         if not debugging():
-            seed_port = int(os.getenv('TEST_ECF_PORT', '3153'))
+            seed_port = int(os.getenv('ECF_FREE_PORT', '3153'))
             if sys.version_info[0] == 3: # python3 can run at same time
-                seed_port = int(os.getenv('TEST_ECF_PORT', '3154'))  
+                seed_port = int(os.getenv('ECF_FREE_PORT', '3154'))  
             if debug_build(): seed_port = seed_port + 1 
             self.lock_file = EcfPortLock()
             self.the_port = self.lock_file.find_free_port(seed_port)   
@@ -231,16 +231,10 @@ class Server(object):
                 else:
                     print("   Server failed to start after 60 second, trying next port !!!!!! " + self.at_time())
                     self.the_port = self.lock_file.find_free_port( int(self.the_port) + 1 )   
-      
-        try:
-            print("   About to ping(again) localhost: " + self.the_port +  " : " + self.at_time())       
-            self.ci.ping() 
-        except RuntimeError as e:
-            print("   Server failed to start: ping failed ?????? " + self.at_time())
-            exit(1)
-            
-        # return the Client, that can call to the server
+             
         print("   Run the tests, leaving Server:__enter__:") 
+
+        # return the Client, that can call to the server
         return self.ci
     
     def __exit__(self,exctype,value,tb):
@@ -251,12 +245,12 @@ class Server(object):
         print("   Terminate server "+ self.at_time() )
         self.ci.terminate_server()  
         print("   Terminate server OK " + self.at_time() )
-        self.lock_file.remove(self.the_port)
+        if not debugging():
+            self.lock_file.remove(self.the_port)
         
         # Do not clean up data, if an assert was raised. This allow debug
-        if exctype == None:
-            clean_up_server(str(self.the_port))
-            clean_up_data(str(self.the_port))
+        if exctype is None:
+            if not debugging():
+                clean_up_server(str(self.the_port))
+                clean_up_data(str(self.the_port))
         return False
-        
-  
\ No newline at end of file
