@@ -13,12 +13,11 @@
 // Description :
 //============================================================================
 
-#include <iostream>
 #include "boost/bind.hpp"
 
 #include "ServerEnvironment.hpp"
 #include "NodeTreeTraverser.hpp"
-#include "Server.hpp"
+#include "BaseServer.hpp"
 #include "Defs.hpp"
 #include "JobsParam.hpp"
 #include "Jobs.hpp"
@@ -26,20 +25,23 @@
 #include "CalendarUpdateParams.hpp"
 #include "Calendar.hpp"
 
+//#define DEBUG_TRAVERSER 1
+//#define DEBUG_POLL 1
+
+#ifdef DEBUG_TRAVERSER
+#include <iostream>
+#endif
 using namespace std;
 using namespace ecf;
 using namespace boost::posix_time;
 
-//#define DEBUG_TRAVERSER 1
-//#define DEBUG_POLL 1
-
 // ***********************************************************************
 // It was noticed that having a poll of 60 seconds was not very accurate
 // even when the server was not active, could be out by as much as 25 seconds
 // Hence we poll every second, and check it against the minute boundary
 // ************************************************************************
 
-NodeTreeTraverser::NodeTreeTraverser( Server* s,
+NodeTreeTraverser::NodeTreeTraverser( BaseServer* s,
 		boost::asio::io_service& io,
 		const ServerEnvironment& serverEnv)
 : server_( s ),
@@ -269,11 +271,7 @@ void NodeTreeTraverser::start_timer()
 {
 	/// Appears that expires_from_now is more accurate then expires_at i.e timer_.expires_at( timer_.expires_at() + boost::posix_time::seconds( poll_at ) );
 	timer_.expires_from_now(  boost::posix_time::seconds( 1 ) );
-#ifdef ECFLOW_MT
-	timer_.async_wait( server_->strand_.wrap( boost::bind( &NodeTreeTraverser::traverse,this,boost::asio::placeholders::error ) ) );
-#else
 	timer_.async_wait( server_->io_service_.wrap( boost::bind( &NodeTreeTraverser::traverse,this,boost::asio::placeholders::error ) ) );
-#endif
 }
 
 void NodeTreeTraverser::traverse(const boost::system::error_code& error )
@@ -304,30 +302,28 @@ void NodeTreeTraverser::update_suite_calendar_and_traverse_node_tree(const boost
 	//       There seems to be greater stability in terms of testing as it allows
 	//       process to complete, before the calendar is incremented.
 	// *****************************************************************************
-	if ( server_->defs_ ) {
 
-		// This functions gets called every 60 seconds or so, update calendar && time
-		// dependent variables in case any jobs depend on them. By default the calendar
-		// update interval is the same as submitJobsInterval for non-real calendars,
-		// however for testing both real/non-real calendars the calendar increment can be
-		// changed to speed up calendar. This is done by setting the calendar increment
-		// on the suite(i.e in the defs file)  which will the _override_ this setting.
-		//
-		// Additionally by passing in the flag running_, it allow suites which want to
-		// stop the calendar updates, when the server is stopped to do so.
-		// For real time calendars we make one system call here, instead of many times in each suite
-		//
-		// In the case where defs/node tree is suspended updateCalendar will continue
-		// to mark those time dep' are free, as free. This information is then used
-		// during the resume
+	// This functions gets called every 60 seconds or so, update calendar && time
+	// dependent variables in case any jobs depend on them. By default the calendar
+	// update interval is the same as submitJobsInterval for non-real calendars,
+	// however for testing both real/non-real calendars the calendar increment can be
+	// changed to speed up calendar. This is done by setting the calendar increment
+	// on the suite(i.e in the defs file)  which will the _override_ this setting.
+	//
+	// Additionally by passing in the flag running_, it allow suites which want to
+	// stop the calendar updates, when the server is stopped to do so.
+	// For real time calendars we make one system call here, instead of many times in each suite
+	//
+	// In the case where defs/node tree is suspended updateCalendar will continue
+	// to mark those time dep' are free, as free. This information is then used
+	// during the resume
 #ifdef DEBUG_TRAVERSER
-		++count_;
+	++count_;
 #endif
-		CalendarUpdateParams calParams(time_now, interval_/* calendar increment */, running_ );
-		server_->defs_->updateCalendar( calParams );
+	CalendarUpdateParams calParams(time_now, interval_/* calendar increment */, running_ );
+	server_->defs_->updateCalendar( calParams );
 
-		traverse_node_tree_and_job_generate(time_now, false /* not in command context */);
-	}
+	traverse_node_tree_and_job_generate(time_now, false /* not in command context */);
 }
 
 void NodeTreeTraverser::traverse_node_tree_and_job_generate(
@@ -340,7 +336,7 @@ void NodeTreeTraverser::traverse_node_tree_and_job_generate(
 	// increment_job_generation_count()
 	// **************************************************************************************
 
-	if ( running_ && server_->defs_) {
+	if ( running_) {
 #ifdef DEBUG_JOB_SUBMISSION
 		jobsParam.logDebugMessage(" from NodeTreeTraverser::traverse_node_tree_and_job_generate()");
 #endif
@@ -381,9 +377,9 @@ void NodeTreeTraverser::traverse_node_tree_and_job_generate(
 			// It could be that we started job generation a few seconds before the poll time,
 			// Hence to avoid excessive warnings, Only warn if time_out_time > next_poll_time_ and forgive about 5 seconds
 			if (!time_out_time.is_special() && time_out_time > next_poll_time_ ) {
-				int leeway = ( serverEnv_.submitJobsInterval() == 60) ? 5 : 1;
+				const int leeway = 5;
 				time_duration duration = time_out_time - next_poll_time_;
-				if ( duration.total_seconds() >= leeway) {
+				if ( duration.total_seconds() >= leeway || serverEnv_.submitJobsInterval() != 60) {
 
 					std::stringstream ss;
 					ss << "Job generation *timed* out: start time:" << start_time << "  time_out_time:" << time_out_time << "  poll_time:" << next_poll_time_;
