@@ -14,19 +14,14 @@
 //
 //============================================================================
 
-#include <stdlib.h>    // for getenv()
-
+#include <cstdlib>    // for getenv()
 #include <iostream>
-#include <fstream>
-#include <iterator>
 
 #include <boost/program_options.hpp>
+#include <boost/bind.hpp>
 #include "boost/filesystem/operations.hpp"
 #include "boost/filesystem/path.hpp"
 #include <boost/lexical_cast.hpp>
-#ifdef ECFLOW_MT
-#include <boost/thread/thread.hpp> // needed for ECFLOW_MT and debug() to print thread ID
-#endif
 
 #include "ServerEnvironment.hpp"
 #include "ServerOptions.hpp"
@@ -36,8 +31,6 @@
 #include "Ecf.hpp"
 #include "Version.hpp"
 #include "Calendar.hpp"
-#include "File.hpp"
-#include "boost_archive.hpp"
 #include "JobProfiler.hpp"
 #include "Pid.hpp"
 
@@ -73,17 +66,15 @@ ServerEnvironment::ServerEnvironment( int argc, char* argv[])
   checkPtInterval_(0),
   checkpt_save_time_alarm_(CheckPt::default_save_time_alarm()),
   submitJobsInterval_(defaultSubmitJobsInterval),
-#ifdef ECFLOW_MT
-  threads_(boost::thread::hardware_concurrency()),
-#endif
   jobGeneration_(true),
   debug_(false),
   help_option_(false),
   version_option_(false),
-  allow_old_client_new_server_(0),
   checkMode_(ecf::CheckPt::ON_TIME),
   tcp_protocol_(boost::asio::ip::tcp::v4())
 {
+   Ecf::set_server(true);
+
    init(argc,argv,"server_environment.cfg");
    if (debug_) std::cout << dump() << "\n";
 }
@@ -95,17 +86,15 @@ ServerEnvironment::ServerEnvironment(int argc, char* argv[], const std::string&
   checkPtInterval_(0),
   checkpt_save_time_alarm_(CheckPt::default_save_time_alarm()),
   submitJobsInterval_(defaultSubmitJobsInterval),
-#ifdef ECFLOW_MT
-  threads_(boost::thread::hardware_concurrency()),
-#endif
   jobGeneration_(true),
   debug_(false),
   help_option_(false),
   version_option_(false),
-  allow_old_client_new_server_(0),
   checkMode_(ecf::CheckPt::ON_TIME),
   tcp_protocol_(boost::asio::ip::tcp::v4())
 {
+   Ecf::set_server(true);
+
    init(argc,argv,path_to_config_file);
    if (debug_) std::cout << dump() << "\n";
 }
@@ -149,6 +138,7 @@ void ServerEnvironment::init(int argc, char* argv[], const std::string& path_to_
    assert(!log_file_name.empty());              // expect name of form "ecf.log"
    assert(!ecf_white_list_file_.empty());       // expect name of form "ecf.lists"
    assert(!ecf_passwd_file_.empty());           // expect name of form "ecf.passwd"
+   assert(!ecf_passwd_custom_file_.empty());    // expect name of form "ecf.custom_passwd"
 
    std::string port = boost::lexical_cast<std::string>(serverPort_);
 
@@ -179,6 +169,9 @@ void ServerEnvironment::init(int argc, char* argv[], const std::string& path_to_
    if (ecf_passwd_file_ == Str::ECF_PASSWD())
       ecf_passwd_file_ = host_name_.prefix_host_and_port(port,ecf_passwd_file_);
 
+   if (ecf_passwd_custom_file_ == Str::ECF_CUSTOM_PASSWD() )
+       ecf_passwd_custom_file_ = host_name_.prefix_host_and_port(port,ecf_passwd_custom_file_);
+
    // Change directory to ECF_HOME and check thats its accessible
    change_dir_to_ecf_home_and_check_accesibility();
 
@@ -201,9 +194,6 @@ void ServerEnvironment::init(int argc, char* argv[], const std::string& path_to_
       LOG(Log::MSG, "Host(" <<  hostPort().first << ")  Port(" << hostPort().second << ") using TCP/IP v6");
    LOG(Log::MSG, "ECF_HOME " <<  ecf_home());
    LOG(Log::MSG, "Job scheduling interval: " <<  submitJobsInterval_);
-   if (allow_old_client_new_server_ != 0) {
-      LOG(Log::MSG, "ECF_ALLOW_OLD_CLIENT_NEW_SERVER enabled: " <<  allow_old_client_new_server_);
-   }
 }
 
 ServerEnvironment::~ServerEnvironment()
@@ -216,6 +206,9 @@ ServerEnvironment::~ServerEnvironment()
 bool ServerEnvironment::valid(std::string& errorMsg) const
 {
    /// This must be called *AFTER* the constructor
+//   cout << "*****************************************************************************\n";
+//   cout << "ServerEnvironment::valid\n";
+//   cout << "*****************************************************************************\n";
 
  	if (serverHost_.empty()) {
   		errorMsg = "Could not determine the server host.";
@@ -317,7 +310,6 @@ bool ServerEnvironment::valid(std::string& errorMsg) const
  		return false;
 	}
 
-#ifdef ECF_SECURE_USER
  	if (!ecf_passwd_file_.empty() && fs::exists(ecf_passwd_file_)) {
 	   if (!passwd_file_.load(ecf_passwd_file_, debug(), errorMsg)) {
  	      std::cout << "Error: could not parse ECF_PASSWD file " << ecf_passwd_file_ << "\n" << errorMsg << "\n";
@@ -329,7 +321,18 @@ bool ServerEnvironment::valid(std::string& errorMsg) const
          return false;
  	   }
  	}
-#endif
+
+   if (!ecf_passwd_custom_file_.empty() && fs::exists(ecf_passwd_custom_file_)) {
+      if (!passwd_custom_file_.load(ecf_passwd_custom_file_, debug(), errorMsg)) {
+         std::cout << "Error: could not parse ECF_CUSTOM_PASSWD file " << ecf_passwd_custom_file_  << "\n" << errorMsg << "\n";
+         return false;
+      }
+      if (!passwd_custom_file_.check_at_least_one_user_with_host_and_port(serverHost_,the_port())) {
+         std::cout << "Error: custom password file " << ecf_passwd_custom_file_;
+         std::cout << " does not contain any users, which match the host and port of this server\n";
+         return false;
+      }
+   }
 
 	// If the white list file is empty or does not exist, *ON* server start, its perfectly valid
 	// i.e any user is free to access the server
@@ -358,39 +361,42 @@ void ServerEnvironment::variables(std::vector<std::pair<std::string,std::string>
 	   // Need to setup client environment.
 	   // The server sets these variable for use by the client. i.e when creating the jobs
 	   // The clients then uses them to communicate back with the server.
-  	theRetVec.push_back( std::make_pair(Str::ECF_PORT(), the_port()) );
-   theRetVec.push_back( std::make_pair(Str::ECF_HOST(), serverHost_) );
-   theRetVec.push_back( std::make_pair(Str::ECF_NODE(), "%ECF_HOST%") );
+  	theRetVec.emplace_back(Str::ECF_PORT(), the_port() );
+   theRetVec.emplace_back(Str::ECF_HOST(), serverHost_ );
 
-	theRetVec.push_back( std::make_pair(Str::ECF_HOME(), ecfHome_) );
-	if (Log::instance()) theRetVec.push_back( std::make_pair(std::string("ECF_LOG"), Log::instance()->path()) );
-	else                 theRetVec.push_back( std::make_pair(std::string("ECF_LOG"), std::string() ));
-	theRetVec.push_back( std::make_pair(std::string("ECF_CHECK"), ecf_checkpt_file_) );
-	theRetVec.push_back( std::make_pair(std::string("ECF_CHECKOLD"), ecf_backup_checkpt_file_) );
-   theRetVec.push_back( std::make_pair(std::string("ECF_INTERVAL"), boost::lexical_cast<std::string>(submitJobsInterval_)) );
+	theRetVec.emplace_back(Str::ECF_HOME(), ecfHome_ );
+	theRetVec.emplace_back(std::string("ECF_LOG"), Log::instance()->path() );
+	theRetVec.emplace_back(std::string("ECF_CHECK"), ecf_checkpt_file_ );
+	theRetVec.emplace_back(std::string("ECF_CHECKOLD"), ecf_backup_checkpt_file_ );
+   theRetVec.emplace_back(std::string("ECF_INTERVAL"), boost::lexical_cast<std::string>(submitJobsInterval_) );
 
 	// These variable are read in from the environment, but are not exposed
 	// since they only affect the server
 	// ECF_CHECKINTERVAL
 
-   theRetVec.push_back( std::make_pair(std::string("ECF_LISTS"), ecf_white_list_file_) ); // read only variable, changing it has no effect
-#ifdef ECF_SECURE_USER
-   theRetVec.push_back( std::make_pair(std::string("ECF_PASSWD"), ecf_passwd_file_) );    // read only variable, changing it has no effect
-#endif
+   theRetVec.emplace_back(std::string("ECF_LISTS"), ecf_white_list_file_ );           // read only variable, changing it has no effect
+   theRetVec.emplace_back(std::string("ECF_PASSWD"), ecf_passwd_file_ );              // read only variable, changing it has no effect
+   theRetVec.emplace_back(std::string("ECF_CUSTOM_PASSWD"), ecf_passwd_custom_file_ );// read only variable, changing it has no effect
 
 	// variables that can be overridden, in the suite definition
-	theRetVec.push_back( std::make_pair(std::string("ECF_JOB_CMD"), ecf_cmd_) );
-	theRetVec.push_back( std::make_pair(std::string("ECF_KILL_CMD"), killCmd_) );
-   theRetVec.push_back( std::make_pair(std::string("ECF_STATUS_CMD"), statusCmd_) );
-   theRetVec.push_back( std::make_pair(std::string("ECF_CHECK_CMD"), checkCmd_) );
-	theRetVec.push_back( std::make_pair(std::string("ECF_URL_CMD"), urlCmd_) );
-	theRetVec.push_back( std::make_pair(std::string("ECF_URL_BASE"), urlBase_) );
-	theRetVec.push_back( std::make_pair(std::string("ECF_URL"), url_) );
-   theRetVec.push_back( std::make_pair(std::string("ECF_MICRO"), ecf_micro_) );
+	theRetVec.emplace_back(std::string("ECF_JOB_CMD"), ecf_cmd_ );
+	theRetVec.emplace_back(std::string("ECF_KILL_CMD"), killCmd_ );
+   theRetVec.emplace_back(std::string("ECF_STATUS_CMD"), statusCmd_ );
+   theRetVec.emplace_back(std::string("ECF_CHECK_CMD"), checkCmd_ );
+	theRetVec.emplace_back(std::string("ECF_URL_CMD"), urlCmd_ );
+	theRetVec.emplace_back(std::string("ECF_URL_BASE"), urlBase_ );
+	theRetVec.emplace_back(std::string("ECF_URL"), url_ );
+   theRetVec.emplace_back(std::string("ECF_MICRO"), ecf_micro_ );
 
    // Reference variable, these should be read only
-   theRetVec.push_back( std::make_pair(std::string("ECF_PID"), ecf_pid_) );          // server PID
-   theRetVec.push_back( std::make_pair(std::string("ECF_VERSION"), Version::raw()) );// server version
+   theRetVec.emplace_back(std::string("ECF_PID"), ecf_pid_ );          // server PID
+   theRetVec.emplace_back(std::string("ECF_VERSION"), Version::raw() );// server version
+
+#ifdef ECF_OPENSSL
+   if (ssl_.enabled()) {
+      theRetVec.emplace_back(std::string("ECF_SSL"), ssl_.ssl()); // 1 means shared ssl certificate, <host>.<port> means server specific certificates
+   }
+#endif
 }
 
 bool ServerEnvironment::reloadWhiteListFile(std::string& errorMsg)
@@ -446,28 +452,48 @@ bool ServerEnvironment::reloadPasswdFile(std::string& errorMsg)
    return passwd_file_.load(ecf_passwd_file_, debug(), errorMsg );
 }
 
-bool ServerEnvironment::authenticateReadAccess(const std::string& user,const std::string& passwd) const
+bool ServerEnvironment::reloadCustomPasswdFile(std::string& errorMsg)
 {
-#ifdef ECF_SECURE_USER
-   if (!passwd_file_.authenticate(user,passwd)) return false;
-#endif
-	// if *NO* users specified then all users are valid
+   if (debug()) cout << "ServerEnvironment::reloadPasswdFile:(" << ecf_passwd_custom_file_ << ") CWD(" << fs::current_path().string() << ")\n";
+   if (ecf_passwd_custom_file_.empty()) {
+      errorMsg += "The ECF_CUSTOM_PASSWD file ";
+      errorMsg += ecf_passwd_custom_file_ ;
+      errorMsg += " has not been specified.";
+      return false;
+   }
+   if (!fs::exists(ecf_passwd_custom_file_)) {
+      errorMsg += "The ECF_CUSTOM_PASSWD file ";
+      errorMsg += ecf_passwd_custom_file_;
+      errorMsg += " does not exist. Server CWD : " + fs::current_path().string();
+      return false;
+   }
+
+   // Only override valid users if we successfully opened and parsed file
+   return passwd_custom_file_.load(ecf_passwd_custom_file_ , debug(), errorMsg );
+}
+
+bool ServerEnvironment::authenticateReadAccess(const std::string& user,bool custom_user,const std::string& passwd) const
+{
+   if (!custom_user){ if (!passwd_file_.authenticate(user,passwd)) return false;}
+   else             { if (!passwd_custom_file_.authenticate(user,passwd)) return false;}
+
+   // if *NO* users specified then all users are valid
 	return white_list_file_.verify_read_access(user);
 }
 
-bool ServerEnvironment::authenticateReadAccess(const std::string& user,const std::string& passwd,const std::string& path) const
+bool ServerEnvironment::authenticateReadAccess(const std::string& user,bool custom_user,const std::string& passwd,const std::string& path) const
 {
-#ifdef ECF_SECURE_USER
-   if (!passwd_file_.authenticate(user,passwd)) return false;
-#endif
+   if (!custom_user){ if (!passwd_file_.authenticate(user,passwd)) return false; }
+   else             { if (!passwd_custom_file_.authenticate(user,passwd)) return false;}
+
    return white_list_file_.verify_read_access(user,path);
 }
 
-bool ServerEnvironment::authenticateReadAccess(const std::string& user,const std::string& passwd,const std::vector<std::string>& paths) const
+bool ServerEnvironment::authenticateReadAccess(const std::string& user,bool custom_user,const std::string& passwd,const std::vector<std::string>& paths) const
 {
-#ifdef ECF_SECURE_USER
-   if (!passwd_file_.authenticate(user,passwd)) return false;
-#endif
+   if (!custom_user){ if (!passwd_file_.authenticate(user,passwd)) return false; }
+   else             { if (!passwd_custom_file_.authenticate(user,passwd)) return false;}
+
    return white_list_file_.verify_read_access(user,paths);
 }
 
@@ -519,6 +545,7 @@ void ServerEnvironment::read_config_file(std::string& log_file_name,const std::s
          ("ECF_MICRODEF",  po::value<std::string>(&ecf_micro_)->default_value(Ecf::MICRO()), "Preprocessor character for variable substitution and including files")
          ("ECF_LISTS",     po::value<std::string>(&ecf_white_list_file_)->default_value(Str::WHITE_LIST_FILE()), "Path name to file the list valid users and their access rights")
          ("ECF_PASSWD",    po::value<std::string>(&ecf_passwd_file_)->default_value(Str::ECF_PASSWD()), "Path name to passwd file")
+         ("ECF_CUSTOM_PASSWD",po::value<std::string>(&ecf_passwd_custom_file_)->default_value(Str::ECF_CUSTOM_PASSWD()), "Path name to custom passwd file, for user who don't use login name")
          ("ECF_TASK_THRESHOLD",po::value<int>(&the_task_threshold)->default_value(JobProfiler::task_threshold_default()),"The defaults thresholds when profiling job generation")
          ;
 
@@ -593,15 +620,18 @@ void ServerEnvironment::read_environment_variables(std::string& log_file_name)
    char* passwd = getenv("ECF_PASSWD");
    if (passwd) ecf_passwd_file_ = passwd;
 
+   char* custom_passwd = getenv("ECF_CUSTOM_PASSWD");
+   if (custom_passwd) ecf_passwd_custom_file_ = custom_passwd;
+
 	if (getenv("ECF_DEBUG_SERVER")) {
 		debug_ = true; // can also be enabled via --debug option
 	}
 
-	if (getenv("ECF_ALLOW_OLD_CLIENT_NEW_SERVER")) {
-	   // we don't need exact version of boost archive version of old client, since we can determine
-	   // this from the message sent to the server from the *old* client
-	   allow_old_client_new_server_ = ecf::boost_archive::version_1_47();
-	}
+#ifdef ECF_OPENSSL
+	// IF ECF_SSL= 1 search server.crt
+	// ELSE          search <host>.<port>.crt
+   ssl_.enable_if_defined(serverHost_,the_port());
+#endif
 
    char* threshold = getenv("ECF_TASK_THRESHOLD");
    if ( threshold ) {
@@ -641,8 +671,7 @@ std::string ServerEnvironment::dump() const
 {
    std::stringstream ss;
    ss << "ECF_HOME = '" << ecfHome_ << "'\n";
-   if (Log::instance()) ss << "ECF_LOG = '" << Log::instance()->path() << "'\n";
-   else                 ss << "ECF_LOG = ''\n";
+   ss << "ECF_LOG = '" << Log::instance()->path() << "'\n";
    ss << "ECF_PORT = '" << serverPort_ << "'\n";
    ss << "ECF_CHECK = '" << ecf_checkpt_file_ << "'\n";
    ss << "ECF_CHECKOLD = '" << ecf_backup_checkpt_file_ << "'\n";
@@ -660,44 +689,44 @@ std::string ServerEnvironment::dump() const
    ss << "check pt save time alarm " << checkpt_save_time_alarm_ << "\n";
    ss << "Job generation " << jobGeneration_ << "\n";
    ss << "Server host name " << serverHost_ << "\n";
-#ifdef ECF_SECURE_USER
    ss << "ECF_PASSWD = " << ecf_passwd_file_ << "\n";
-#endif
-#ifdef ECFLOW_MT
-   ss << "No of threads used by server " << threads_ << "\n";
-#endif
+   ss << "ECF_CUSTOM_PASSWD = " << ecf_passwd_custom_file_ << "\n";
    if ( tcp_protocol_.family() ==  2 /*PF_INET*/)  ss << "TCP Protocol  v4 \n";
    else if ( tcp_protocol_.family() ==  10 /*PF_INET6*/)  ss << "TCP Protocol  v6 \n";
 
+#ifdef ECF_OPENSSL
+   ss << "ECF_SSL = " << ssl_ << "\n";
+#endif
+
    ss << white_list_file_.dump_valid_users();
    return ss.str();
 }
 
-
 std::vector<std::string> ServerEnvironment::expected_variables()
 {
    std::vector<std::string> expected_variables;
    expected_variables.push_back(  Str::ECF_HOME() );
-   expected_variables.push_back( "ECF_LOG"  );
-   expected_variables.push_back( "ECF_CHECK" );
-   expected_variables.push_back( "ECF_CHECKOLD" );
-   expected_variables.push_back( "ECF_JOB_CMD" );
-   expected_variables.push_back( "ECF_KILL_CMD" );
-   expected_variables.push_back( "ECF_STATUS_CMD" );
-   expected_variables.push_back( "ECF_CHECK_CMD" );
-   expected_variables.push_back( "ECF_URL_CMD" );
-   expected_variables.push_back( "ECF_URL_BASE" );
-   expected_variables.push_back( "ECF_URL" );
-   expected_variables.push_back( "ECF_MICRO" );
-   expected_variables.push_back( "ECF_PID" );
-   expected_variables.push_back( "ECF_VERSION" );
-   expected_variables.push_back( "ECF_LISTS" );
+   expected_variables.emplace_back("ECF_LOG"  );
+   expected_variables.emplace_back("ECF_CHECK" );
+   expected_variables.emplace_back("ECF_CHECKOLD" );
+   expected_variables.emplace_back("ECF_JOB_CMD" );
+   expected_variables.emplace_back("ECF_KILL_CMD" );
+   expected_variables.emplace_back("ECF_STATUS_CMD" );
+   expected_variables.emplace_back("ECF_CHECK_CMD" );
+   expected_variables.emplace_back("ECF_URL_CMD" );
+   expected_variables.emplace_back("ECF_URL_BASE" );
+   expected_variables.emplace_back("ECF_URL" );
+   expected_variables.emplace_back("ECF_MICRO" );
+   expected_variables.emplace_back("ECF_PID" );
+   expected_variables.emplace_back("ECF_VERSION" );
+   expected_variables.emplace_back("ECF_LISTS" );
    expected_variables.push_back(  Str::ECF_PORT() );
-   expected_variables.push_back(  Str::ECF_NODE() );
    expected_variables.push_back(  Str::ECF_HOST() );
-   expected_variables.push_back( "ECF_INTERVAL");
-#ifdef ECF_SECURE_USER
-   expected_variables.push_back( "ECF_PASSWD");
+   expected_variables.emplace_back("ECF_INTERVAL");
+   expected_variables.emplace_back("ECF_PASSWD");
+   expected_variables.emplace_back("ECF_CUSTOM_PASSWD");
+#ifdef ECF_OPENSSL
+   if (getenv("ECF_SSL")) expected_variables.emplace_back("ECF_SSL");
 #endif
    return expected_variables;
 }
