@@ -26,9 +26,6 @@
 
 #include <string>
 #include <vector>
-#include <map>
-#include <exception>
-#include <boost/noncopyable.hpp>
 #include <boost/asio.hpp>
 
 #include "Host.hpp"
@@ -36,6 +33,10 @@
 #include "WhiteListFile.hpp"
 #include "PasswdFile.hpp"
 
+#ifdef ECF_OPENSSL
+#include "Openssl.hpp"
+#endif
+
 // Added ServerEvinronmentException so that it can be in the same scope as server
 // in ServerMain. Previously we had a separate try block
 // Also distinguish between server and environment errors
@@ -45,7 +46,9 @@ public:
 };
 
 
-class ServerEnvironment : private boost::noncopyable {
+class ServerEnvironment {
+   ServerEnvironment(const ServerEnvironment&) = delete;
+   const ServerEnvironment& operator=(const ServerEnvironment&) = delete;
 public:
    ServerEnvironment(int argc, char* argv[]);
    ServerEnvironment(int argc, char* argv[], const std::string& path_to_config_file); // *only used in test*
@@ -61,6 +64,13 @@ public:
 	/// returns the server host and port
 	std::pair<std::string,std::string> hostPort() const;
 
+#ifdef ECF_OPENSSL
+	/// return true if ssl enable via command line, AND SSL libraries are found
+	ecf::Openssl& openssl() { return ssl_;}
+	bool ssl() const { return ssl_.enabled();}
+	void enable_ssl() { ssl_.enable(serverHost_,the_port());} // search server.crt first, then <host>.<port>.crt
+#endif
+
 	/// returns the server port. This has a default value defined in server_environment.cfg
 	/// but can be overridden by the environment variable ECF_PORT
 	int port() const { return serverPort_;}
@@ -73,13 +83,6 @@ public:
 	/// for test/debug we can elect to disable, it.
 	bool jobGeneration() const { return jobGeneration_;}
 
-   /// Forward compatibility allow old clients to talk to new server
-	/// Temp: This allows ecflow  to be built. i.e we have old clients(3.0.x), installed on different machines
-	///       they need to talk to new server. (i.e since we can't change the old clients)
-   /// This is controlled with ECF_ALLOW_OLD_CLIENT_NEW_SERVER
-   int allow_old_client_new_server() const { return allow_old_client_new_server_;}
-
-
 	/// Whenever we save the checkpt, we time how long this takes.
 	/// For very large definition the time can be significant and start to interfere with
 	/// the scheduling. (i.e since write to disk is blocking).
@@ -128,7 +131,6 @@ public:
 	/// Some of these variables hold environment variables
 	/// Note:: additional variable are created for use by clients, i.e like
 	///        ECF_PORT(jobs, server, client)
-   ///        ECF_NODE(jobs).   ECF_NODE is the server machine host name  /* OLD NAME, TOBE DEPRECATED*/
    ///        ECF_HOST(jobs).   ECF_HOST is the server machine host name
   	void variables(std::vector<std::pair<std::string,std::string> >&) const;
 
@@ -142,12 +144,8 @@ public:
 	/// If errors arise the exist user still stay in affect
   	bool reloadWhiteListFile(std::string& errorMsg);
 
-  	bool reloadPasswdFile(std::string& errorMsg);
-
-#ifdef ECFLOW_MT
-  	// returns the numbers threads to be used by the server.
-   size_t threads() const { return threads_; };
-#endif
+   bool reloadPasswdFile(std::string& errorMsg);
+   bool reloadCustomPasswdFile(std::string& errorMsg);
 
 	/// There are several kinds of authentification:
 	///     a/ None
@@ -156,9 +154,9 @@ public:
 	/// At the moment we will only implement options a/ and b/
 	//
 	/// Returns true if the given user has access to the server, false otherwise
-   bool authenticateReadAccess(const std::string& user,const std::string& passwd)const;
-   bool authenticateReadAccess(const std::string& user,const std::string& passwd,const std::string& path)const;
-   bool authenticateReadAccess(const std::string& user,const std::string& passwd,const std::vector<std::string>& paths)const;
+   bool authenticateReadAccess(const std::string& user,bool custom_user,const std::string& passwd)const;
+   bool authenticateReadAccess(const std::string& user,bool custom_user,const std::string& passwd,const std::string& path)const;
+   bool authenticateReadAccess(const std::string& user,bool custom_user,const std::string& passwd,const std::vector<std::string>& paths)const;
  	bool authenticateWriteAccess(const std::string& user) const;
    bool authenticateWriteAccess(const std::string& user,const std::string& path)const;
    bool authenticateWriteAccess(const std::string& user,const std::vector<std::string>& paths)const;
@@ -195,14 +193,10 @@ private:
 	int  checkPtInterval_;
 	int  checkpt_save_time_alarm_;
 	int  submitJobsInterval_;
-#ifdef ECFLOW_MT
-	size_t threads_;
-#endif
-	bool jobGeneration_;   // used in debug/test mode only
+   bool jobGeneration_;   // used in debug/test mode only
 	bool debug_;
    bool help_option_;
    bool version_option_;
-	int allow_old_client_new_server_;
  	ecf::CheckPt::Mode checkMode_;
 	std::string ecfHome_;
 	std::string ecf_checkpt_file_;
@@ -219,8 +213,14 @@ private:
    std::string ecf_white_list_file_;
    mutable WhiteListFile white_list_file_;
 
-   std::string ecf_passwd_file_;      // Only used if ECF_SECURE_USER defined
-   mutable PasswdFile passwd_file_;   // Only used if ECF_SECURE_USER defined
+   std::string ecf_passwd_file_;
+   std::string ecf_passwd_custom_file_;
+   mutable PasswdFile passwd_file_;
+   mutable PasswdFile passwd_custom_file_;
+
+#ifdef ECF_OPENSSL
+   ecf::Openssl ssl_;
+#endif
 
 	boost::asio::ip::tcp tcp_protocol_;      // defaults to IPv4 TCP protocol
 	friend class ServerOptions;
