@@ -15,8 +15,7 @@
 #include <QToolButton>
 
 LineEdit::LineEdit(QWidget *parent):
-      QLineEdit(parent),
-      iconLabel_(0)
+      QLineEdit(parent)
 {
 	clearTb_=new QToolButton(this);
 	QPixmap pix(":/viewer/clear_left.svg");
