@@ -44,9 +44,11 @@
 #include "ServerListSyncWidget.hpp"
 #include "SessionHandler.hpp"
 #include "SaveSessionAsDialog.hpp"
+#include "ShortcutHelpDialog.hpp"
 #include "CommandOutputDialog.hpp"
 #include "TextFormat.hpp"
 #include "UiLog.hpp"
+#include "VAttributeType.hpp"
 #include "VConfig.hpp"
 #include "VIcon.hpp"
 #include "VSettings.hpp"
@@ -66,7 +68,7 @@ LogViewerCom* MainWindow::logCom_=NULL;
 
 MainWindow::MainWindow(QStringList idLst,QWidget *parent) :
     QMainWindow(parent),
-    serverSyncNotifyTb_(0)
+    serverSyncNotifyTb_(nullptr)
 {
     setupUi(this);
     
@@ -80,7 +82,7 @@ MainWindow::MainWindow(QStringList idLst,QWidget *parent) :
     winTitle_->update();
 
     //Create the main layout
-    QVBoxLayout* layout=new QVBoxLayout();
+    auto* layout=new QVBoxLayout();
     layout->setContentsMargins(0,0,0,0);
     QWidget *w=new QWidget(this);
     w->setObjectName("c");
@@ -118,7 +120,7 @@ MainWindow::MainWindow(QStringList idLst,QWidget *parent) :
 
 
     //insert a spacer after the the server refresh widget
-    QWidget* spacer = new QWidget();
+    auto* spacer = new QWidget();
     spacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
     //viewToolBar->insertWidget(actionSearch,spacer);
     viewToolBar->addWidget(spacer);
@@ -161,7 +163,7 @@ MainWindow::MainWindow(QStringList idLst,QWidget *parent) :
     }
 
     //Add notification widget
-    ChangeNotifyWidget* chw=new ChangeNotifyWidget(this);
+    auto* chw=new ChangeNotifyWidget(this);
     statusBar()->addPermanentWidget(chw);
 
     //Add clock widget
@@ -194,17 +196,16 @@ void MainWindow::init(MainWindow *win)
 
 void MainWindow::addInfoPanelActions(QToolBar *toolbar)
 {
-   for(std::vector<InfoPanelDef*>::const_iterator it=InfoPanelHandler::instance()->panels().begin();
-           it != InfoPanelHandler::instance()->panels().end(); ++it)
+   for(auto it : InfoPanelHandler::instance()->panels())
    {
-	   if((*it)->show().find("toolbar") != std::string::npos)
+	   if(it->show().find("toolbar") != std::string::npos)
 	   {
-		   QAction *ac=toolbar->addAction(QString::fromStdString((*it)->label()));
-           QPixmap pix(":/viewer/" + QString::fromStdString((*it)->icon()));
-           ac->setObjectName(QString::fromStdString((*it)->label()));
+		   QAction *ac=toolbar->addAction(QString::fromStdString(it->label()));
+           QPixmap pix(":/viewer/" + QString::fromStdString(it->icon()));
+           ac->setObjectName(QString::fromStdString(it->label()));
            ac->setIcon(QIcon(pix));
-		   ac->setData(QString::fromStdString((*it)->name()));
-           ac->setToolTip(QString::fromStdString((*it)->buttonTooltip()));
+		   ac->setData(QString::fromStdString(it->name()));
+           ac->setToolTip(QString::fromStdString(it->buttonTooltip()));
 
 		   connect(ac,SIGNAL(triggered()),
 				   this,SLOT(slotOpenInfoPanel()));
@@ -216,7 +217,7 @@ void MainWindow::addInfoPanelActions(QToolBar *toolbar)
 
 ServerHandler* MainWindow::selectedServer() const
 {
-    return(selection_)?(selection_->server()):0;
+    return(selection_)?(selection_->server()):nullptr;
 }
 
 //==============================================================
@@ -284,7 +285,7 @@ void MainWindow::on_actionPreferences_triggered()
 
 void MainWindow::on_actionManageSessions_triggered()
 {
-	QMessageBox::information(0, tr("Manage Sessions"),
+	QMessageBox::information(nullptr, tr("Manage Sessions"),
 		tr("To manage sessions, please restart ecFlowUI with the -s command-line option"));
 }
 
@@ -345,6 +346,12 @@ void MainWindow::on_actionAbout_triggered()
     d.exec();
 }
 
+void MainWindow::on_actionShortcutHelp_triggered()
+{
+    ShortcutHelpDialog d;
+    d.exec();
+}
+
 void MainWindow::on_actionSaveSessionAs_triggered()
 {
     SaveSessionAsDialog d;
@@ -373,7 +380,11 @@ void MainWindow::slotSelectionChanged(VInfo_ptr info)
 
     //Get the set of visible info panel tabs for the selection
 	std::vector<InfoPanelDef*> ids;
-	InfoPanelHandler::instance()->visible(selection_,ids);
+    if(info && info->isAttribute()) {
+        InfoPanelHandler::instance()->visible(VInfo::createParent(info), ids);
+    } else {
+        InfoPanelHandler::instance()->visible(selection_,ids);
+    }
 
     //Set status of the info panel actions in the toolbar accordingly
 	Q_FOREACH(QAction* ac,infoPanelActions_)
@@ -401,7 +412,7 @@ void MainWindow::slotSelectionChanged(VInfo_ptr info)
 
 void MainWindow::updateRefreshActions()
 {
-    ServerHandler* s=0;
+    ServerHandler* s=nullptr;
 
     QString serverName;
     if(selection_)
@@ -413,7 +424,7 @@ void MainWindow::updateRefreshActions()
 
     if(s && s->isEnabled())
     {
-        bool hasSel=(selection_ != 0);
+        bool hasSel=(selection_ != nullptr);
         actionRefreshSelected->setEnabled(hasSel);
         actionResetSelected->setEnabled(hasSel);
     }
@@ -427,7 +438,7 @@ void MainWindow::updateRefreshActions()
 
 void MainWindow::slotOpenInfoPanel()
 {
-	if(QAction* ac=static_cast<QAction*>(sender()))
+	if(auto* ac=static_cast<QAction*>(sender()))
 	{
 		std::string name=ac->data().toString().toStdString();
 		nodePanel_->openDialog(selection_,name);
@@ -447,7 +458,7 @@ void MainWindow::rerenderContents()
 
 void MainWindow::slotContentsChanged()
 {
-	MainWindow::saveContents(NULL);
+	MainWindow::saveContents(nullptr);
 }
 
 bool MainWindow::selectInTreeView(VInfo_ptr info)
@@ -556,7 +567,7 @@ void MainWindow::writeSettings(VComboSettings *vs)
 
 void MainWindow::readSettings(VComboSettings *vs)
 {
-	int cnt=vs->get<int>("infoPanelCount",0);
+	auto cnt=vs->get<int>("infoPanelCount",0);
 	for(int i=0; i < cnt ; i++)
 	{
 		//addInfoPanel();
@@ -607,7 +618,7 @@ MainWindow* MainWindow::makeWindow(QString id)
 
 MainWindow* MainWindow::makeWindow(QStringList idLst)
 {
-	MainWindow *win=new MainWindow(idLst);
+	auto *win=new MainWindow(idLst);
 	windows_ << win;
 	return win;
 }
@@ -675,7 +686,7 @@ bool MainWindow::aboutToClose(MainWindow* win)
 		  	int tabCnt=win->nodePanel_->count();
 			if(tabCnt > 1)
 			{
-		  		if(QMessageBox::question(0,tr("Confirm close"),tr("You are about to close <b>") + QString::number(tabCnt) + tr("</b> tabs. Are you sure you want to continue?"),
+		  		if(QMessageBox::question(nullptr,tr("Confirm close"),tr("You are about to close <b>") + QString::number(tabCnt) + tr("</b> tabs. Are you sure you want to continue?"),
 			    	      QMessageBox::Yes | QMessageBox::Cancel,QMessageBox::Cancel) == QMessageBox::Cancel)
 				{
 					return false;
@@ -716,7 +727,7 @@ bool MainWindow::aboutToQuit(MainWindow* topWin)
 		{
 			if (si->askToPreserveTemporarySession())
 			{
-				if(QMessageBox::question(0,tr("Delete temporary session?"),
+				if(QMessageBox::question(nullptr,tr("Delete temporary session?"),
 							tr("This was a temporary session - would you like to preserve it for future use?"),
 							QMessageBox::Yes | QMessageBox::No,QMessageBox::No) == QMessageBox::No)
 					SessionHandler::destroyInstance();
@@ -727,6 +738,9 @@ bool MainWindow::aboutToQuit(MainWindow* topWin)
 			}
 		}
 
+        // all qt-based classes can decect the the app is quitting
+        qApp->setProperty("inQuit","1");
+
         //Ensure the ServerHandler destructors are called
         MainWindow::cleanUpOnQuit(topWin);
 
@@ -754,8 +768,8 @@ void MainWindow::init()
 	}
 
 	//Get number of windows and topWindow index.
-	int cnt=vs.get<int>("windowCount",0);
-	int topWinId=vs.get<int>("topWindowId",-1);
+	auto cnt=vs.get<int>("windowCount",0);
+	auto topWinId=vs.get<int>("topWindowId",-1);
 
 	if(cnt > maxWindowNum_)
 	{
@@ -808,6 +822,9 @@ void MainWindow::save(MainWindow *topWin)
     //Save server list
 	ServerHandler::saveSettings();
 
+    //Save attribute type list
+    VAttributeType::saveLastNames();
+
     //Save icon name list
     VIcon::saveLastNames();
 }
@@ -861,7 +878,7 @@ MainWindow* MainWindow::findWindow(QWidget *childW)
 			return w;
 	}
 
-	return 0;
+	return nullptr;
 }
 
 MainWindow* MainWindow::firstWindow()
@@ -879,7 +896,7 @@ void MainWindow::startPreferences(MainWindow *w,QString option)
 {
     Q_ASSERT(w);
 
-    PropertyDialog* d=new PropertyDialog; //belongs to the whole app
+    auto* d=new PropertyDialog; //belongs to the whole app
 
     d->showPage(option);
 
@@ -918,8 +935,7 @@ MainWindowTitleHandler::MainWindowTitleHandler(MainWindow *win) : win_(win)
 }
 
 MainWindowTitleHandler::~MainWindowTitleHandler()
-{
-}
+= default;
 
 void MainWindowTitleHandler::update(ServerHandler *sh)
 {
@@ -933,7 +949,7 @@ void MainWindowTitleHandler::update()
     Q_ASSERT(win_);
 
     char *userTitle = getenv("ECFLOWUI_TITLE");
-    std::string mainTitle = (userTitle != NULL) ? std::string(userTitle) + " (" + ecf::Version::raw() + ")"
+    std::string mainTitle = (userTitle != nullptr) ? std::string(userTitle) + " (" + ecf::Version::raw() + ")"
                                                 : VConfig::instance()->appLongName();
     QString title=QString::fromStdString(mainTitle);
 
