@@ -55,15 +55,15 @@ class MainWindow : public QMainWindow, private Ui::MainWindow
     Q_OBJECT
 
 public:
-    MainWindow(QStringList,QWidget *parent=0);
-    ~MainWindow();
+    MainWindow(QStringList,QWidget *parent=nullptr);
+    ~MainWindow() override;
     
     ServerHandler* selectedServer() const;
 
     static void init();
     static void showWindows();
-    static void openWindow(QString id,QWidget *fromW=0);
-    static void openWindow(QStringList id,QWidget *fromW=0);
+    static void openWindow(QString id,QWidget *fromW=nullptr);
+    static void openWindow(QStringList id,QWidget *fromW=nullptr);
     static void reload();
     static void saveSession(SessionItem*);
     static void lookUpInTree(VInfo_ptr);
@@ -89,7 +89,8 @@ protected Q_SLOTS:
 	void on_actionSearch_triggered();
     void on_actionNotification_triggered();
     void on_actionCommandOutput_triggered();
-	void on_actionAbout_triggered();
+    void on_actionAbout_triggered();
+    void on_actionShortcutHelp_triggered();
 	void on_actionSaveSessionAs_triggered();
 	void on_actionManageSessions_triggered();
 
@@ -104,7 +105,7 @@ protected Q_SLOTS:
 
 private:
     void init(MainWindow*);
-    void closeEvent(QCloseEvent*);
+    void closeEvent(QCloseEvent*) override;
     void addInfoPanelActions(QToolBar *toolbar);
     void reloadContents();
     void rerenderContents();
