@@ -24,18 +24,18 @@ class NodeSearchDialog : public QDialog, protected Ui::NodeSearchDialog
     Q_OBJECT
 
 public:
-    explicit NodeSearchDialog(QWidget *parent = 0);
-    ~NodeSearchDialog();
+    explicit NodeSearchDialog(QWidget *parent = nullptr);
+    ~NodeSearchDialog() override;
 
     NodeSearchWidget* queryWidget() const;
 
 protected Q_SLOTS:
-	void accept();
-    void reject();
+	void accept() override;
+    void reject() override;
     void slotOwnerDelete();
 
 protected:
-	void closeEvent(QCloseEvent * event);
+	void closeEvent(QCloseEvent * event) override;
 
 private:
     void readSettings();
