@@ -10,6 +10,7 @@
 #include "ServerComThread.hpp"
 
 #include "Defs.hpp"
+#include "Suite.hpp"
 #include "ClientInvoker.hpp"
 #include "ArgvCreator.hpp"
 
@@ -56,24 +57,28 @@ void ServerComThread::task(VTask_ptr task)
         //We set the parameters needed to run the task. These members are not protected by
         //a mutex, because apart from this function only run() can access them!!
         serverName_=server_->longName();
-        command_=task->command();
-        params_=task->params();
-        contents_=task->contents();
-        vars_=task->vars();
-        nodePath_.clear();
         taskType_=task->type();
-        nodePath_=task->targetPath();
-        zombie_=task->zombie();
 
-        //Suite filter
-        hasSuiteFilter_=server_->suiteFilter()->isEnabled();
-        autoAddNewSuites_=server_->suiteFilter()->autoAddNewSuites();
-        if(hasSuiteFilter_)
-            filteredSuites_=server_->suiteFilter()->filter();
-        else
-            filteredSuites_.clear();
+        if (taskType_ != VTask::LogOutTask) {
 
-        maxLineNum_=server_->conf()->intValue(VServerSettings::MaxOutputFileLines);
+            command_=task->command();
+            params_=task->params();
+            contents_=task->contents();
+            vars_=task->vars();
+            nodePath_.clear();
+            nodePath_=task->targetPath();
+            zombie_=task->zombie();
+
+            //Suite filter
+            hasSuiteFilter_=server_->suiteFilter()->isEnabled();
+            autoAddNewSuites_=server_->suiteFilter()->autoAddNewSuites();
+            if(hasSuiteFilter_)
+                filteredSuites_=server_->suiteFilter()->filter();
+            else
+                filteredSuites_.clear();
+
+            maxLineNum_=server_->conf()->intValue(VServerSettings::MaxOutputFileLines);
+        }
 
         //Start the thread execution
         start();
@@ -91,175 +96,247 @@ void ServerComThread::run()
 
     try
     {
-        switch (taskType_)
+        // define the tasks that will explicitly/implicitly call ci_->sync_local()
+        std::set<VTask::Type> sync_tasks = {VTask::CommandTask, VTask::SyncTask,
+                      VTask::WhySyncTask,VTask::ScriptSubmitTask,VTask::ZombieCommandTask,
+                      VTask::JobStatusTask, VTask::PlugTask};
+
+        // this is called during reset - it requires a special treatment
+        if(taskType_  == VTask::ResetTask)
         {
-            case VTask::CommandTask:
-            {
-                UiLog(serverName_).dbg() << " COMMAND";
-                //special treatment for variable add/change to allow values with "--"  characters.
-                //See issue ECFLOW-1414. The command_ string is supposed to contain these values:
-                //ecflow_client --alter change variable NAME VALUE PATH
-                //note that we can assume there is a space, and not '=' after '--alter',
-                //as this is taken care of in CommandHandler.cpp
-                if(command_.size() >=6 && command_[1] == "--alter" && command_[3] == "variable" &&
-                   (command_[2] == "change" || command_[2] == "add"))
-                {
-                    std::vector<std::string> nodeList(command_.begin()+6, command_.end());
-                    ci_->alter(nodeList,command_[2],command_[3],command_[4],command_[5]);
-                }
+            UiLog(serverName_).dbg() << " RESET";
+            reset();
+        }
 
-                // call the client invoker with the saved command
-                else
+        // logout is also special!!!
+        else if(taskType_  == VTask::LogOutTask)
+        {
+            UiLog(serverName_).dbg() << " LOGOUT";
+            detach();
+            if(ci_->client_handle() > 0)
+            {
+                ci_->ch1_drop();
+            }
+        }
+
+        // this is very important to exclude non-compatible servers
+        else if(taskType_  == VTask::ServerVersionTask)
+        {
+            UiLog(serverName_).dbg() << " SERVER_VERSION";
+            ci_->server_version();
+            UiLog(serverName_).dbg() << " SERVER_VERSION FINISHED";
+        }
+        // tasks that will explicitly/implicitly call ci_->sync_local()
+        else if(sync_tasks.find(taskType_) != sync_tasks.end())
+        {
+            //we need to lock the mutex on the defs
+            ServerDefsAccess defsAccess(server_);
+
+            switch(taskType_)
+            {
+                case VTask::CommandTask:
                 {
-                    ArgvCreator argvCreator(command_);
+                    //will automatically call ci_->sync_local() after the
+                    //command finished!!!
+
+                    UiLog(serverName_).dbg() << " COMMAND";
+                    //special treatment for variable add/change to allow values with "--"  characters.
+                    //See issue ECFLOW-1414. The command_ string is supposed to contain these values:
+                    //ecflow_client --alter change variable NAME VALUE PATH
+                    if(command_.size() >=7 && command_[1] == "--alter" && command_[3] == "variable" &&
+                       (command_[2] == "change" || command_[2] == "add"))
+                    {
+                        ci_->alter(command_[6],command_[2],command_[3],command_[4],command_[5]);
+                    }
+
+                    // call the client invoker with the saved command
+                    else
+                    {
+                        ArgvCreator argvCreator(command_);
 #ifdef _UI_SERVERCOMTHREAD_DEBUG
-                    UiLog(serverName_).dbg() << " args="  << argvCreator.toString();
+                        UiLog(serverName_).dbg() << " args="  << argvCreator.toString();
 #endif
-                    ci_->invoke(argvCreator.argc(), argvCreator.argv());
+                        ci_->invoke(argvCreator.argc(), argvCreator.argv());
+                    }
+                    break;
                 }
 
-                break;
-            }
+                case VTask::SyncTask:
+                {
+                    UiLog(serverName_).dbg() << " SYNC";
+                    UiLog(serverName_).dbg() << " sync begin";
+                    ci_->sync_local();
+                    UiLog(serverName_).dbg() << " sync end";
+                    break;
+                }
 
-            case VTask::NewsTask:
-            {
-                UiLog(serverName_).dbg() << " NEWS";
-                ci_->news_local(); // call the server
-                break;
-            }
+                case VTask::WhySyncTask:
+                {
+                    UiLog(serverName_).dbg() << " WHYSYNC";
+                    UiLog(serverName_).dbg() <<  "sync begin";
+                    ci_->sync_local(true); //sync_suite_clock
+                    UiLog(serverName_).dbg() << " sync end";
+                    break;
+                }
 
-            case VTask::SyncTask:
-            {
-                UiLog(serverName_).dbg() << " SYNC";
-                sync_local();
-                break;
-            }
-
-            //This is called during reset
-            case VTask::ResetTask:
-            {
-                UiLog(serverName_).dbg() << " SYNC";
-                reset();
-                break;
-            }
-
-            // this is very important to exclude non-compatible servers
-            case VTask::ServerVersionTask:
-            {
-                 UiLog(serverName_).dbg() << " SERVER_VERSION";
-                 ci_->server_version();
-                 break;
-            }
-
-            case VTask::JobTask:
-            case VTask::ManualTask:
-            case VTask::ScriptTask:
-            case VTask::OutputTask:
-            {
-                UiLog(serverName_).dbg() << " FILE" << " " << params_["clientPar"];
-                if(maxLineNum_ < 0)
-                    ci_->file(nodePath_,params_["clientPar"]);
-                else
-                    ci_->file(nodePath_,params_["clientPar"],boost::lexical_cast<std::string>(maxLineNum_));
-
-                break;
-            }
-
-            case VTask::MessageTask:
-            {
-                UiLog(serverName_).dbg() << " EDIT HISTORY";
-                ci_->edit_history(nodePath_);
-                break;
-            }
-
-            case VTask::StatsTask:
-            {
-                UiLog(serverName_).dbg() << " STATS";
-                ci_->stats();
-                break;
-            }
-
-            case VTask::HistoryTask:
-            {
-                UiLog(serverName_).dbg() << " SERVER LOG";
-                ci_->getLog(100);
-                break;
-            }
-
-            case VTask::ScriptPreprocTask:
-                UiLog(serverName_).dbg() << " SCRIP PREPROCESS";
-                ci_->edit_script_preprocess(nodePath_);
-                break;
-
-            case VTask::ScriptEditTask:
-                UiLog(serverName_).dbg() << " SCRIP EDIT";
-                ci_->edit_script_edit(nodePath_);
-                break;
-
-            case VTask::ScriptSubmitTask:
-                UiLog(serverName_).dbg() << " SCRIP SUBMIT";
-                ci_->edit_script_submit(nodePath_, vars_, contents_,
+                case VTask::ScriptSubmitTask:
+                {
+                    UiLog(serverName_).dbg() << " SCRIP SUBMIT";
+                    ci_->edit_script_submit(nodePath_, vars_, contents_,
                         (params_["alias"]=="1")?true:false,
                         (params_["run"] == "1")?true:false);
-                break;
-
-            case VTask::SuiteListTask:
-                UiLog(serverName_).dbg() << " SUITES";
-                ci_->suites();
-                break;
-
-            case VTask::SuiteAutoRegisterTask:
-                UiLog(serverName_).dbg() << " SUITE AUTO REGISTER";
-                if(hasSuiteFilter_)
-                {
-                    ci_->ch1_auto_add(autoAddNewSuites_);
+                    break;
                 }
-                break;
 
-            case VTask::ZombieCommandTask:
-            {
-                std::string cmd=params_["command"];
-                UiLog(serverName_).dbg() << " ZOMBIE COMMAND " << cmd << " path=" << zombie_.path_to_task();
-                if(cmd == "zombie_fob")
-                    ci_->zombieFob(zombie_);
-                else if(cmd == "zombie_fail")
-                    ci_->zombieFail(zombie_);
-                else if(cmd == "zombie_adopt")
-                    ci_->zombieAdopt(zombie_);
-                else if(cmd == "zombie_block")
-                    ci_->zombieBlock(zombie_);
-                else if(cmd == "zombie_remove")
-                    ci_->zombieRemove(zombie_);
-                else if(cmd == "zombie_kill")
-                    ci_->zombieKill(zombie_);
+                case VTask::ZombieCommandTask:
+                {
+                    std::string cmd=params_["command"];
+                    UiLog(serverName_).dbg() << " ZOMBIE COMMAND " << cmd << " path=" << zombie_.path_to_task();
+                    if(cmd == "zombie_fob")
+                        ci_->zombieFob(zombie_);
+                    else if(cmd == "zombie_fail")
+                        ci_->zombieFail(zombie_);
+                    else if(cmd == "zombie_adopt")
+                        ci_->zombieAdopt(zombie_);
+                    else if(cmd == "zombie_block")
+                        ci_->zombieBlock(zombie_);
+                    else if(cmd == "zombie_remove")
+                        ci_->zombieRemove(zombie_);
+                    else if(cmd == "zombie_kill")
+                        ci_->zombieKill(zombie_);
+
+                    break;
+                }
+                case VTask::JobStatusTask:
+                {
+                    UiLog(serverName_).dbg() << " JOB STATUS";
+                    ci_->status(nodePath_);
+                    break;
+                }
+                case VTask::PlugTask:
+                {
+                    UiLog(serverName_).dbg() << " PLUG";
+                    ci_->plug(params_["source"], params_["target"]);
+                    break;
+                }
+
+                default:
+                {
+                    Q_ASSERT(0);
+                    exit(1);
+                 }
 
-                break;
             }
 
-            case VTask::ZombieListTask:
-                UiLog(serverName_).dbg() << " ZOMBIES";
-                ci_->zombieGet();
-                break;
+            //ci_->sync_local() was called!!
+            //If a rescan or fullscan is needed we have either added/remove nodes or deleted the defs.
+            //So there were significant changes.
+
+            //We detach the nodes currently available in defs, then we attach them again. We can still have nodes
+            //that were removed from the defs but are still attached. These will be detached when in ServerHandler we
+            //clear the tree. This tree contains shared pointers to the nodes, so when the tree is cleared
+            //the shared pointer are reset, the node descturctor is called and finally update_delete is called and
+            //we can detach the node.
+
+            if(rescanNeed_ || ci_->server_reply().full_sync())
+            {
+                UiLog(serverName_).dbg() << " rescan needed!";
+                detach(defsAccess.defs());
+                attach(defsAccess.defs());
+            }
 
-            case VTask::LogOutTask:
-                UiLog(serverName_).dbg() << " LOGOUT";
-                detach();
-                if(ci_->client_handle() > 0)
-                {
-                    ci_->ch1_drop();
-                }
-                break;
-            default:
-                break;
         }
-    }
+
+        // all the other tasks - these will not call ci_->sync_local(()
+        else
+        {
+
+            switch(taskType_)
+            {
+
+                case VTask::NewsTask:
+                {
+                    UiLog(serverName_).dbg() << " NEWS";
+                    ci_->news_local();
+                    break;
+                }
+
+                case VTask::JobTask:
+                case VTask::ManualTask:
+                case VTask::ScriptTask:
+                case VTask::OutputTask:
+                case VTask::JobStatusFileTask:
+                {
+                    UiLog(serverName_).dbg() << " FILE" << " " << params_["clientPar"];
+                    if(maxLineNum_ < 0)
+                        ci_->file(nodePath_,params_["clientPar"]);
+                    else
+                        ci_->file(nodePath_,params_["clientPar"],boost::lexical_cast<std::string>(maxLineNum_));
+
+                    break;
+                }
+
+                case VTask::MessageTask:
+                    UiLog(serverName_).dbg() << " EDIT HISTORY";
+                    ci_->edit_history(nodePath_);
+                    break;
+
+                case VTask::StatsTask:
+                    UiLog(serverName_).dbg() << " STATS";
+                    ci_->stats();
+                    break;;
+
+                case VTask::HistoryTask:
+                    UiLog(serverName_).dbg() << " SERVER LOG";
+                    ci_->getLog(100);
+                    break;
+
+                case VTask::ScriptPreprocTask:
+                    UiLog(serverName_).dbg() << " SCRIP PREPROCESS";
+                    ci_->edit_script_preprocess(nodePath_);
+                    break;
+
+                case VTask::ScriptEditTask:
+                    UiLog(serverName_).dbg() << " SCRIP EDIT";
+                    ci_->edit_script_edit(nodePath_);
+                    break;
+
+                case VTask::SuiteListTask:
+                    UiLog(serverName_).dbg() << " SUITES";
+                    ci_->suites();
+                    break;
+
+                case VTask::SuiteAutoRegisterTask:
+                {
+                    UiLog(serverName_).dbg() << " SUITE AUTO REGISTER";
+                    if(hasSuiteFilter_)
+                    {
+                        ci_->ch1_auto_add(autoAddNewSuites_);
+                    }
+                    break;
+                }
+
+                case VTask::ZombieListTask:
+                    UiLog(serverName_).dbg() << " ZOMBIES";
+                    ci_->zombieGet();
+                    break;
+
+                default:
+                    break;
+
+           } //switch
+
+        } //if
+
+    } //try
+
     catch(std::exception& e)
     {
         isMessage = true;
         errorString = e.what();
     }
 
-    // we can get an error string in one of two ways - either an exception is raised, or
+    // we can get an error string in two ways - either an exception is raised, or
     // the get_string() of the server reply is non-empty.
     if (!isMessage && (taskType_ == VTask::CommandTask) && !(ci_->server_reply().get_string().empty()))
     {
@@ -267,12 +344,18 @@ void ServerComThread::run()
         errorString = ci_->server_reply().get_string();
     }
 
+    //we  failed or we have a string returned
     if (isMessage)
     {
         // note that we need to emit a signal rather than directly call a message function
         // because we can't call Qt widgets from a worker thread
 
         UiLog(serverName_).dbg() << " thread failed: " <<  errorString;
+        if (taskType_ == VTask::LogOutTask) {
+            Q_EMIT logoutDone();
+            return;
+        }
+
         Q_EMIT failed(errorString);
 
         //Reset flags
@@ -282,6 +365,12 @@ void ServerComThread::run()
         return;
     }
 
+    if (taskType_ == VTask::LogOutTask) {
+        Q_EMIT logoutDone();
+        return;
+    }
+
+
     //Reset flags
     rescanNeed_=false;
 
@@ -289,15 +378,15 @@ void ServerComThread::run()
     //UserMessage::message(UserMessage::DBG, false, std::string("  ServerComThread::run finished"));
 }
 
-
-void ServerComThread::sync_local()
+#if 0
+void ServerComThread::sync_local(bool sync_suite_clock)
 {
     //For this part we need to lock the mutex on defs
     {
         ServerDefsAccess defsAccess(server_);
 
         UiLog(serverName_).dbg() << "ComThread::sync_local --> sync begin";
-        ci_->sync_local();
+        ci_->sync_local(sync_suite_clock);
         UiLog(serverName_).dbg() << " sync end";
 
         //If a rescan or fullscan is needed we have either added/remove nodes or deleted the defs.
@@ -317,6 +406,7 @@ void ServerComThread::sync_local()
         }
     }
 }
+#endif
 
 void ServerComThread::reset()
 {
@@ -328,30 +418,17 @@ void ServerComThread::reset()
     //Detach the defs and the nodes from the observer
     detach(defsAccess.defs());
 
-    //We drop the handle belonging to the current client!
-    if(ci_->client_handle() > 0)
-    {
-        try
-        {
-            ci_->ch1_drop();
-        }
-        catch (std::exception &e)
-        {
-            UiLog(serverName_).warn() << " cannot drop handle for client: " << e.what();
-        }
-    }
+    // reset client handle + defs
+    ci_->reset();
 
     if(hasSuiteFilter_)
     {
-        //reset client handle + defs
-        ci_->reset();
-
         if(!filteredSuites_.empty())
         {
             UiLog(serverName_).dbg() << " register suites=" << filteredSuites_;
 
             //This will add a new handle to the client
-            ci_->ch_register(autoAddNewSuites_, filteredSuites_);
+            ci_->ch_register(autoAddNewSuites_, filteredSuites_);  // will drop handle in server and auto_sync if enabled
         }
         //If the suite filter is empty
         else
@@ -363,19 +440,17 @@ void ServerComThread::reset()
 
             std::vector<std::string> fsl;
             fsl.push_back(SuiteFilter::dummySuite());
-            ci_->ch_register(autoAddNewSuites_, fsl);
+            ci_->ch_register(autoAddNewSuites_, fsl);   // will drop handle in server, and auto_sync if enabled
         }
     }
     else
     {
-        // reset client handle + defs
-        ci_->reset();
+        UiLog(serverName_).dbg() << " sync begin";
+        ci_->sync_local();
+        //if (!ci_->is_auto_sync_enabled())  ci_->sync_local();  // temp
+        UiLog(serverName_).dbg() << " sync end";
     }
 
-    UiLog(serverName_).dbg() << " sync begin";
-    ci_->sync_local();
-    UiLog(serverName_).dbg() << " sync end";
-
     //Attach the nodes to the observer
     attach(defsAccess.defs());
 
@@ -386,7 +461,10 @@ void ServerComThread::reset()
 void ServerComThread::update(const Node* node, const std::vector<ecf::Aspect::Type>& types)
 {
     //This function can only be called during a SYNC_LOCAl task!!!!
-    assert(taskType_ == VTask::SyncTask);
+    assert(taskType_ == VTask::CommandTask ||
+           taskType_ == VTask::SyncTask || taskType_ == VTask::WhySyncTask ||
+           taskType_ == VTask::JobStatusTask || taskType_ == VTask::ScriptSubmitTask ||
+           taskType_ == VTask::ZombieCommandTask);
 
     std::vector<ecf::Aspect::Type> typesCopy=types;
 
@@ -462,7 +540,7 @@ void ServerComThread::update(const Defs* dc, const std::vector<ecf::Aspect::Type
 
 void ServerComThread::update_delete(const Node* nc)
 {
-    Node *n=const_cast<Node*>(nc);
+    auto *n=const_cast<Node*>(nc);
     n->detach(this);
 }
 
@@ -472,7 +550,7 @@ void ServerComThread::update_delete(const Defs* dc)
 {
     UiLog(serverName_).dbg() << "ServerComThread::update_delete -->";
 
-    Defs *d=const_cast<Defs*>(dc);
+    auto *d=const_cast<Defs*>(dc);
     d->detach(this);
 
     //If we are in  a SYNC_LOCAl task!!!!
@@ -492,7 +570,7 @@ void ServerComThread::attach()
 {
     ServerDefsAccess defsAccess(server_);  // will reliquish its resources on destruction
     defs_ptr d = defsAccess.defs();
-    if(d == NULL)
+    if(d == nullptr)
         return;
 
     attach(d);
@@ -502,15 +580,15 @@ void ServerComThread::attach()
 //the defs is safe so we do not need to set a mutex on it.
 void ServerComThread::attach(defs_ptr d)
 {
-    if(d == NULL)
+    if(d == nullptr)
         return;
 
     d->attach(this);
 
     const std::vector<suite_ptr> &suites = d->suiteVec();
-    for(unsigned int i=0; i < suites.size();i++)
+    for(const auto & suite : suites)
     {
-        attach(suites.at(i).get());
+        attach(suite.get());
     }
 }
 
@@ -533,7 +611,7 @@ void ServerComThread::detach()
 {
     ServerDefsAccess defsAccess(server_);  // will reliquish its resources on destruction
     defs_ptr d = defsAccess.defs();
-    if(d == NULL)
+    if(d == nullptr)
         return;
 
     detach(d);
@@ -544,15 +622,15 @@ void ServerComThread::detach()
 //the defs is safe so we do not need to set a mutex on it.
 void ServerComThread::detach(defs_ptr d)
 {
-    if(d == NULL)
+    if(d == nullptr)
         return;
 
     d->detach(this);
 
     const std::vector<suite_ptr> &suites = d->suiteVec();
-    for(unsigned int i=0; i < suites.size();i++)
+    for(const auto & suite : suites)
     {
-        detach(suites.at(i).get());
+        detach(suite.get());
     }
 }
 
@@ -572,10 +650,23 @@ void ServerComThread::detach(Node *node)
 
 void ServerComThread::aspectToStr(std::stringstream& ss,const std::vector<ecf::Aspect::Type>& t) const
 {
-    for(std::vector<ecf::Aspect::Type>::const_iterator it=t.begin(); it != t.end(); ++it)
+    for(auto it : t)
     {
         if(!ss.str().empty())
             ss << ",";
-        ss << *it;
+        ss << it;
+    }
+}
+
+//performa a logout task without running a thread
+void ServerComThread::blockingLogout()
+{
+    Q_ASSERT(isRunning() == false);
+
+    UiLog(serverName_).dbg() << " LOGOUT";
+    detach();
+    if(ci_->client_handle() > 0)
+    {
+        ci_->ch1_drop();
     }
 }
