@@ -42,13 +42,12 @@
 
 TimelineInfoModel::TimelineInfoModel(QObject *parent) :
           QAbstractItemModel(parent),
-          data_(0), firstRowInPeriod_(-1), lastRowInPeriod_(-1)
+          data_(nullptr), firstRowInPeriod_(-1), lastRowInPeriod_(-1)
 {
 }
 
 TimelineInfoModel::~TimelineInfoModel()
-{
-}
+= default;
 
 void TimelineInfoModel::setData(TimelineItem *data,unsigned int viewStartDateSec,unsigned int viewEndDateSec,
                                 unsigned int endDateSec)
@@ -87,7 +86,7 @@ void TimelineInfoModel::determineRowsInPeriod()
 
     if(firstRowInPeriod_ != -11)
     {
-        for(unsigned int i=static_cast<unsigned int>(firstRowInPeriod_); i < data_->size(); i++)
+        for(auto i=static_cast<unsigned int>(firstRowInPeriod_); i < data_->size(); i++)
         {
             if(viewEndDateSec_ <= data_->start_[i])
             {
@@ -101,7 +100,7 @@ void TimelineInfoModel::determineRowsInPeriod()
 void TimelineInfoModel::clearData()
 {
     beginResetModel();
-    data_=0;
+    data_=nullptr;
     firstRowInPeriod_=-1;
     lastRowInPeriod_=-1;
     endResetModel();
@@ -109,7 +108,7 @@ void TimelineInfoModel::clearData()
 
 bool TimelineInfoModel::hasData() const
 {
-    return (data_ != NULL);
+    return (data_ != nullptr);
 }
 
 int TimelineInfoModel::columnCount( const QModelIndex& /*parent */) const
@@ -247,7 +246,7 @@ QModelIndex TimelineInfoModel::index( int row, int column, const QModelIndex & p
 {
     if(!hasData() || row < 0 || column < 0)
     {
-        return QModelIndex();
+        return {};
     }
 
     //When parent is the root this index refers to a node or server
@@ -256,13 +255,13 @@ QModelIndex TimelineInfoModel::index( int row, int column, const QModelIndex & p
         return createIndex(row,column);
     }
 
-    return QModelIndex();
+    return {};
 
 }
 
 QModelIndex TimelineInfoModel::parent(const QModelIndex &child) const
 {
-    return QModelIndex();
+    return {};
 }
 
 
@@ -291,7 +290,7 @@ TimelineInfoWidget::TimelineInfoWidget(QWidget *parent) :
     ui_->titleLabel->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);
 
     //Time tree
-    TimelineInfoDelegate *delegate=new TimelineInfoDelegate(this);
+    auto *delegate=new TimelineInfoDelegate(this);
     ui_->timeTree->setItemDelegate(delegate);
 
     model_=new TimelineInfoModel(this);
@@ -371,7 +370,7 @@ void TimelineInfoWidget::createSummary()
 
     s+="<table width=\'100%\'>";
 
-    s+="<tr><td class=\'title\' align=\'center\' colspan=\'2\' bgcolor=\'"  + QColor(238,238,238).name() +
+    s+=R"(<tr><td class='title' align='center' colspan='2' bgcolor=')"  + QColor(238,238,238).name() +
             "\'><b>Statistics for the whole log period</b></td></tr>";
 
     createSummary(s,VNState::find("active"));
@@ -399,7 +398,7 @@ void TimelineInfoWidget::createSummary(QString &txt,VNState* state)
     if(num <=0)
         return;
 
-    txt+="<tr><td class=\'title\' bgcolor=\'" + state->colour().name() + "\' align=\'center\' colspan=\'2\' >" +
+    txt+=R"(<tr><td class='title' bgcolor=')" + state->colour().name() + R"(' align='center' colspan='2' >)" +
             Viewer::formatText(state->label(),state->fontColour()) + "</td></tr>";
 
     txt+=Viewer::formatTableRow("Number",QString::number(num),true);
@@ -415,7 +414,7 @@ void TimelineInfoWidget::createSummary(QString &txt,VNState* state)
         QBuffer buffer(&byteArray);
         pix.save(&buffer, "PNG");
 
-        txt+="<tr><td align=\'center\' colspan=\'2\'><img src=\"data:image/png;base64," +
+        txt+=R"(<tr><td align='center' colspan='2'><img src="data:image/png;base64,)" +
             byteArray.toBase64() + "\"/></td></tr>";
     }
 }
@@ -486,13 +485,13 @@ TimelineInfoDialog::TimelineInfoDialog(QWidget* parent) : QDialog(parent)
 {
     setWindowTitle(tr("ecFlowUI - Timeline details"));
 
-    QVBoxLayout *vb=new QVBoxLayout(this);
+    auto *vb=new QVBoxLayout(this);
     vb->setContentsMargins(4,4,4,4);
 
     infoW_=new TimelineInfoWidget(this);
     vb->addWidget(infoW_);
 
-    QDialogButtonBox *buttonBox=new QDialogButtonBox(this);
+    auto *buttonBox=new QDialogButtonBox(this);
     vb->addWidget(buttonBox);
 
     buttonBox->addButton(QDialogButtonBox::Close);
