@@ -63,7 +63,7 @@ TimelineDelegate::TimelineDelegate(TimelineModel *model,QWidget *parent) :
     //Property
     if(propVec.empty())
     {
-        propVec.push_back("view.table.font");
+        propVec.emplace_back("view.table.font");
 
         //Base settings
         //addBaseSettings(propVec);
@@ -75,8 +75,7 @@ TimelineDelegate::TimelineDelegate(TimelineModel *model,QWidget *parent) :
 }
 
 TimelineDelegate::~TimelineDelegate()
-{
-}
+= default;
 
 void TimelineDelegate::notifyChange(VProperty* p)
 {
@@ -104,7 +103,7 @@ void TimelineDelegate::updateSettings()
 QSize TimelineDelegate::sizeHint(const QStyleOptionViewItem & option, const QModelIndex & index ) const
 {
     QSize size=QStyledItemDelegate::sizeHint(option,index);
-    return QSize(size.width(),fm_.height() + topPadding_ + bottomPadding_);
+    return {size.width(),fm_.height() + topPadding_ + bottomPadding_};
     return size;
 }
 
@@ -566,10 +565,10 @@ int TimelineDelegate::getDurationMaxTextWidth(int duration) const
 TimelineView::TimelineView(TimelineSortModel* model,QWidget* parent) :
      QTreeView(parent),
      model_(model),
-     header_(NULL),
+     header_(nullptr),
      headerBeingAdjusted_(false),
      needItemsLayout_(false),
-     prop_(NULL),
+     prop_(nullptr),
      setCurrentIsRunning_(false),
      viewMode_(TimelineMode),
      durationColumnWidthInitialised_(false)
@@ -641,7 +640,7 @@ TimelineView::TimelineView(TimelineSortModel* model,QWidget* parent) :
 
     //Properties
     std::vector<std::string> propVec;
-    propVec.push_back("view.table.background");
+    propVec.emplace_back("view.table.background");
     prop_=new PropertyMapper(propVec,this);
 
     //Initialise bg
@@ -756,9 +755,9 @@ void TimelineView::handleContextMenu(QModelIndex indexClicked,QModelIndexList in
     if(!indexClicked.isValid())
         return;
 
-    QMenu *menu=new QMenu(this);
+    auto *menu=new QMenu(this);
 
-    QAction *ac=new QAction(this);
+    auto *ac=new QAction(this);
     ac->setText(tr("Show details"));
     QFont fBold;
     fBold.setBold(true);
