@@ -7,6 +7,7 @@
 // nor does it submit to any jurisdiction.
 //============================================================================
 
+#include <stdexcept>
 #include "TimelineWidget.hpp"
 
 #include <QtGlobal>
@@ -46,7 +47,7 @@ TimelineWidget::TimelineWidget(QWidget *parent) :
     ui_(new Ui::TimelineWidget),
     maxReadSize_(100*1024*1024),
     logMode_(LatestMode),
-    data_(0),
+    data_(nullptr),
     filterTriggeredByEnter_(false),
     filterTriggerLimit_(200000),
     ignoreTimeEdited_(false),
@@ -56,7 +57,7 @@ TimelineWidget::TimelineWidget(QWidget *parent) :
     localLog_(true),
     logLoaded_(false),
     logTransferred_(false),
-    fileTransfer_(0),
+    fileTransfer_(nullptr),
     detached_(false)
 {
     ui_->setupUi(this);
@@ -108,7 +109,7 @@ TimelineWidget::TimelineWidget(QWidget *parent) :
     ui_->loadFileTb->hide();
 
     // view mode
-    QButtonGroup *viewModeGr = new QButtonGroup(this);
+    auto *viewModeGr = new QButtonGroup(this);
     viewModeGr->addButton(ui_->timelineViewTb,0);
     viewModeGr->addButton(ui_->durationViewTb,1);
     viewModeGr->setExclusive(true);
@@ -130,7 +131,7 @@ TimelineWidget::TimelineWidget(QWidget *parent) :
     ui_->sortCombo->addItem("Sort by tree","tree");
     ui_->sortCombo->setCurrentIndex(0);
 
-    QButtonGroup *sortGr = new QButtonGroup(this);
+    auto *sortGr = new QButtonGroup(this);
     sortGr->addButton(ui_->sortUpTb,0);
     sortGr->addButton(ui_->sortDownTb,1);
     sortGr->setExclusive(true);
