@@ -44,8 +44,8 @@ class TimelineWidget : public QWidget
 Q_OBJECT
 
 public:
-    explicit TimelineWidget(QWidget *parent=0);
-    ~TimelineWidget();
+    explicit TimelineWidget(QWidget *parent=nullptr);
+    ~TimelineWidget() override;
 
     void clear();
     void initLoad(QString serverName, QString host, QString port, QString logFile,
