@@ -14,6 +14,7 @@
 #include <QHeaderView>
 #include <QPalette>
 #include <QScrollBar>
+#include <QShortcut>
 #include <QTime>
 
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
@@ -45,7 +46,7 @@ TreeNodeView::TreeNodeView(AbstractNodeView* view,TreeNodeModel* model,NodeFilte
     view_(view),
     model_(model),
     needItemsLayout_(false),
-    prop_(NULL),
+    prop_(nullptr),
     setCurrentIsRunning_(false),
     setCurrentFromExpandIsRunning_(false),
     canRegainCurrentFromExpand_(true),
@@ -77,18 +78,18 @@ TreeNodeView::TreeNodeView(AbstractNodeView* view,TreeNodeModel* model,NodeFilte
 
     //Properties
     std::vector<std::string> propVec;
-    propVec.push_back("view.tree.indentation");
-    propVec.push_back("view.tree.background");
-    propVec.push_back("view.tree.autoExpandLeafNode");
-    propVec.push_back("view.tree.drawBranchLine");
-    propVec.push_back("view.tree.branchLineColour");
-    propVec.push_back("view.tree.serverToolTip");
-    propVec.push_back("view.tree.nodeToolTip");
-    propVec.push_back("view.tree.attributeToolTip");
+    propVec.emplace_back("view.tree.indentation");
+    propVec.emplace_back("view.tree.background");
+    propVec.emplace_back("view.tree.autoExpandLeafNode");
+    propVec.emplace_back("view.tree.drawBranchLine");
+    propVec.emplace_back("view.tree.branchLineColour");
+    propVec.emplace_back("view.tree.serverToolTip");
+    propVec.emplace_back("view.tree.nodeToolTip");
+    propVec.emplace_back("view.tree.attributeToolTip");
 
     prop_=new PropertyMapper(propVec,this);
 
-    VProperty *prop=0;
+    VProperty *prop=nullptr;
     std::string propName;
 
     //Initialise indentation
@@ -328,6 +329,21 @@ void TreeNodeView::slotDoubleClickItem(const QModelIndex& idx)
     }
 }
 
+void TreeNodeView::slotCommandShortcut()
+{
+    if (auto* sc = static_cast<QShortcut*>(QObject::sender())) {
+        QModelIndexList indexLst=selectedList();
+        std::vector<VInfo_ptr> nodeLst;
+        for(int i=0; i < indexLst.count(); i++)
+        {
+            VInfo_ptr info=model_->nodeInfo(indexLst[i]);
+            if(info && !info->isEmpty())
+                nodeLst.push_back(info);
+        }
+        actionHandler_->runCommand(nodeLst, sc->property("id").toInt());
+    }
+}
+
 void TreeNodeView::slotViewCommand(VInfo_ptr info,QString cmd)
 {
     //Expand all the children of the given node
