@@ -22,23 +22,23 @@ class VariableEditorWidget :  public QWidget, protected Ui::VariableEditorWidget
 {
 friend class VariableEditor;
 public:
-    VariableEditorWidget(QWidget *parent=0);
+    VariableEditorWidget(QWidget *parent=nullptr);
 };
 
 class VariableEditor : public AttributeEditor
 {
 Q_OBJECT
 public:
-    VariableEditor(VInfo_ptr,QWidget* parent=0);
-    ~VariableEditor();
+    VariableEditor(VInfo_ptr,QWidget* parent=nullptr);
+    ~VariableEditor() override;
 
 protected Q_SLOTS:
     void slotValueChanged();
 
 protected:
-    void apply();
-    void resetValue();
-    bool isValueChanged();
+    void apply() override;
+    void resetValue() override;
+    bool isValueChanged() override;
     void readSettings();
     void writeSettings();
 
