@@ -119,7 +119,11 @@ void WhyItemWidget::reload(VInfo_ptr info)
     //set the info. we do not need to observe the node!!!
     info_=info;
 
-    load();
+    //Info must be a node
+    if(info_)
+    {
+        infoProvider_->info(info_);
+    }
 }
 
 void WhyItemWidget::load()
@@ -137,6 +141,27 @@ void WhyItemWidget::clearContents()
 	textEdit_->clear();
 }
 
+void WhyItemWidget::infoReady(VReply* reply)
+{
+    Q_ASSERT(reply);
+    messageLabel_->clear();
+    messageLabel_->hide();
+    load();
+}
+
+void WhyItemWidget::infoFailed(VReply* reply)
+{
+    QString s="Failed to refresh server before building the Why? output.\
+                     Time-related attributes might show out-of-date values.";
+
+    QString err=QString::fromStdString(reply->errorText());
+    if(!err.isEmpty())
+        s = s + "\n" + err;
+
+    messageLabel_->showWarning(s);
+    load();
+}
+
 QString WhyItemWidget::why() const
 {
 	QString s;
@@ -193,9 +218,9 @@ QString WhyItemWidget::makeHtml(const std::vector<std::string>& bottomUpTxt,
 QString WhyItemWidget::makeHtml(const std::vector<std::string>& rawTxt) const
 { 
     QString s;
-    for(std::vector<std::string>::const_iterator it=rawTxt.begin(); it != rawTxt.end(); ++it)
+    for(const auto & it : rawTxt)
     {
-        QString line=QString::fromStdString(*it);
+        QString line=QString::fromStdString(it);
 
         //UiLog().dbg() << line;
 
@@ -234,8 +259,6 @@ void WhyItemWidget::updateState(const FlagSet<ChangeFlag>& flags)
         }
     }
 
-    Q_ASSERT(!flags.isSet(SelectedChanged));
-
 }
 
 void WhyItemWidget::anchorClicked(const QUrl& link)
