@@ -16,9 +16,7 @@
 #include "UiLog.hpp"
 
 LogModel::LogModel(QObject *parent) :
-          QAbstractItemModel(parent),
-          filterPeriod_(false), periodStart_(0), periodEnd_(0),
-          highlightPeriod_(false), highlightStart_(0), highlightEnd_(0)
+          QAbstractItemModel(parent)
 {
 	IconProvider::add(":/viewer/log_error.svg","log_error");
 	IconProvider::add(":/viewer/log_info.svg","log_info");
@@ -26,8 +24,7 @@ LogModel::LogModel(QObject *parent) :
 }
 
 LogModel::~LogModel()
-{
-}
+= default;
 
 void LogModel::loadFromFile(const std::string& fileName,size_t startPos)
 {
@@ -101,9 +98,9 @@ void LogModel::appendData(const std::vector<std::string>& data)
 {
 	int num=0;
 
-    for(std::vector<std::string>::const_iterator it=data.begin(); it != data.end(); ++it)
+    for(const auto & it : data)
 	{
-		QString s=QString::fromStdString(*it);
+		QString s=QString::fromStdString(it);
 		if(!s.simplified().isEmpty())
 		{
 			num++;
@@ -340,7 +337,7 @@ QModelIndex LogModel::index( int row, int column, const QModelIndex & parent ) c
 {
 	if(!hasData() || row < 0 || column < 0)
 	{
-		return QModelIndex();
+		return {};
 	}
 
 	//When parent is the root this index refers to a node or server
@@ -349,13 +346,13 @@ QModelIndex LogModel::index( int row, int column, const QModelIndex & parent ) c
 		return createIndex(row,column);
 	}
 
-	return QModelIndex();
+	return {};
 
 }
 
 QModelIndex LogModel::parent(const QModelIndex &child) const
 {
-	return QModelIndex();
+	return {};
 }
 
 QModelIndex LogModel::lastIndex() const
@@ -424,7 +421,7 @@ QModelIndex LogModel::highlightPeriodIndex() const
     {
         return index(realRow(highlightStart_),0);
     }
-    return QModelIndex();
+    return {};
 }
 
 //========================================================
