@@ -1,4 +1,17 @@
 #!/bin/ksh
+#/////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
+# Name        :
+# Author      : Avi
+# Revision    : $Revision: #5 $ 
+#
+# Copyright 2009-2020 ECMWF.
+# This software is licensed under the terms of the Apache Licence version 2.0 
+# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
+# In applying this licence, ECMWF does not waive the privileges and immunities 
+# granted to it by virtue of its status as an intergovernmental organisation 
+# nor does it submit to any jurisdiction. 
+#/////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
+
 # ==================================================================
 # Error handling
 set -e # stop the shell on first error
@@ -23,13 +36,14 @@ show_error_and_exit() {
    echo "   test_safe      - only run deterministic tests"
    echo "   ctest          - all ctest -R <test> -V"
    echo "   clang          - build with clang compiler"
+   echo "   intel          - build with intel compiler"
    echo "   clang_tidy     - create compilation database for clang_tdiy and then call run-clang-tidy.py"
+   echo "   iwyu           - INCLUDE what you use"
    echo "   tsan           - is short for clang thread sanitiser"
    echo "   asan           - is short for address sanitiser"
    echo "   msan           - is short for memory sanitiser"
    echo "   no_gui         - Don't build the gui"
-   echo "   ssl            - build using openssl"
-   echo "   secure_user    - enable password for client server"
+   echo "   no_ssl         - build without using openssl"
    echo "   log            - enable debug output"
    echo "   package_source - produces ecFlow-<version>-Source.tar.gz file, for users"
    echo "                    copies the tar file to $SCRATCH"
@@ -50,6 +64,7 @@ test_arg=
 test_safe_arg=
 clang_arg=
 clang_tidy_arg=
+clang_tidy_args=
 intel_arg=
 tsan_arg=
 mode_arg=release
@@ -57,11 +72,12 @@ verbose_arg=
 ctest_arg=
 clean_arg=
 no_gui_arg=
-ssl_arg=
-secure_user_arg=
+no_ssl_arg=
 log_arg=
 asan_arg=
 msan_arg=
+ubsan_arg=
+iwyu_arg=
 while [[ "$#" != 0 ]] ; do   
    if [[ "$1" = debug || "$1" = release ]] ; then
       mode_arg=$1
@@ -81,19 +97,27 @@ while [[ "$#" != 0 ]] ; do
          shift
       done
       break
-   elif [[ "$1" = no_gui ]] ;      then no_gui_arg=$1 ;
-   elif [[ "$1" = ssl ]]   ;       then ssl_arg=$1 ;
+   elif [[ "$1" = clang_tidy ]] ; then 
+      clang_tidy_arg=$1 ;
+      shift
+      while [[ "$#" != 0 ]] ; do
+         clang_tidy_args="$clang_tidy_args $1"
+         shift
+      done
+      break      
+   elif [[ "$1" = iwyu ]] ;    then iwyu_arg=$1 ;
+   elif [[ "$1" = no_gui ]] ;  then no_gui_arg=$1 ;
+   elif [[ "$1" = no_ssl ]] ;  then no_ssl_arg=$1 ;
    elif [[ "$1" = sys_install ]] ; then sys_install=$1 ;
-   elif [[ "$1" = secure_user ]] ; then secure_user_arg=$1 ;
    elif [[ "$1" = ecbuild ]] ; then ecbuild_arg=$1 ;
    elif [[ "$1" = log ]]   ; then log_arg=$1 ;
    elif [[ "$1" = clang ]] ; then clang_arg=$1 ;
-   elif [[ "$1" = clang_tidy ]] ; then clang_tidy_arg=$1 ;
    elif [[ "$1" = intel ]] ; then intel_arg=$1 ;
    elif [[ "$1" = clean ]] ; then clean_arg=$1 ;
    elif [[ "$1" = tsan ]]   ; then tsan_arg=$1 ;
    elif [[ "$1" = asan ]]  ; then asan_arg=$1 ;
    elif [[ "$1" = msan ]]  ; then msan_arg=$1 ;
+   elif [[ "$1" = ubsan ]]  ; then ubsan_arg=$1 ;
    elif [[ "$1" = package_source ]] ; then package_source_arg=$1 ;
    elif [[ "$1" = copy_tarball ]] ; then copy_tarball_arg=$1 ;
    elif [[ "$1" = test ]] ;  then test_arg=$1 ;
@@ -129,39 +153,63 @@ echo "ecbuild_arg=$ecbuild_arg"
 set -x # echo script lines as they are executed
 set -o pipefail # fail if last(rightmost) command exits with a non-zero status
 
+source_dir=$(pwd)
+
 # ==================== compiler flags ========================================
 # 
 # GNU 4.8+ -Wno-unused-local-typedefs   -> get round warning in boost headers
 # GNU 5.3  -Wno-unused-variable         -> get round warning in boost headers
 # GNU 6.1  -Wno-deprecated-declarations -> auto_ptr deprecated warning, mostly in boost headers  
+# GNU 7.3  -Wno-maybe-uninitialized     -> boost python warnings, specific to Pyext only
 # CLANG    -ftemplate-depth=512
 #
-CXX_FLAGS="-Wno-unused-local-typedefs -Wno-unused-variable -Wno-deprecated-declarations -Wno-address -Wno-maybe-uninitialized"
- 
+CXX_FLAGS="-Wno-unused-local-typedefs -Wno-unused-variable -Wno-deprecated-declarations -Wno-uninitialized"
+CXX_LINK_FLAGS=""
 
 # ==================== modules ================================================
 # To load module automatically requires Korn shell, system start scripts
-#
-module unload clang
-module load gnu/7.3.0
-cmake_extra_options="-DCMAKE_EXPORT_COMPILE_COMMANDS=ON"
-if [[ "$clang_arg" = clang || "$clang_tidy_arg" = clang_tidy ]] ; then
-    # ecflow fails to write boost ser' files with clang 6.0.1/7.0.1, but in debug all tests pass
-    # Had to apply fix: http://clang-developers.42468.n3.nabble.com/boost-serialization-crash-with-clang-5-0-0-td4058283.html
-    # - still have other crashes n serilisation see: ECFLOW-1328
-    module unload gnu
-    module load clang/7.0.1
+
+module swap gnu/7.3.0     
+
+cmake_extra_options=""
+if [[ "$clang_arg" = clang || "$clang_tidy_arg" = clang_tidy || "$iwyu_arg" = iwyu ]] ; then
+	module unload gnu
+	module load clang/9.0.1
+
+    # [-Wdeprecated-register] /usr/local/apps/python/2.7.12-01/include/python2.7/unicodeobject.h:534:5: warning: 'register' storage class specifier is deprecated and incompatible with C++17 [-Wdeprecated-register]
+    # [-Wmacro-redefined]     /usr/local/apps/python/2.7.12-01/include/python2.7/pyconfig.h:1215:9: warning: '_XOPEN_SOURCE' macro redefined
     CXX_FLAGS=""
-    CXX_FLAGS="$CXX_FLAGS -Wno-deprecated-declarations -Wno-deprecated-register -Wno-expansion-to-defined"
+    CXX_FLAGS="$CXX_FLAGS -Wno-deprecated-declarations -Wno-deprecated-register -Wno-expansion-to-defined -Wno-exceptions"
+
+    if [[ "$iwyu_arg" = iwyu ]] ; then
+        cmake_extra_options="-DCMAKE_CXX_INCLUDE_WHAT_YOU_USE=/usr/local/apps/iwyu/0.11/bin/include-what-you-use;-Xiwyu;--mapping_file=$source_dir/build_scripts/iwyu/boost.imp"
+    fi
+fi
+if [[ "$intel_arg" = intel ]] ; then
+    # fails because:
+    # /tmp/ma0/workspace/ecflow/ACore/src/cereal_optional_nvp.hpp(52): error: namespace "std" has no member "enable_if_t"
+    # > icc -v
+    # > icc version 19.0.4.243 (gcc version 4.8.0 compatibility)
+    # Require setting LD_RUN_PATH an CXX, to correctly pick the right std library
+    module unload eccodes
+    module unload gnu
+    module load intel/19.0.4
+    export LD_RUN_PATH="/usr/local/apps/gcc/8.3.0/lib64"
+    export CXX="icpc -cxxlib=/usr/local/apps/gcc/8.3.0"
+    export PATH="/usr/local/apps/gcc/8.3.0/bin:$PATH"
+    $CXX -v
+    #exit 1
+    CXX_FLAGS="-std=c++14"
+    #CXX_FLAGS="$CXX_FLAGS -Wno-deprecated-declarations -Wno-deprecated-register -Wno-expansion-to-defined -Wno-exceptions"
 fi
 
-# to use boost 1.71 or newer need at least cmake/3.15.3
-module load cmake/new    # need cmake 3.12.0 to build python3. Allow boost python 2 and 3 libs to be found  
-module load ecbuild/new
-module load boost/1.53.0     # uncomment to use local BOOST_ROOT  may need -DBOOST_ROOT=$BOOST_ROOT 
-module load python3/3.6.5-01
-# To build python3 when cmake < 3.12.0 use
-# -DPYTHON_EXECUTABLE=/usr/local/apps/python3/%PYTHON3_VERSION%/bin/python3 
+if [[ $ecbuild_arg != ecbuild ]] ; then
+    module load ecbuild/new   
+fi
+module load python
+module load python3/3.6.8-01
+module load cmake/new   # need cmake 3.12.0 to build python3. Allow boost python 2 and 3 libs to be found  
+
 
 # ==============================================================================================
 # sanitisers
@@ -170,16 +218,23 @@ if [[ "$tsan_arg" = tsan && "$asan_arg" = asan ]] ; then
 fi
 if [[ "$tsan_arg" = tsan ]] ; then
    CXX_FLAGS="$CXX_FLAGS -fsanitize=thread -fno-omit-frame-pointer"
-   cmake_extra_options="$cmake_extra_options -DCMAKE_EXE_LINKER_FLAGS='-fsanitize=thread'"  # LINK FLAGS
+   CXX_LINK_FLAGS="$CXX_LINK_FLAGS -fsanitize=thread"
 fi
 if [[ "$asan_arg" = asan ]] ; then
    CXX_FLAGS="$CXX_FLAGS -fsanitize=address -fno-omit-frame-pointer"
-   cmake_extra_options="$cmake_extra_options -DCMAKE_EXE_LINKER_FLAGS='-fsanitize=address'"  # LINK FLAGS
+   CXX_LINK_FLAGS="$CXX_LINK_FLAGS -fsanitize=address"
+   export ECF_TEST_SANITIZER_AS=1  # enable address sanitizer tests
+fi
+if [[ "$ubsan_arg" = ubsan ]] ; then
+   #CXX_FLAGS="$CXX_FLAGS -fsanitize=undefined -fsanitize-minimal-runtime -fno-omit-frame-pointer -fno-sanitize-recover=all" # exit after all errors minimal info
+   #CXX_FLAGS="$CXX_FLAGS -fsanitize=undefined -fno-omit-frame-pointer -fno-sanitize-recover=all"                           # exit on first error, max info
+   CXX_FLAGS="$CXX_FLAGS -fsanitize=undefined -fno-omit-frame-pointer"                                                     # report error, max info, no exit on error
+   CXX_LINK_FLAGS="$CXX_LINK_FLAGS -fsanitize=undefined"
+   export ECF_TEST_SANITIZER_UB=1  # enable undefined behaviour tests
 fi
 if [[ "$msan_arg" = msan ]] ; then
    CXX_FLAGS="$CXX_FLAGS -fsanitize=memory -fPIE -fno-omit-frame-pointer -fsanitize-memory-track-origins"
-   cmake_extra_options="$cmake_extra_options -DCMAKE_EXE_LINKER_FLAGS=-fsanitize=memory"  # LINK FLAGS
-   #LINK_FLAGS='-DCMAKE_EXE_LINKER_FLAGS="-fsanitize=memory -fPIE -pie"'
+   CXX_LINK_FLAGS="$CXX_LINK_FLAGS -fsanitize=memory" # -fPIE -pie"
 fi
 
 if [[ "$ARCH" = cray ]] ; then
@@ -187,21 +242,31 @@ if [[ "$ARCH" = cray ]] ; then
     # disable new UI, no QT on cray
     # Use the cray wrappers, these will add the correct flags.
     # Assumes we have CRAY_ADD_RPATH=yes
-    cmake_extra_options="$cmake_extra_options -DENABLE_UI=OFF -DCMAKE_C_COMPILER=cc -DCMAKE_CXX_COMPILER=CC -DCMAKE_Fortran_COMPILER=ftn"
+    cmake_extra_options="$cmake_extra_options -DENABLE_UI=OFF -DCMAKE_C_COMPILER=cc -DCMAKE_CXX_COMPILER=CC"
     
     if [[ $intel_arg = intel ]] ; then
         module swap PrgEnv-cray PrgEnv-intel
     else
     	module swap PrgEnv-cray PrgEnv-gnu
+    	module swap gcc/7.3.0     
     fi
+    module unload eccodes                 # Otherwise ecflow.so pulls in libeccodes.so
     module unload atp                     # must use for NON MPI code (ATP abnormal termination processing only works with cray MPI for ESM modes)
     module load craype-target-local_host  # must use for NON MPI code
     export CRAY_ADD_RPATH=yes
     export ECFLOW_CRAY_BATCH=1
 fi
 
- 
-########################################################################################
+#####################################################################################
+# Boost:
+#  By default it looks for environment variable BOOST_ROOT, if not it can specified on the command line. i.e
+#  -DBOOST_ROOT=/var/tmp/$USER/boost/boost_1_53_0
+#
+# This must be done after change of compiler/environment
+# versions of boost >= 1.67 now tag the python libs, i.e. libboost_python27-mt.a, libboost_python36-mt.so
+module load boost/1.71.0     # comment to use local BOOST_ROOT
+
+
 # ====================================================================================
 # default to Release  
 cmake_build_type=
@@ -221,22 +286,35 @@ fi
 
 # ====================================================================================
 # Use for local install
-release=$(cat VERSION.cmake | grep 'set( ECFLOW_RELEASE' | awk '{print $3}'| sed 's/["]//g')
-major=$(cat VERSION.cmake   | grep 'set( ECFLOW_MAJOR'   | awk '{print $3}'| sed 's/["]//g')
-minor=$(cat VERSION.cmake   | grep 'set( ECFLOW_MINOR'   | awk '{print $3}'| sed 's/["]//g')
-
-# =======================================================================================
-# Change directory
 #
-source_dir=$(pwd)
-workspace=$(pwd)/..
+cd $WK
+release=$(cat ACore/src/ecflow_version.h | grep 'ECFLOW_RELEASE' | awk '{print $3}'| sed 's/["]//g')
+major=$(cat ACore/src/ecflow_version.h   | grep 'ECFLOW_MAJOR'   | awk '{print $3}'| sed 's/["]//g')
+minor=$(cat ACore/src/ecflow_version.h   | grep 'ECFLOW_MINOR'   | awk '{print $3}'| sed 's/["]//g')
 
-if [[ $clean_arg = clean ]] ; then
-	rm -rf ../bdir4/$mode_arg/ecflow
+install_prefix=/var/tmp/$USER/install/cmake/ecflow/$release.$major.$minor
+if [[ $sys_install = sys_install ]] ; then
+   install_prefix=/usr/local/apps/ecflow/$release.$major.$minor
 fi
 
-mkdir -p ../bdir4/$mode_arg/ecflow
-cd ../bdir4/$mode_arg/ecflow
+# =======================================================================================
+# Change directory to build directory
+#
+workspace=$(pwd)/..
+
+build_dir_root=../bdir
+
+# separate build dir for clang tidy, so we can check fixes
+if [[ "$clang_tidy_arg" = clang_tidy ]] ; then
+    build_dir_root=../bdir_clang
+fi
+
+if [[ $clean_arg = clean ]] ; then
+	rm -rf $build_dir_root/$mode_arg/ecflow
+fi
+
+mkdir -p $build_dir_root/$mode_arg/ecflow
+cd $build_dir_root/$mode_arg/ecflow
 
 # =============================================================================================
 # ctest
@@ -250,21 +328,28 @@ if [[ $test_safe_arg = test_safe ]] ; then
 	   exit 0
 	fi
 fi
+
+
 if [[ "$ctest_arg" != "" ]] ; then
     if [[ "$asan_arg" = asan ]] ; then
-        if [[ $clang_arg != "clang" ]] ; then
-            # for python module we need to preload asan as it needs to be the very first library
-            # ==2971==ASan runtime does not come first in initial library list; 
-            #              you should either link runtime to your application or manually preload it with LD_PRELOAD.
-	        export LD_PRELOAD=/usr/local/apps/gcc/6.3.0/lib64/gcc/x86_64-suse-linux/6.3.0/libasan.so.3 
-	    fi
+    
+        # LD_PRELOAD is only required when creating a shared lib, than needs asan
+        # export LD_PRELOAD=/usr/local/apps/clang/7.0.1/lib64/libasan.so  
+        #if [[ $clang_arg != "clang" ]] ; then
+        #    # for python module we need to preload asan as it needs to be the very first library
+        #    # ==2971==ASan runtime does not come first in initial library list; 
+        #    #              you should either link runtime to your application or manually preload it with LD_PRELOAD.
+	    #    export LD_PRELOAD=/usr/local/apps/gcc/7.3.0/lib64/gcc/x86_64-suse-linux/7.3.0/libasan.so 
+	    #fi
 	    export ASAN_OPTIONS=suppressions=$WK/build_scripts/ecflow_asan.supp  
 	    export LSAN_OPTIONS=suppressions=$WK/build_scripts/ecflow_lsan.supp
 	    $ctest_arg  
     elif [[ "$tsan_arg" = tsan ]] ; then
+        export LD_PRELOAD=/usr/local/apps/clang/7.0.1/lib64/libtsan.so  
+
         if [[ $clang_arg != "clang" ]] ; then
             # LD_PRELOAD needed otherwise we get: .... cannot allocate memory in static TLS block
-            export LD_PRELOAD=/usr/local/apps/gcc/6.3.0/lib64/gcc/x86_64-suse-linux/6.3.0/libtsan.so
+            export LD_PRELOAD=/usr/local/apps/gcc/7.3.0/lib64/gcc/x86_64-suse-linux/7.3.0/libtsan.so
         fi
         export ASAN_OPTIONS=suppressions=$WK/build_scripts/ecflow_asan.supp  
         export LSAN_OPTIONS=suppressions=$WK/build_scripts/ecflow_lsan.supp
@@ -288,13 +373,8 @@ fi
 #
 
 ssl_options=
-if [[ $ssl_arg = ssl ]] ; then
-    ssl_options="-DENABLE_SSL=ON"
-fi
-
-secure_user_options=
-if [[ $secure_user_arg = secure_user ]] ; then
-    secure_user_options="-DENABLE_SECURE_USER=ON"
+if [[ $no_ssl_arg = no_ssl ]] ; then
+    ssl_options="-DENABLE_SSL=OFF"
 fi
 
 log_options=
@@ -304,7 +384,7 @@ fi
 
 gui_options=
 if [[ $no_gui_arg = no_gui ]] ; then
-    gui_options="-DENABLE_GUI=OFF -DENABLE_UI=OFF"
+    gui_options="-DENABLE_UI=OFF"
 fi
 
 test_options=
@@ -317,11 +397,6 @@ if [[ $package_source_arg = package_source ]] ; then
     gui_options=  
 fi
 
-install_prefix=/var/tmp/$USER/install/cmake/ecflow/$release.$major.$minor
-if [[ $sys_install = sys_install ]] ; then
-   install_prefix=/usr/local/apps/ecflow/$release.$major.$minor
-fi
-
 ecbuild=ecbuild
 if [[ $ecbuild_arg = ecbuild ]] ; then
    ecbuild=$workspace/ecbuild/bin/ecbuild
@@ -331,21 +406,24 @@ fi
 # cmake -C $workspace/ecflow/bamboo/macosx1010-flags.cmake $source_dir \
 #        -DCMAKE_MODULE_PATH=$workspace/ecbuild/cmake \
 #  .....
+#
+# DCMAKE_EXPORT_COMPILE_COMMANDS create compiler database. used by clang-tidy and others
 
 $ecbuild $source_dir \
             -DCMAKE_BUILD_TYPE=$cmake_build_type \
             -DCMAKE_INSTALL_PREFIX=$install_prefix  \
             -DENABLE_WARNINGS=ON \
             -DCMAKE_CXX_FLAGS="$CXX_FLAGS" \
+            -DCMAKE_EXE_LINKER_FLAGS="$CXX_LINK_FLAGS" \
             -DSITE_SPECIFIC_SERVER_SCRIPT="/home/ma/emos/bin/ecflow_site.sh" \
             ${cmake_extra_options} \
+            -DCMAKE_EXPORT_COMPILE_COMMANDS=ON \
             ${gui_options} \
             ${ssl_options} \
-            ${secure_user_options} \
             ${log_options} \
             ${test_options} # -DCMAKE_PREFIX_PATH="/tmp/$USER/opt/qt5/"
             
-            #-DPYTHON_EXECUTABLE=/usr/local/apps/python3/3.6.5-01/bin/python3 \
+            #-DPYTHON_EXECUTABLE=/usr/local/apps/python3/3.6.8-01/bin/python3 \
             #-DPYTHON_EXECUTABLE=/usr/local/apps/python/2.7.12-01/bin/python \
             #-DCMAKE_EXE_LINKER_FLAGS='-fsanitize=memory -fPIE -pie' 
             #-DENABLE_STATIC_BOOST_LIBS=ON \
@@ -354,7 +432,6 @@ $ecbuild $source_dir \
             #-DENABLE_PYTHON_PTR_REGISTER=ON  \
             #-DCMAKE_PYTHON_INSTALL_PREFIX=/var/tmp/$USER/install/cmake/ecflow/$release.$major.$minor   \
             #-DCMAKE_PREFIX_PATH="/usr/local/apps/qt/5.5.0/5.5/gcc_64/" \
-            #-DENABLE_GUI=ON       \  # ecflowview
             #-DENABLE_UI=ON        \  # ecflow_ui      
             #-DENABLE_ALL_TESTS=ON \
             #-DENABLE_SERVER=OFF   \
@@ -363,8 +440,10 @@ $ecbuild $source_dir \
 # export PATH=/tmp/$USER/opt/qt5/bin:$PATH
 
 if [[ "$clang_tidy_arg" = clang_tidy ]] ; then
-    python $WK/build_scripts/run-clang-tidy.py
+    python $WK/build_scripts/run-clang-tidy.py $clang_tidy_args
+    exit 0
 fi
+
 # =============================================================================================
 if [[ "$make_arg" != "" ]] ; then
 	$make_arg 
@@ -400,12 +479,6 @@ if [[ $package_source_arg = package_source ]] ; then
 	cp ecFlow-$release.$major.$minor-Source.tar.gz $SCRATCH/.
 fi
 
-# =========================================================================================
-# NOTES:
-# Boost:  
-#  By default it looks for environment variable BOOST_ROOT, if not it can specified on the command line. i.e
-#  -DBOOST_ROOT=/var/tmp/$USER/boost/boost_1_53_0
-
 # ============================================================================================
 # Python:
 # -DCMAKE_PYTHON_INSTALL_TYPE = [ local | setup ]
